#!/usr/bin/env python
# Copyright (c) 2017 Red Hat, Inc. All rights reserved. This copyrighted material
# is made available to anyone wishing to use, modify, copy, or
# redistribute it subject to the terms and conditions of the GNU General
# Public License v.2.
#
# This program is distributed in the hope that it will be useful, but WITHOUT ANY
# WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A
# PARTICULAR PURPOSE. See the GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
#
# Author: Shreyas Patil
#
from django.http import HttpResponseRedirect
from django.http import Http404

try: # django 2
    from django.utils.deprecation import MiddlewareMixin
    baseClass = MiddlewareMixin
except (ImportError, NameError) as e: # django 1
    baseClass = object
try: # django 2
    from django.urls import reverse, resolve
except (ImportError, NameError) as e: # django 1
    from django.core.urlresolvers import reverse, resolve

from rhcert.compatability import urlparse
from rhcert.tags import ServerConfiguration, Constants
from rhcert.environment import Environment
from wsgi.view import View, LoginRequiredException


class UrlRouterMiddleware(baseClass):

    # Return None in case of default to allow processing request
    # or else proceed to serve the request appropriately for default
    # Server Configuration

    def __initViews(self):
        self.management_not_allowed_views = ['sandbox', 'sandboxes', 'sandbox-results', 'sandbox-activity-results',
                                             'sandbox-activity-testing', 'create-sandbox', 'create-sandbox',
                                             'hoststatus', 'hostresults', 'hostinteractive', 'hostlog', 'remove-host'
                                             'resultsXML', 'results', 'testing', 'test', 'addTest', 'interactive',
                                             'run'
                                             ]

        self.testing_not_allowed_views = ['dialog', 'progress', 'details', 'cloud-details', 'related',
                                          'certifications', 'certification', 'testing', 'assign',
                                          'certification-specification', 'certification-specification-details',
                                          'openstack-specification', 'openstack-specification-details',
                                          'cloud-specification-details', 'hardware-publish', 'hardware-specification',
                                          'hardware-specification-details', 'hardware-specification-sample',
                                          'hardware-specification-related', 'hardware-specification-dialog'
                                          ]

    def process_view(self, request, view_func, view_args, view_kwargs):
        self.__initViews()
        current_server_config = Environment().getServerConfiguration()
        self.debug_print('rhcert running in {} configuration'.format(current_server_config))
        self.debug_print('Requested URL: {}'.format(request.get_full_path()))

        # Managment Configuration
        parsed_url = urlparse(request.get_full_path())
        if current_server_config == ServerConfiguration.management:
            return self.is_management_allowed_url(parsed_url.path)

        # Testing Configuration
        elif current_server_config == ServerConfiguration.testing:
            return self.is_testing_allowed_url(parsed_url.path)

        # Default Configuration
        else:
            self.debug_print('Continuing to requested route')
            return None

    def is_management_allowed_url(self, request_url_path):
        matched_view_name = resolve(request_url_path).url_name

        if matched_view_name in self.management_not_allowed_views:
            self.debug_print('Requested Url Not Found')

            # NOTE: Perform Django reverse lookup instead of actual URL to
            # avoid any failure in case url is updated
            # raise PageNotFound if the url is labelled not accessible
            raise Http404()

        else:
            return None

    def is_testing_allowed_url(self, request_url_path):
        matched_view_name = resolve(request_url_path).url_name
        if matched_view_name in self.testing_not_allowed_views:
            self.debug_print('Redirecting to sandboxes')

            # NOTE: Perform Django reverse lookup instead of actual URL to
            # avoid any failure in case url is updated
            return HttpResponseRedirect(reverse('sandboxes'))

        else:
            return None

    def debug_print(self, message, parameters=None, debug=Constants.off):
        if debug != Constants.off:
            print(message)
            if type(parameters) is not dict:
                print("%s" % parameters)
                return
            for (key, value) in parameters.items():
                if "password" in key or "token" in key:
                    print("%s: <-hidden-> (type: %s)" % (key, type(value)))
                    continue
                if not "data" == key:
                    print("%s: %s (type: %s)" % (key, value, type(value)))


class HttpRedirectMiddleware(baseClass):
    """
    Handles LoginRequiredException
    """
    def process_exception(self, request, exception):
        if isinstance(exception, LoginRequiredException):
            query = "?return-after=" + request.get_full_path()
            return HttpResponseRedirect(reverse("login", args=None) + query)
        return None

