#!/usr/bin/env python
# Copyright (c) 2014 Red Hat, Inc. All rights reserved. This copyrighted material 
# is made available to anyone wishing to use, modify, copy, or 
# redistribute it subject to the terms and conditions of the GNU General 
# Public License v.2.
# 
# This program is distributed in the hope that it will be useful, but WITHOUT ANY
# WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A
# PARTICULAR PURPOSE. See the GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
#
# Author: Greg Nichols <gnichols@redhat.com>
#

import os
from rhcert.compatability import XmlrpcServerProxy

from rhcert.model.resultsdocument import ResultsDocument
from rhcert.tags import Constants, Tags
from rhcert.model.task import Task
from rhcert.model.testparameters import TestParameters
from rhcert.host import Host

from run import Run


class AddTest(Run):
    def __init__(self, request, environment):
        Run.__init__(self, request, environment)
        self.testHostname = None
        self.certificationID = None
        self.certificationProgram = None
        self.resultsPath = None
        self.resultsDocument = None
        self.hostStatus = None
        self.hostInfo = None

    def validateParameters(self):
        errors = list()

        # try results path 
        if self.getParameter("rpath") and os.path.exists(self.getParameter("rpath")):
            self.resultsPath = self.getParameter("rpath")
            self.resultsDocument = ResultsDocument()
            self.resultsDocument.loadFiltered(self.fieldStorage["rpath"].value, os.path.join(self.environment.getServerDirectory(), "xsl/default.xsl"), self.getXSLTParameters())
            self.testHostname = self.resultsDocument.getOS(Tags.hostname)
            self.certificationID = self.resultsDocument.getCertificationID()
            self.certificationProgram = self.resultsDocument.getCertificationProgram()
            (self.hostStatus, self.hostInfo) = self.getHostStatus(self.testHostname)

        # otherwise, try host/task
        elif self.getParameter("host"):
            # use the hostname to get certID and Type
            self.testHost = Host(self.getParameter("host"))
            (self.hostStatus, self.hostInfo) = self.getHostStatus(self.testHost)
            # if the host is on a task, get the document from the task
            if self.hostStatus == "task":
                self.certificationID = self.hostInfo.getParameter(Task.certificationID)
                self.certificationProgram =  self.hostInfo.getParameter(Task.certificationProgram)
                if self.certificationID and self.certificationProgram:
                    (self.resultsDocument, self.resultsPath) = self.getStore().getLatestResultsDocument(self.testHost,
                                                        self.certificationProgram,
                                                        self.certificationID)
                else:
                    errors.append("Could not find certification type and id from host")

        if self.testHostname:
            self.testHostURL = 'http://%s:%s' % (self.testHostname, self.environment.getServerListenerPort())
            self.server = XmlrpcServerProxy(self.testHostURL, allow_none=True)
        else:
            errors.append("Error: could not determine test host name")

        return errors


    def addTest(self):
        errors = list()
        if not self.getParameter("test-path"):
            errors.append("Error: no test path")
        task = Task(Task.plan)
        task.setParameter(Task.test, self.getParameter("test-path"))
        task.setParameter("add", True)
        # defaults - not relevant yet
        task.setParameter("disable", False)
        task.setParameter("enable", False)
        task.setParameter("server", "")
        task.setParameter("device", "bogus")
        for param in self.fieldStorage.keys():
            if "param-" not in param:
                continue
            paramNumber = param.split('-')[-1]
            paramName = self.getParameter("param-" + paramNumber)
            paramValue = self.getParameter("param-value-" + paramNumber)
            if paramName and paramValue and not task.getParameter(paramName):
                task.setParameter(paramName, paramValue)

        try:
            self.result = self.server.run(task.toXML())
            if not self.result:
                errors.append("Error: Add Test Request Failed.")

        except Exception as exception:
            errors.append("Error: call to host %s failed:\n" % self.testHostURL)
            errors.append("%s\n" % exception)
            errors.append("%s\n" % task.toXML())

        return errors


    def makeHTMLBody(self):

        if self.hostStatus=="error":
            self.body.write("<pre>%s</pre>" % self.hostInfo)
            return


        self.body.write("""<h1>%s Add Test:</h1>""" % (self.testHostname))
        self.body.write("""
        <hr/>
        <form method="post" action="addTest">
            <div class="row">
                <div class="col-md-4">
                    <div class="form-group">
                        <label for="test-path">Test Path</label>
                        <div class="input-group">
                            <input class="form-control" type="text" name="test-path" id="test-path"/>
                        </div>""")
        for i in range(3):
            self.body.write("""
                        <label for="para-%s">Parameter</label>
                        <div class="input-group">
                            <input class="form-control" type="text" name="para-%s" id="para-%s"/>
                            <input class="form-control" type="text" name="para-value-%s" id="para-value-%s"/>
                        </div>""" % (i, i, i, i, i))
        self.body.write(""" <span class="input-group-btn">
                                    <input type="submit" value="Submit" class="btn btn-primary">
                                </span>""")
        self.body.write("""
                    </div>
                </div>
            </div>""")
        self.renderHiddenFormParameters()
        self.body.write("""</form>""")
        return

    def renderHiddenFormParameters(self):
        if self.resultsPath:
            self.body.write("""<input type="hidden" name="rpath" value="%s">\n""" % self.resultsPath)
        if self.testHostname:
            self.body.write("""<input type="hidden" name="host" value="%s">\n"""% self.testHostname)
        if self.certificationID:
            self.body.write("""<input type="hidden" name="certificationID" value="%s">\n"""% self.certificationID)
        if self.certificationProgram:
            self.body.write("""<input type="hidden" name="certificationProgram" value="%s">\n"""% self.CertificationProgram)

    def redirect(self):
        self.responseStatus = "301 Redirect"
        self.responseLocation = (self.path + "?host=" +  self.testHostname)
        responseHeaders = [('Content-Type', self.responseContent),
                       ('Content-Length', str(len(self.body.getvalue())))]
        if self.responseLocation:
            responseHeaders.append(('Location', str(self.responseLocation)))

        return (self.responseStatus, responseHeaders, self.body.getvalue())

    def redirectToCertification(self):
        self.responseStatus = "301 Redirect"
        self.responseLocation = "%s?type=%s&id=%s" % ("/certification", self.certificationProgram, self.certificationID)
        responseHeaders = [('Content-Type', self.responseContent),
                       ('Content-Length', str(len(self.body.getvalue())))]
        if self.responseLocation:
            responseHeaders.append(('Location', str(self.responseLocation)))

        return (self.responseStatus, responseHeaders, self.body.getvalue())

    def serve(self):

        errors = self.validateParameters()
        if errors:
            return self.serveErrorPage(errors)

        if "test-path" in self.fieldStorage:
            errors = self.addTest()
            if errors:
                return self.serveErrorPage(errors)
            # otherwise - redirect back to certification
            return self.redirectToCertification()

        # otherwise
        self.makeHTMLHeader()
        self.makeHTMLBody()
        self.body.write(self.makeHTMLFooter())
        status = '200 OK'
        responseHeaders = [('Content-Type', 'text/html'),
                   ('Content-Length', str(len(self.body.getvalue())))]

        return (status, responseHeaders, self.body.getvalue())

