# Copyright (c) 2017 Red Hat, Inc. All rights reserved. This copyrighted material
# is made available to anyone wishing to use, modify, copy, or
# redistribute it subject to the terms and conditions of the GNU General
# Public License v.2.
#
# This program is distributed in the hope that it will be useful, but WITHOUT ANY
# WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A
# PARTICULAR PURPOSE. See the GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
#
# Author: Greg Nichols
import json

from django.contrib import messages

class Alert(object):
    """ Alerts are simple notifications """
    debug = "debug"
    info = "info"
    success = "success"
    warning = "warning"
    danger = "danger"
    level_tags = {
                  debug: messages.DEBUG,
                  info: messages.INFO,
                  success: messages.SUCCESS,
                  warning: messages.WARNING,
                  danger: messages.ERROR,
                 }
    extra_tags = {
                  debug: "pficon-ok",
                  info: "pficon-ok",
                  success: "pficon-ok",
                  warning: "pficon-warning-triangle-o",
                  danger: "pficon-error-circle-o"
                 }

    def __init__(self, level, message, request=None, dismissable=True):
        self.level = level
        self.messageslist = [message]
        self.correction = None
        self.request = request
        if self.request:
            self.setDjangoMessage()
        self.dismissable = dismissable

    def __str__(self):
        return ("%s: %s" % (self.level, "; ".join(self.messageslist)))

    def serialize(self):
        return {
            "level": self.level,
            "correction": self.correction,
            "dismissable": self.dismissable,
            "messages": self.messageslist,
        }

    def getLevel(self):
        return self.level

    def getIcon(self):
        return Alert.extra_tags[self.level]

    def getMessage(self):
        try:
            return self.messageslist[0]
        except:
            return ""

    def getMessages(self):
        return self.messageslist

    def appendMessage(self, message):
        self.messageslist.append(message)
        if self.request:
            self.setDjangoMessage()

    @staticmethod
    def clearMessages(request):
        for each_msg in messages.get_messages(request):
            # Iterating over each message stored in the message store,
            # marks the message to get cleared while processing the response
            pass

    def setDjangoMessage(self):
        for each_msg in self.messageslist:
            messages.add_message(self.request, Alert.level_tags[self.level], each_msg, extra_tags=Alert.extra_tags[self.level])

    def setCorrection(self, urlName, parameterList, message):
        self.correction = {"url": urlName, "parameters": parameterList, "message": message}

    def getCorrection(self):
        return self.correction

    def isDismissable(self):
        return self.dismissable

    def json(self):
        data = {}
        data.update(self.__dict__)
        data["icon"] = self.getIcon()
        return json.dumps(data)

