# Copyright (c) 2019 Red Hat, Inc. All rights reserved. This copyrighted material
# is made available to anyone wishing to use, modify, copy, or
# redistribute it subject to the terms and conditions of the GNU General
# Public License v.2.
#
# This program is distributed in the hope that it will be useful, but WITHOUT ANY
# WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A
# PARTICULAR PURPOSE. See the GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
#
# Author: Greg Nichols <gnichols@redhat.com>
#
# API is a base class that is a View that returns the context (self.data) as
# a JSON response
import sys
import traceback
import json

from django.http import JsonResponse, HttpResponseNotAllowed
from django.views.decorators.cache import never_cache
from django.utils.cache import add_never_cache_headers

from rhcert.tags import Constants
from rhcert.environment import Environment
from wsgi.connection import Connection
from wsgi.alert import Alert

class NotAllowedException(Exception):
    pass

class API(object):
    POST = 'POST'
    GET = 'GET'
    PUT = 'PUT'
    PATCH = 'PATCH'
    DELETE = 'DELETE'

    def __init__(self):
        self.environment = Environment()
        self.loginRequired = True # default
        self.method = {
            API.POST: self.post,
            API.GET: self.get,
            API.PUT: self.put,
            API.PATCH: self.patch,
            API.DELETE: self.delete
        }
        self.debug = Constants.off
        self.__perRequestInit() # just to be sure

    def __perRequestInit(self):
        """ per request initialization goes here, as API objects are initialized and bound when the server starts """
        self.request = None
        self.connection = None
        self.json = {}
        self.data = {}
        self.meta = {"messages": []}
        self.alerts = {}

    @never_cache
    def serve(self, request):
        self.__perRequestInit()
        self.__debugPrint("call to %s" % self.__class__.__name__)
        self.__debugPrint("request type %s" % request.META.get("CONTENT_TYPE") )

        self.request = request
        self.connection = Connection(self.request)
        if self.isLoginRequired(self.request.method) and not self.getWorkflow().isLoggedIn():
            return self.errorResponse("Error: login required")

        if self.request.method not in self.supportedMethods():
            return self.__notAllowed()

        if self.request.body and request.META.get("CONTENT_TYPE") == 'application/json':
            try:
                self.json = json.loads(self.request.body)
            except Exception as error:
                return self.errorResponse(error)

        result = None
        try: # better catch exception here, as no one will see a traceback
            result = self.method[self.request.method]()
            if not result:
                return self.__notAllowed()
        except Exception as error:
            # log the traceback
            print(traceback.format_exc())
            return self.errorResponse("Error: RHCert API Exception: %s" % self.__tracebackSummary(), status=500)

        # otherwise
        if isinstance(result, (dict, list)):
            serialized = self.serializeContext(result)
            meta = self.serializeContext(self.meta)
            Alert.clearMessages(self.request)
            self.__debugPrint("%s returning Success: %s" % (self.__class__.__name__, {"data": serialized, "meta": meta}))
            return self.__noCache(JsonResponse({"data": serialized, "meta": meta}))
        # otherwise
        self.__debugPrint("%s returning Success: %s" % (self.__class__.__name__, result))
        return result

    def __tracebackSummary(self):
        exc_type, exc_value, exc_traceback = sys.exc_info()
        formatted_lines = traceback.format_exception(exc_type, exc_value,exc_traceback)
        return formatted_lines[-2] + ", " + formatted_lines[-1]

    def __debugPrint(self, message):
        if self.debug != Constants.off:
            print("API: %s" % message)

    def isLoginRequired(self, method):
        # default uses boolean for all methods
        return self.loginRequired

    def __noCache(self, response):
        add_never_cache_headers(response)
        return response

    def __notAllowed(self):
        return HttpResponseNotAllowed(self.supportedMethods())

    def errorResponse(self, errors, status=401):

        if isinstance(errors, list):
            errorsText = " \n".join(errors)
        else:
            errorsText = errors
        response = {
            "error" : {
                "status" : status,
                "message" : errorsText,
                "type": ""
            }
        }
        self.__debugPrint("returning Error: %s" % json.dumps(response))
        return JsonResponse(response, status=status)

    def append(self, alert):
        assert(isinstance(alert, Alert))
        if not self.alerts.get(alert.getMessage()):
            self.alerts[alert.getMessage()] = alert
            self.meta["messages"].append(alert)

    def serializeContext(self, data):
        # if the node is json serializable, we're done
        try:
            json.dumps(data)
            return data
        except:
            pass

        # traverse dictionaries and lists
        if isinstance(data, dict):
            for (key, value) in data.items():
                data[key] = self.serializeContext(value)
            return data
        if isinstance(data, list):
            serialized = []
            for value in data:
                serialized.append(self.serializeContext(value))
            return serialized

        # otherwise assume it's an object, and call its serialize method
        return self.serializeContext(data.serialize())

    def getConnection(self):
        if not self.connection:
            self.connection = Connection(self.request)
        return self.connection

    def getWorkflow(self):
        return self.getConnection().getCertificationWorkflow()
    def getVendorProductWorkflow(self):
        return self.getConnection().getVendorProductWorkflow()
    def getErrors(self):
        return self.getWorkflow().getErrors() + self.getVendorProductWorkflow().getErrors()

    def getParameter(self, name):
        for requestDictionary in [self.request.GET, self.request.POST, self.request.FILES]:
            value = requestDictionary.get(name)
            if value:
                return value
        return None

    def getStore(self):
        return self.getConnection().getStore()

    # abstract interface
    def supportedMethods(self): return []
    def post(self):   {}
    def get(self):    {}
    def put(self):    {}
    def patch(self):  {}
    def delete(self): {}
