# Copyright (c) 2019 Red Hat, Inc. All rights reserved. This copyrighted material
# is made available to anyone wishing to use, modify, copy, or
# redistribute it subject to the terms and conditions of the GNU General
# Public License v.2.
#
# This program is distributed in the hope that it will be useful, but WITHOUT ANY
# WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A
# PARTICULAR PURPOSE. See the GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
#
# Author: Greg Nichols
#
from django.http import HttpResponse

from rhcert.tags import Constants
from wsgi.api.api import API
from wsgi.alert import Alert


class Certifications(API):
    """ presents the certifications  api json """

    def __init__(self):
        API.__init__(self)
        self.loginRequired = True

    def supportedMethods(self): return [API.GET]

    def get(self):
        certifications = self.getConnection().getCertifications()
        self.compileView(certifications)

        return self.data

    def compileView(self, certifications):
        certificationsData = list()
        for certification in certifications:
            if certification.isSandbox():
                continue
            vendor = None
            vendorProduct = None
            (vendorID, productID) = certification.getPartnerAndProductID()
            if vendorID and productID:
                (vendor, vendorProduct) = self.getConnection().getVendorProducts().getVendorProductByID(vendorID, productID)
            if not (vendor and vendorProduct):
                self.append(Alert(Alert.warning,
                    "Warning: could not access vendor product information for certification %s" % certification.getCertificationID()))
                continue
            if not certification.getProductCertification():
                self.append(Alert(Alert.warning,
                    "Warning: could not determine Red Hat product for certification %s" % certification.getCertificationID()))
                continue
            certificationDict = certification.getCertificationParameters().getAll()
            certificationDict.update({
                        "document": certification,
                        "product": certification.getProductCertification(),
                        "program": certification.getCertificationProgram(),
                        "id": certification.getCertificationID(),
                        "cert_id": certification.getCertificationID(), # just in case: see CERTPX-5812
                        "vendorObject": vendor,
                        "vendorProduct": vendorProduct
                        })
            certificationDict.update(self.getWorkflowParameters(certification))
            if not certificationDict.get("category") and certificationDict.get("component"):
                certificationDict["category"] = certificationDict.get("component")
                try:
                    certificationDict["category"] = certificationDict.get("component").split('-')[1]
                except:
                    pass
            certificationsData.append(certificationDict)

        # sort most recent updates first
        self.data["certifications"] = sorted(certificationsData,
            key = lambda certificationDict: certificationDict['document'].getUpdateTime(),
            reverse=True)

    def getWorkflowParameters(self, certification):
        workflowParameters = {"workflowID": certification.getCertificationID(),
                              "workflow": certification.getCertificationState().capitalize()}

        if certification.getCertificationState() != Constants.local:
            workflowParameters["workflowID"] = certification.getBugID()
        if certification.getCertificationState() == Constants.bz:
            workflowParameters["workflow"] = "Bugzilla"
            workflowParameters["workflowID"] = certification.getBugID()
        elif certification.getCertificationState() == Constants.cwe:
            workflowParameters["workflow"] = "Catalog"
            workflowParameters["workflowID"] = certification.getBugID()
            workflowParameters["ci"] = certification.isSandbox()
            if certification.getNodeID():
                workflowParameters["workflowID"] = certification.getNodeID()
        return workflowParameters
