# Copyright (c) 2014 Red Hat, Inc. All rights reserved. This copyrighted material
# is made available to anyone wishing to use, modify, copy, or
# redistribute it subject to the terms and conditions of the GNU General
# Public License v.2.
#
# This program is distributed in the hope that it will be useful, but WITHOUT ANY
# WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A
# PARTICULAR PURPOSE. See the GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
#
# Author: Greg Nichols
#
import time
import pprint
import json
import sys

from django.http import HttpResponse
from django.utils.translation.trans_real import catalog

from rhcert.compatability import StringIO
from rhcert.tags import Constants, Sync, Programs, ServerConfiguration

from wsgi.api.api import API
from wsgi.alert import Alert

class CWESync(API):
    """ updates rhcert with CWE products and certifications """

    def __init__(self):
        API.__init__(self)
        self.catalogVendorProducts = None
        self.debugLog = None # example: sys.stdout
        self.serverSettings = None
        self.vendorProducts = None
        self.debug = Constants.high

    def supportedMethods(self): return [API.GET]

    def get(self):
        self.serverSettings = self.getConnection().getServerSettings()
        self.data["message"] = ""

        # only run in default config!
        if self.environment.getServerConfiguration() != ServerConfiguration.default:
            # no-op
            self.log("No sync needed: not a default confguration")
            return self.data

        # already syncing in background?
        if self.serverSettings.getSyncStatus() == Sync.running:
            self.data["status"] = Sync.running
            if self.debug:
                self.log("Sync response: %s %s" % (self.data["status"], self.data["message"]))
            return self.data

        # otherwise - check login
        if not self.getWorkflow().isLoggedIn():
            # 401 - unauthorized
            return HttpResponse("Unauthorized. Please log in.", status=401)

        # otherwise sync!
        newCertifications = self.__doSync()
        self.data["status"] = Sync.idle
        if self.getErrors():
            if newCertifications > 0:
                self.data["message"] = Alert(Alert.warning, "Synchronization failed (Updated %s Certifications): %s" %
                                             (newCertifications, self.getErrors()))
            else:
                self.data["message"] = Alert(Alert.danger, "Errors during synchronization: %s" % self.getErrors())
        elif newCertifications > 0:
            self.data["message"] = Alert(Alert.info, "Synchronization complete: Updated %s Certifications" % newCertifications)
        else:
            self.data["message"] = Alert(Alert.info, "Synchronization complete: no updates")
        if self.debug:
            self.log("Sync response: %s %s" % (self.data["status"], self.data["message"]))
        return self.data

    def __doSync(self):
        Alert.clearMessages(self.request) # clear django-session alerts
        self.serverSettings.setSyncStatus(Sync.running)
        self.serverSettings.save()
        newCertifications = self.downloadProductsAndCertifications()
        self.serverSettings.setSyncTime(int(time.time()))
        self.serverSettings.setSyncStatus(Sync.idle)
        self.serverSettings.save()
        return newCertifications

    def downloadProductsAndCertifications(self):
        if not self.getWorkflow().isLoggedIn():
            return False

        # Update local cache with product and vendor information
        if not self.__updateProducts():
            return False

        # User account might be associated with a vendor that has no product created yet
        # Add the information of such vendors to local cache
        current_user_vendors = self.getWorkflow().getCurrentUserVendors()
        # ToDo: Once CERTPX-2342 is fixed, store the vendor portal id to cache as well
        if current_user_vendors:
            for vendor in current_user_vendors:
                if not self.vendorProducts.getVendorByID(vendor.get('vendor_id')):
                    local_vendor = self.vendorProducts.createVendor(vendor.get('vendor_name'),
                                                                    vendor.get('vendor_id'))
                    local_vendor.setWorkflow(Constants.cwe)
            self.vendorProducts.save()

        # Update the local cache with certification information
        return self.__updateCertifications()

    def log(self, message):
        if self.debugLog:
            if type(message) == dict:
                pprint.pprint(message, indent=4, width=1, stream=self.debugLog)
            else:
                self.debugLog.write(str(message))
                self.debugLog.write("\n")

    def __updateProducts(self):
        """
        {'bug_id': 1155199,
                            'certification_id': 30251,
                            'make': 'SYSTEM Z9 BUSINESS CLASS (Z9 BC)',
                            'product_id': 6142,
                            'product_name': 'dsadsad111zz',
                            'product_nid': 2710513,
                            'product_portal_id': 2710513,
                            'program': 'hardware',
                            'vendor': 'IBM',
                            'vendor_id': 5},

        """
        if self.debug != Constants.off:
            self.log("Sync Products start...")
        self.vendorProducts = self.getStore().getVendorProducts(reload=True)
        start = time.time()
        catalogProducts = self.getVendorProductWorkflow().list()
        apiCallTime = time.time()-start
        if self.getVendorProductWorkflow().getErrors():
            self.log("Error: Could not download vendor products:")
            for error in self.getVendorProductWorkflow().getErrors():
                self.log(error)
            if not catalogProducts:
                # Return False only when an error occurred and no data was returned
                return False

        if not catalogProducts:
            self.log("Warning: User has no vendor products")
            if self.debug != Constants.off:
                self.log ("Sync Products: api: %.1f sec." % (apiCallTime))
            return True

        start = time.time()
        self.getVendorProductWorkflow().updateVendorProducts(catalogProducts, self.vendorProducts)
        self.vendorProducts.save()
        processingTime = time.time()-start
        if self.debug != Constants.off:
            self.log ("Sync Products: api: %.1f sec. processing %.1f sec." % (apiCallTime, processingTime))
        return True


    def __updateCertifications(self):
        if self.debug != Constants.off:
            self.log("Sync Loading Product Index...")
        start = time.time()
        self.vendorProducts = self.getStore().getVendorProducts(reload=True)
        self.vendorProducts.index()
        processingTime = (time.time()-start)
        if self.debug != Constants.off:
            self.log("Sync Loading Product Index took %.1f sec." % processingTime)
        syncRange = self.serverSettings.getSyncRange()
        if self.debug != Constants.off:
            if syncRange == "all":
                self.log("Sync %s Certifications start..." % syncRange)
            else:
                self.log("Sync Certifications from the last %s days start..." % syncRange)
        newCertifications = 0
        start = time.time()
        certificationsData = self.getWorkflow().getCertifications(dateRange=syncRange)
        apiCallTime = (time.time()-start)
        if not certificationsData:
            if self.debug != Constants.off:
                self.log("Warning: no certifications from CWE")
            return newCertifications

        start = time.time()
        for certificationData in certificationsData:
            self.log("certification from CWE:")
            self.log(certificationData.get("cert_id"))
            if certificationData.get("cert_type") == "container":
                continue # ignore container certs
            catalogCertification = self.getWorkflow().makeCertification(certificationData)
            if not catalogCertification:
                continue
            # is this a new certification to be added?
            (_, existingCertification) = self.getConnection().getStore().getCertification(
                            catalogCertification.getCertificationProgram(),
                            catalogCertification.getCertificationID())
            if not existingCertification:
                newCertifications = newCertifications + 1
            catalogCertification.setCertificationState(Constants.cwe)
            self.__updateVendorAndProduct(certificationData, catalogCertification)
            catalogCertification = self.getStore().updateCertification(catalogCertification)

        processingTime = (time.time()-start)
        self.log("Sync Certifications: api: %.1f sec.  processing: %.1f sec." % (apiCallTime, processingTime))
        return newCertifications

    def __updateVendorAndProduct(self, certificationData, catalogCertification):
        productID = certificationData.get("vendor_product_id")
        vendorID = certificationData.get("vendor_id")
        productName = certificationData.get("vendor_product_name")

        (vendor, product) = self.vendorProducts.getVendorProductByID(vendorID, productID)
        if vendor and product:
            vendor = catalogCertification.setVendor(vendor)
            vendor.setProduct(product)
            return True

        # otherwise - errors
        if not vendor:
            self.log("Error: no such vendor ID %s" % vendorID)
        if not product:
            self.log("Error no such product ID %s name %s" % (productID, productName))
        return False
