# Copyright (c) 2019 Red Hat, Inc. All rights reserved. This copyrighted material
# is made available to anyone wishing to use, modify, copy, or
# redistribute it subject to the terms and conditions of the GNU General
# Public License v.2.
#
# This program is distributed in the hope that it will be useful, but WITHOUT ANY
# WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A
# PARTICULAR PURPOSE. See the GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
#
# Author: Greg Nichols
#
import os
import gzip
from StringIO import StringIO

from rhcert.tags import ServerConfiguration, Constants
from rhcert.environment import Environment
from rhcert.model.resultsdocument import ResultsDocument
from ..utils import checkGzippedFile

from wsgi.alert import Alert
from wsgi.api.api import API

class Results(API):
    """ presents the "/results/"  top-level UI page """

    def __init__(self):
        API.__init__(self)
        self.loginRequired = False

    def supportedMethods(self): return [API.GET, API.POST]

    # for the get we should check for management mode and return via the api call not the store.
    def get(self):
        testResults = list()
        results = self.getStore().getResultsDictionary()

        for (path, (host, date, time)) in results.items():
            print("Path: {}, Host: {}, Date: {}, Time: {}".format(path, host, date, time))
            try:
                resultsDocument = self.getStore().getResultsDocument(path)
            except Exception as error:
                self.append(Alert(Alert.danger, "Error: %s: %s" % (path, error)))
                continue

            if resultsDocument:
                (_, certificationDocument) = self.getStore().getCertification(resultsDocument.getCertificationProgram(), resultsDocument.getCertificationID())
            else:
                certificationDocument = None

            fileInfo = {
                "path": path,
                "filename": os.path.basename(path),
                "host": host,
                "date": date,
                "time": time,
                "document": resultsDocument,
                "certification": certificationDocument
            }
            # vendor product name string
            if certificationDocument:
                (vendor, product) = certificationDocument.getPartnerAndProduct()
                if vendor:
                    fileInfo["vendor"] = vendor
                    if product:
                        fileInfo["product"] = product

            testResults.append(fileInfo)
        self.data["results"] = sorted(testResults, key = lambda fileInfo: fileInfo['document'].getRunTime(),reverse=True)
        return self.data

    def post(self):
        """ POST uploads and processes a results file """

        # django request.FILES is a dictionary of django.core.files.UploadedFile
        uploadedFile = self.request.FILES.get('file')
        if not uploadedFile:
            return self.errorResponse("Error: no \"file\" in upload request")

        try:
            fileName = uploadedFile.name
            if "octet-stream" in uploadedFile.content_type:
                fileContents = uploadedFile.read().decode("base64")
            else:
                fileContents = uploadedFile.read()

            if not (self.request.POST.get("importAction") and self.request.POST.get("id")):
                (pathIsURL, path) = self.getStore().saveResultsFile(fileName, fileContents)
                self.data = {
                    "pathIsURL": pathIsURL,
                    "path": path
                }
                return self.data

            # otherwise - importAction is set
            path = self.getStore().saveResultsFileSimple(fileName, fileContents)

            if self.getStore().getErrors():
                return self.errorResponse(self.getStore().getErrors(), status=400)

            importer = ResultsImporter(self, path, self.request.POST.get("program"), self.request.POST.get("id"))
            if self.request.POST.get("importAction"):
                (pathIsURL, path) = importer.updateCertification()
            messages = None
            if self.request.POST.get("importAction") == 'submit':
                messages = importer.submit()

            self.data = {
                "pathIsURL": pathIsURL,
                "path": path,
                "messages": messages
            }
            return self.data

        except Exception as e:
            return self.errorResponse("Error: Could not save file %s  %s\n" %  (fileName, e))


class ResultsImporter(object):
    def __init__(self, apiObject, resultsPath, program, certificationID):
        self.api = apiObject
        self.program = program
        self.certificationID = certificationID
        self.resultsPath = resultsPath
        self.resultsDocument = ResultsDocument()
        self.resultsDocument.load(resultsPath)
        self.alerts = []

    def updateCertification(self):
        self.__moveFile()
        return (True, self.__makeResultsURL())

    def __fixFileProgramAndID(self):
        # we need to node id too
        certification = self.api.getConnection().getCertification(self.program, self.certificationID)
        if not certification:
            self.api.append(Alert(Alert.danger, "Error: could not access %s certification %s" % (self.program, self.certificationID)))
            return False

        self.resultsDocument.setCertificationProgram(self.program)
        self.resultsDocument.setCertificationID(self.certificationID)
        self.resultsDocument.setCertificationState(certification.getCertificationState())
        self.resultsDocument.setNodeID(certification.getNodeID())
        # if it's not the right format for the name - rename it.
        if self.resultsDocument.hasMultiHostResults():
            newFileName = self.resultsDocument.getMultiHostFileName()
        else:
            newFileName = self.resultsDocument.getVerboseFileName()
        (_, ext) = os.path.splitext(self.resultsDocument.getLoadedFilePath())
        if ext == ".gz":
            newFileName += ".gz"
        self.resultsPath = os.path.join(os.path.dirname(self.resultsDocument.getLoadedFilePath()), newFileName)
        self.resultsDocument.save(self.resultsPath)
        self.resultsDocument.load(self.resultsPath) # reload - resets the loaded file path info in the cert object
        return True

    def __makeResultsURL(self):
        return "/certification/%s/%s/%s/%s/assign" % (
            self.resultsDocument.getCertificationProgram(),
            self.resultsDocument.getCertificationID(),
            self.resultsDocument.getHostname(),
            self.resultsDocument.getPlanTime())

    def __moveFile(self):
        if not self.__fixFileProgramAndID():
            return False
        self.resultsPath = self.api.getStore().fixResultsFileLocation(self.resultsDocument)
        return True

    def submit(self):
        workflow = self.api.getConnection().getCertificationWorkflow()
        workflow.submitTestRun(self.resultsPath, self.resultsDocument)
        for error in workflow.getErrors():
            self.api.append(Alert(Alert.danger, error))
            return Alert(Alert.danger, "Error: could not submit results")

        self.resultsDocument.setSubmitTime(workflow.getCurrentUTCTime())
        self.resultsDocument.setState(Constants.closed)
        self.resultsDocument.save()
        return Alert(Alert.success, "Results file %s submitted for review" % (os.path.basename(self.resultsPath)), self.api.request)