# Copyright (c) 2019 Red Hat, Inc. All rights reserved. This copyrighted material
# is made available to anyone wishing to use, modify, copy, or
# redistribute it subject to the terms and conditions of the GNU General
# Public License v.2.
#
# This program is distributed in the hope that it will be useful, but WITHOUT ANY
# WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A
# PARTICULAR PURPOSE. See the GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
#
# Author: Greg Nichols
#
from django.views.decorators.cache import never_cache
from django.utils.cache import add_never_cache_headers

from wsgi.api.api import API

from rhcert.tags import ServerConfiguration

class Sandboxes(API):
    """ presents the certifications  top-level UI page """

    def __init__(self):
        API.__init__(self)
        self.catalogVendorProducts = None
        self.loginRequired = False

    @never_cache
    def serve(self, request, program=None, sandboxID=None):
        self.program = program
        self.sandboxID = sandboxID
        return API.serve(self, request)

    def supportedMethods(self): return [API.GET, API.DELETE]

    def get(self):
        certifications = []
        if self.program and self.sandboxID:
            (_, certification) = self.getStore().getCertification(self.program, self.sandboxID)
            if certification:
                certifications.append(certification)
            else:
                self.data["message"] = "NOT FOUND: program=%s, id=%s" % (self.program, self.sandboxID)
        if not certifications:
            certifications = self.getStore().getCertifications()
        sandboxes = list()
        for certification in certifications:
            if certification.isSandbox():
                sandboxes.append(certification)
        self.compileView(sandboxes)
        return self.data

    def compileView(self, certifications):
        sandboxes = list()
        for certification in certifications:
            vendor = None
            vendorProduct = None
            (vendorID, productID) = certification.getPartnerAndProductID()
            if vendorID:
                vendor = self.getStore().getVendorProducts().getVendorByID(vendorID)
                if vendor and productID:
                    vendorProduct = vendor.getProduct(productID)
            certificationDict = certification.getCertificationParameters().getAll()
            certificationDict.update({
                        "document": certification,
                        "product": certification.getProductCertification(),
                        "program": certification.getCertificationProgram(),
                        "id": certification.getCertificationID(),
                        "vendor": vendor,
                        "vendorProduct": vendorProduct
                        })

            sandboxes.append(certificationDict)

        # sort by program, then by name
        try:
            name = sandbox['vendorProduct'].getName()
        except:
            name = ""
        self.data["sandboxes"] = sorted(sandboxes, key = lambda sandbox: sandbox['program']+name)

    def delete(self):
        if not (self.program and self.sandboxID):
            return self.errorResponse("Error: missing program and/or sandbox ID")

        (_, certification) = self.getStore().getCertification(self.program, self.sandboxID)
        if not certification:
            return self.errorResponse("Error: no such sandbox for %s with ID %s" % (self.program, self.sandboxID))

        if not certification.isSandbox():
            return self.errorResponse("Error: Attempt to delete %s certification with %s" % (self.program, self.sandboxID))

        if not self.getStore().deleteSandbox(self.program, self.sandboxID):
            return self.errorResonse("Could not delete sandbox")

        return { "deleted": "success"}
