# Copyright (c) 2015 Red Hat, Inc. All rights reserved. This copyrighted material
# is made available to anyone wishing to use, modify, copy, or
# redistribute it subject to the terms and conditions of the GNU General
# Public License v.2.
#
# This program is distributed in the hope that it will be useful, but WITHOUT ANY
# WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A
# PARTICULAR PURPOSE. See the GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
#
# Author: Greg Nichols
#
from django.conf import settings

from rhcert.tags import Tags, ServerConfiguration
from rhcert.host import Host

from .certification import Certification


class Results(Certification):
    """ presents the results/testing UI page """

    def __init__(self):
        Certification.__init__(self)
        self.loginRequired = True if self.isManagementMode else False

    def serve(self, request, template, program, certID, host, time):
        self.host = host
        self.timeStamp = time
        return Certification.serve(self, request, template, program, certID)

    def initializeView(self, request):
        result = Certification.view(self, request)
        if not isinstance(result, dict):
            return result

        self.resultsDocument = self.resultsPath = None
        for (path, results) in self.getStore().getResultsDocumentsFor(self.certification, skipEmpty=False, host=Host(self.host)).items():
            if results.getPlanTime() == self.timeStamp:
                self.resultsDocument = results
                self.resultsPath = path
                break
        if not self.resultsDocument:
            self.appendError("Error: could not find results for %s with time stamp %s" % (self.host, self.timeStamp))
            return False

        self.testHost = Host(self.resultsDocument.getHostOS().getName(), ipAddress=self.resultsDocument.getHostOS().getIPAddress())
        return True

    def view(self, request):
        result = self.initializeView(request)
        if result:
            self.__compileResultsView()
        return self.data

    def __compileResultsView(self):
        self.data["results"] = {
                                "document": self.resultsDocument,
                                "path": self.resultsPath,
                                "host": self.testHost,
                                }
        # since there appears to be no way in current django to loop over a range
        self.data["results"]["runs"] = list()
        for runNumber in range(self.resultsDocument.getNumberOfTestRuns()):
            self.data["results"]["runs"].append("%s" % (runNumber+1))

        self.data["results"]["tests"] = list()
        for test in self.resultsDocument.getTests():
            testDict = {"test": test, "runs": dict()}
            testDict.update(test.getTestParameters().getAll())
            # test spec with parameters are "<test path> [--<parameter> <value>]*"
            testSpec = test.getPath()
            for (parameter, value) in test.getTestParameters().getAll().items():
                testSpec += " --%s %s" % (parameter, value)
            testDict["testSpec"] = testSpec
            # testXMLQuery
            testXMLQuery = "?test=" + test.getPath() + "&amp;rpath=" + self.resultsPath
            if testDict.get("device"):
                testXMLQuery += "&amp;device=" + testDict.get("device")
            if testDict.get("udi"):
                testXMLQuery += "&amp;udi=" + testDict.get("udi")
            testDict["testXMLQuery"] = testXMLQuery

            for runNumber in range(self.resultsDocument.getNumberOfTestRuns()):
                testDict["runs"][runNumber+1] = test.getRun(runNumber+1)
            self.data["results"]["tests"].append(testDict)

    def __getXSLTParameters(self):
        #defaults
        defaultQueryParameters = { "sheet": "default", "content":"xml", "rpath":self.environment.getResultsPath()}
        xsltParameters = { "devices":"\'hide\'", "device-classes": "\'hide\'", "rpath":"\'%s\'" % self.environment.getResultsPath(), "server-email":"\'%s\'" % self.environment.getServerEMail()}
        # add default parameters if they're not in the query string
        parameters = dict()
        for key in defaultQueryParameters.keys():
            if key not in parameters:
                parameters[key] = defaultQueryParameters[key]

        # overide default xslt parameters with those from the query string
        for key in self.request.GET.keys():
            if parameters.get(key):
                xsltParameters[key] = "\'%s\'" % self.request.GET.get(key)
        return xsltParameters 


