# Copyright (c) 2014 Red Hat, Inc. All rights reserved. This copyrighted material
# is made available to anyone wishing to use, modify, copy, or
# redistribute it subject to the terms and conditions of the GNU General
# Public License v.2.
#
# This program is distributed in the hope that it will be useful, but WITHOUT ANY
# WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A
# PARTICULAR PURPOSE. See the GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
#
# Author: Greg Nichols
#
import copy
import json

from django.shortcuts import render

from rhcert.tags import Tags, Constants, Programs
from ..alert import Alert
from ..view import View


class Certification(View):
    """ presents the certification UI page """

    def __init__(self):
        View.__init__(self)
        self.loginRequired = True

    def serve(self, request, template, program, certID):
        # check login here before accessing connection
        View.authenticate(self, request)
        self.certificationID = certID
        self.program = program
        self.template = template

        # if this is a call to this abstract view, redirect to default page (tab)
        if type(self) is Certification:
            return self.redirect("progress", queryDict=None, paramList=[self.program,  self.certificationID])

        self.certification = self.getConnection(request).getCertification(program, certID)
        if not self.certification:
            self.appendError("Requested certification %s for %s program not found" % (certID, self.program))
            return self.redirectOnError()

        if self.certification:
            (self.vendorID, self.productID) = self.certification.getPartnerAndProductID()
            (self.vendor, self.product) = self.certification.getPartnerAndProduct()

        result = self.view(request)
        if type(result) != dict:
            return result
        # otherwise
        return render(request, self.template, result)

    def view(self, request):
        View.view(self, request)
        if not self.certification:
            self.appendError("Requested certification for %s program not found" % (self.program))
            return self.redirectOnError()

        # otherwise - call from subpage
        self.data["vendorID"] = self.vendorID
        self.data["productID"] = self.productID
        if not self.vendor or not self.product:
            self.appendAlert(Alert(Alert.danger,
                                   "Error: could not find product information (Vendor ID %s, Product ID %s)"
                                   % (self.vendorID, self.productID)))
        else:
            self.data["productNID"] = self.product.getNodeID()
            self.data["product"] = self.product
            self.data["vendor"] = self.vendor

        self.data["certificationType"] = "certification"
        if self.program == Programs.hardware:
            self.data["activateHardwareCertTabs"] = True

        self.data["certificationTab"] = "progress"
        self.data["certificationID"] = self.certificationID
        self.data["certificationProgram"] = self.program
        self.data.update(self.getWorkflowParameters(self.certification))
        self.__getDetails()
        self.data["productCertification"] = self.certification.getProductCertification()

        self.data["os"] = dict()
        for parameter in [Tags.product, Tags.release]:
            self.data["os"][parameter] = self.certification.getOS(parameter)

        return self.data

    def __getDetails(self):
        details = self.certification.getCertificationParameters().getAll()
        self.data["certificationDetails"] = details
        if self.program == Programs.cloud:
            toDoTabs = []
            for attribute in ["image_name", "build", "subscription_model"]:
                if not details.get(attribute):
                    toDoTabs.extend(['certification', 'details'])
                    break
            for attribute in ["vendor_product_name",
                              "vendor_product_url",
                              "vendor_product_supported_regions",
                              "vendor_product_supported_languages",
                              "vendor_product_third_party_certifications"]:
                if not details.get(attribute):
                    toDoTabs.extend(['specification', 'specification-details'])
                    break
            self.data["toDoTabs"] = json.dumps(toDoTabs)
            return True

        elif self.program == Programs.hardware:
            toDoTabs = []
            for attribute in ["vendor_product_url", "vendor_product_support_url", "vendor_product_spec_url"]:
                if not details.get(attribute):
                    toDoTabs.extend(['specification', 'specification-details'])
                    break
            for attribute in ["shipping_to_rh", "shipping_info"]:
                if not details.get(attribute):
                    toDoTabs.extend(['specification', 'specification-sample'])
                    break
            if details.get('is_public') != '1' and not details.get('deadline'):
                toDoTabs.extend(['certification', 'publish'])
            self.data["toDoTabs"] = json.dumps(toDoTabs)
            return True

        elif self.program == Programs.openstack:
            toDoTabs = []
            for attribute in ["plugin_type", "component", "packing_format", "source_url", "blueprint_url",
                              "install_guide", "director_template", "director_instructions_text",
                              "director_instructions_url", "user_guide", "configuration", "hostname",
                              "app_on_rhel", "app_on_openstack", "ship_hardware"]:
                if attribute == "plugin_type":
                    if details.get(attribute) == "In Tree":
                        if not details.get("availability_date"):
                            toDoTabs.extend(['specification', 'specification-details'])
                            break
                    elif details.get(attribute) == "Out of Tree":
                        if not details.get("download_url"):
                            toDoTabs.extend(['specification', 'specification-details'])
                            break
                else:
                    if not details.get(attribute) and details.get(attribute) != 0:
                        toDoTabs.extend(['specification', 'specification-details'])
                        break

            for item in ['app_on_openstack', 'app_on_rhel', 'ship_hardware']:
                if details.get(item):
                    details[item] = int(details.get(item))

            if details.get("openstack_specific"):
                if not details.get("openstack_specific").get("networking_plugin_type") or\
                   not details.get("openstack_specific").get("is_ha"):
                        toDoTabs.extend(['specification', 'Neutron'])
            self.data["toDoTabs"] = json.dumps(toDoTabs)
            return True
        return True

    def updateCWETestPlan(self):
        cwePlanComponents = self.getCertificationWorkflow().getTestPlan(self.certification.getCertificationID())
        self.appendErrors(self.getCertificationWorkflow().getErrors())
        if self.getErrors():
            return False
        """
        Hardware Example:
        'cwePlanComponents': {'components': [{'bits': '64bits',
                                       'comp_name': 'NerdCore 2000 CPU',
                                       'id': 152343,
                                       'is_supplemental': 0,
                                       'mapped_test_id': 331641,
                                       'name': 'CORE',
                                       'status': 'Confirmed',
                                       'test_desc': '',
                                       'type': 'Integrated'},
                                      {'bits': '64bits',
                                       'comp_name': 'Ethernet Adapter',
                                       'id': 152342,
                                       'is_supplemental': 0,
                                       'name': '10GIGETHERNET',
                                       'status': 'Frozen',
                                       'test_desc': "It's very nice",
                                       'type': 'Integrated'}],
                       'status': 'Frozen'},

        Cloud, openstack Example:
            [{'name': 'OPENSCAP',
                'status': 'Need Review'},
            {'name': 'GENERAL', 'status':
                'Need Review'}],
           """
        if cwePlanComponents.get("status") and cwePlanComponents.get("status") != self.certification.getTestPlanStatus():
            self.certification.setTestPlanStatus(cwePlanComponents.get("status"))
            self.certification.save()

        if not cwePlanComponents.get("components"):
            return True

        # self.data["cwePlanComponents"] = cwePlanComponents

        newPlanComponents = list()
        changedPlanComponents = list()
        for cwePlanComponent in cwePlanComponents["components"]:
            # work-around for missing plan component ids
            componentID = cwePlanComponent.get("id")
            if not componentID:
                componentID = cwePlanComponent.get("name")
            planComponent = self.certification.getPlanComponent(str(componentID))
            if not planComponent:
                planComponent = self.certification.createPlanComponent(cwePlanComponent.get("name"), componentID)

                newPlanComponents.append(planComponent)
            elif planComponent.getStatus() != cwePlanComponent.get("status"):
                changedPlanComponents.append(planComponent)

            planComponent.setStatus(cwePlanComponent.get("status"))
            if cwePlanComponent.get("is_supplemental"):
                planComponent.setSupplimental(True)
            planComponent.setComponent(cwePlanComponent.get("comp_name"))
            planComponent.setDescription(cwePlanComponent.get("test_desc"))
            planComponent.setType(cwePlanComponent.get("type"))
            planComponent.setBits(cwePlanComponent.get("bits"))
            planComponent.setStatus(cwePlanComponent.get("status"))
            planComponent.setTestID(cwePlanComponent.get("mapped_test_id"))
            planComponent.setLeveragedTestID(cwePlanComponent.get("leverage_test_id"))
            if cwePlanComponent.get("original_cert_id"):
                planComponent.setLeveragedCertificationID(cwePlanComponent.get("original_cert_id"))
            elif cwePlanComponent.get("leverage_cert_id"):
                planComponent.setLeveragedCertificationID(cwePlanComponent.get("leverage_cert_id"))
            else:
                planComponent.setLeveragedCertificationID(None)

            # host roles for multihost?
            if cwePlanComponent.get("role"):
                host = planComponent.getRequiredOS()
                host.setRole(cwePlanComponent.get("role"))
                host.setID(cwePlanComponent.get("host_id"))

            # features for granularity
            if cwePlanComponent.get("features"):
                for feature in cwePlanComponent.get("features"):
                    planComponent.removeFeature(feature.get("id"))
                    planComponent.addFeature(feature.get("id"), feature.get("name"), feature.get("category"))

        self.certification.save()

        if newPlanComponents or changedPlanComponents:
            self.__updateResultsFromComponents(newPlanComponents, changedPlanComponents)
            if newPlanComponents:
                self.data["message"] = "Added %s test plan components." % len(newPlanComponents)
            if changedPlanComponents:
                self.data["message"] = "Updated %s test plan components." % len(changedPlanComponents)

        return True

    def __updateResultsFromComponents(self, newComponents, changedPlanComponents):
        for (_, resultsDocument) in self.getStore().getResultsDocumentsFor(self.certification).items():
            updated = False
            resultsDocument.reload() # get unfiltered copy for modification
            for newComponent in newComponents:
                if not resultsDocument.getPlanComponent(newComponent.getID()):
                    resultsDocument.copyPlanComponent(newComponent)
                    updated = True
            for changedComponent in changedPlanComponents:
                if not resultsDocument.getPlanComponent(changedComponent.getID()):
                    resultsDocument.copyPlanComponent(changedComponent)
                    updated = True
                else:
                    resultsDocument.replacePlanComponent(changedComponent)
                    updated = True
            if updated:
                resultsDocument.assignComponents()
                resultsDocument.save()

