# Copyright (c) 2017 Red Hat, Inc. All rights reserved. This copyrighted material
# is made available to anyone wishing to use, modify, copy, or
# redistribute it subject to the terms and conditions of the GNU General
# Public License v.2.
#
# This program is distributed in the hope that it will be useful, but WITHOUT ANY
# WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A
# PARTICULAR PURPOSE. See the GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
#
# Author: Greg Nichols
#
import os, mimetypes

from django.shortcuts import render
from django.http import StreamingHttpResponse
from wsgiref.util import FileWrapper

from rhcert.model.resultsdocument import ResultsDocument
from rhcert.tags import ServerConfiguration, Constants


from ..view import View
from ..alert import Alert


class DownloadResults(View):

    def __init__(self):
        View.__init__(self)
        self.loginRequired = True
        self.resultsDocument = None

    def serve(self, request, program, certificationID, resultsFileID):
        View.authenticate(self, request)

        self.program = program
        self.certificationID = certificationID
        self.resultsFileID = resultsFileID

        result = self.view(request)
        if type(result) != dict:
            return result
        return render(request, self.template, self.data)

    def view(self, request):
        View.view(self, request)
        self.__download()

        if self.environment.getServerConfiguration() == ServerConfiguration.management:
            return self.__downloadFileContents()

        # go to view the results
        if self.resultsDocument:
            url_param = [
                self.resultsDocument.getCertificationProgram(),
                self.resultsDocument.getCertificationID()
            ]
            if self.resultsDocument.isMultiHost():
                url_param.append(self.resultsDocument.getMultiHostRunTime())
            else:
                url_param.append(self.resultsDocument.getHostname())
                url_param.append(self.resultsDocument.getPlanTime())

            return self.redirect("assign", queryDict=None, paramList=url_param)

        # otherwise, go to the results list
        return self.redirect("results", queryDict=None,
                                     paramList=[self.program,  self.certificationID])


    def __download(self):
        download = self.getCertificationWorkflow().downloadSubmittedResultsFile(self.certificationID, self.resultsFileID)
        self.appendErrors(self.getCertificationWorkflow().getErrors())
        try:
            self.resultsTmpPath = os.path.join("/tmp", download.get("name"))
            resultsFile = open(self.resultsTmpPath, "w+")
            resultsFile.write(download.get("data").decode("base64"))
            resultsFile.close()
            self.resultsDocument = ResultsDocument()
            self.resultsDocument.load(self.resultsTmpPath)
        except Exception as error:
            self.appendError("Error: could not download submitted results file (cert %s id %s)" % (self.certificationID, self.resultsFileID))
            self.appendError(str(error))
            return False

        if self.environment.getServerConfiguration() == ServerConfiguration.management:
            # actually download it to the user's browser
            return True

        # download it to be viewed here
        # fix up state and submitted time?
        self.resultsDocument.setState(Constants.closed)
        if not self.resultsDocument.getSubmitTime():
            self.resultsDocument.setSubmitTime(self.getCurrentUTCTime())
        # fix up certification ID and program CERTPX-2753
        self.resultsDocument.setCertificationID(self.certificationID)
        self.resultsDocument.setCertificationProgram(self.program)
        return self.getStore().saveResultsDocument(self.resultsDocument)

    def __downloadFileContents(self):
        filename = os.path.basename(self.resultsTmpPath)
        (mimeType, encoding) = mimetypes.guess_type(filename)
        chunkSize = 8192
        response = StreamingHttpResponse(FileWrapper(open(self.resultsTmpPath), chunkSize), content_type=mimeType)
        response["Content-Disposition"] =  "attachment; filename=\"%s\"" % filename
        response['Content-Length'] = os.path.getsize(self.resultsTmpPath)

        return response


