# Copyright (c) 2017 Red Hat, Inc. All rights reserved. This copyrighted material
# is made available to anyone wishing to use, modify, copy, or
# redistribute it subject to the terms and conditions of the GNU General
# Public License v.2.
#
# This program is distributed in the hope that it will be useful, but WITHOUT ANY
# WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A
# PARTICULAR PURPOSE. See the GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
#
# Author: Greg Nichols
#
# Implement leverageing of component test results
import json
from django.shortcuts import render
from django.http import HttpResponse

from rhcert.tags import Programs
from rhcert.workflow.vendorProduct import VendorProduct as VendorProductWorkflow

from ..view import View
from ..alert import Alert


class LeverageDialog(View):
    save = "save"
    apply = "apply"
    get = "get"
    submit = "submit"

    def __init__(self):
        View.__init__(self)

    def serve(self, request, action, certificationID=None, componentID=None, testID=None, leverageType=None, leverageID=None):
        View.authenticate(self, request)
        self.action = action
        self.certificationID = certificationID
        self.componentID = componentID if componentID and componentID != 'null' else None
        self.testID = testID if testID and testID != 'null' else None
        self.leverageType = leverageType if leverageType and leverageType != 'null' else None
        self.leverageID = leverageID if leverageID and leverageID != 'null' else None
        self.template = "certification/leverage.html"
        result = self.view(request)
        if type(result) != dict:
            return result
        return render(request, self.template, self.data)

    def view(self, request):
        View.view(self, request)
        if self.confirming():
            return self.data

        if self.action == LeverageDialog.save:
            if not self.__compileLeverage():
                return self.data #error
            return self.__doSave()
        elif self.action == LeverageDialog.get:
            return self.__doGet()
        elif self.action == LeverageDialog.apply:
            if not self.__compileLeverage():
                return self.data #error 
            return self.__doApply()
        elif self.action == LeverageDialog.submit:
            return self.__doSubmit()

        # otherwise
        self.appendAlert(Alert(Alert.danger, "Error: could not determine leverage action.", self.request))
        return self.data

    def confirming(self):
        # disable confirmation dialog
        return False

        if self.getParameter("confirm"):
            return False
        return True

    def __compileLeverage(self):
        self.data["certificationID"] = self.certificationID
        self.data["componentID"] = self.componentID
        if self.testID:
            self.data["testID"] = self.testID
        self.data["action"] = self.action

        self.leverage = None
        if self.request.session.get("leverage"):
            self.leverage = Leverage()
            self.leverage.update(self.request.session.get("leverage"))
            self.data["leverage"] = self.leverage

        self.leverageCertification = None
        self.leverageComponent = None
        if self.action == LeverageDialog.save:
            (_, self.leverageCertification) = self.getStore().getCertification(Programs.hardware, self.certificationID)
            if self.leverageCertification:
                self.leverageComponent = self.leverageCertification.getPlanComponent(self.componentID)
        else: # apply
            if not self.leverage: 
                self.appendAlert(Alert(Alert.danger, "Error: no leverage information available", self.request))
                return False

            (_, self.leverageCertification) = self.getStore().getCertification(Programs.hardware, self.leverage.certificationID)
            if self.leverageCertification:
                self.leverageComponent = self.leverageCertification.getPlanComponent(self.leverage.componentID)
        self.data["leverageCertification"] = self.leverageCertification
        self.data["leverageComponent"] = self.leverageComponent
        return True

    def __doSave(self):
        if not self.getParameter("cancel"):
            if not self.componentID: # then it's a component cert leverage
                self.request.session["leverage"] = Leverage().set(self.certificationID)
                return HttpResponse("Copied Leverage Information from Component Certification %s" % (self.certificationID), content_type="text/html")
            # otherwise, leverage of a component test from a system cert
            self.request.session["leverage"] = Leverage().set(self.certificationID, self.componentID, self.leverageComponent.getName(), self.testID)
            return HttpResponse("Copied Leverage Information from System Certification %s test id %s" % (self.leverageComponent.getName(), self.testID), content_type="text/html")

    def __doGet(self):
        """ get leverage from session and return it in json form """
        return HttpResponse(json.dumps(self.request.session.get("leverage")), content_type="application/javascript")

    def __doApply(self):
        alert = None
        if not self.getParameter("cancel"):
            if not (self.leverageCertification):
                alert = Alert(Alert.warning, "Warning: no leverage information available", None)
            else:

                self.getCertificationWorkflow().applyComponentLeverage(self.leverageCertification, self.leverageComponent, self.certificationID, self.componentID)
                if self.getCertificationWorkflow().getErrors():
                    alert = Alert(Alert.danger, "", None)
                    alert.messageslist = self.getCertificationWorkflow().getErrors()
                else:
                    if self.leverageComponent:
                        alert = Alert(Alert.success, "Leverage of System Test successfully applied.", None)
                    else:
                        alert = Alert(Alert.success, "Leverage of Component Certification successfully applied.", None)

        if alert:
            return HttpResponse(alert.json(), content_type="application/javascript")

        return self.data

    def __doSubmit(self):
        alert = None
        if not self.getParameter("cancel"):
            if not self.leverageType and not self.leverageID:
                alert = Alert(Alert.warning, "Warning: no leverage information available", None)
            else:
                self.getCertificationWorkflow().submitLeverage(self.leverageType, self.leverageID, self.certificationID, self.componentID)
                if self.getCertificationWorkflow().getErrors():
                    alert = Alert(Alert.danger, "", None)
                    alert.messageslist = self.getCertificationWorkflow().getErrors()
                else:
                    if self.leverageType == "testid":
                        alert = Alert(Alert.success, "Leverage of System Test successfully applied.", None)
                    else:
                        alert = Alert(Alert.success, "Leverage of Component Certification successfully applied.", None)

        if alert:
            return HttpResponse(alert.json(), content_type="application/javascript")

        return self.data


class Leverage(object):
    def __init__(self):
        self.certificationID = None
        self.componentID = None
        self.testID = None
        self.testName = None

    def set(self, certificationID, componentID=None, testName=None, testID=None):
        self.certificationID = certificationID
        self.componentID = componentID
        self.testID = testID
        self.testName = testName
        return self.__dict__

    def update(self, params):
        for attribute in ["certificationID", "componentID", "testName", "testID"]:
            if params.get(attribute):
                self.__dict__[attribute] = params.get(attribute)

