# Copyright (c) 2014 Red Hat, Inc. All rights reserved. This copyrighted material
# is made available to anyone wishing to use, modify, copy, or
# redistribute it subject to the terms and conditions of the GNU General
# Public License v.2.
#
# This program is distributed in the hope that it will be useful, but WITHOUT ANY
# WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A
# PARTICULAR PURPOSE. See the GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
#
# Author: Greg Nichols
#
from rhcert.tags import Constants, TestResult, ServerConfiguration
from rhcert.model.resultsdocument import ResultsDocument

from .certification import Certification
from ..alert import Alert
from .leverage import Leverage

class Progress(Certification):
    """ presents the tests UI page, aggregating test status across results for the certification"""

    def __init__(self):
        Certification.__init__(self)

    def view(self, request):
        Certification.view(self, request)

        self.processForm()

        self.__compileTestsView()

        self.__compileLeverage()

        return self.data

    def __compileTestsView(self):
        self.data["certification"] = self.certification
        self.data["tests"] = dict()

        progressColumns = 9
        if not self.certification.isMultiHost():
            progressColumns = progressColumns - 1

        if self.environment.getServerConfiguration() == ServerConfiguration.management:
            progressColumns = progressColumns - 2
        self.data["progressColumns"] = str(progressColumns)

        """ get the overall test plan status """
        self.data["planComponents"] = dict()
        statusMessages = {
            "Specification": {
                "summary": "Testplan Unavailable",
                "message": "The specifications for the product still need to be reviewed. Please ensure the product specification details have been provided. You should receive an notification when the production specification review is complete.",
                "dismissable": False,
                "level": Alert.warning
            },
            "Planning": {
                "summary": "Testplan Not Ready",
                "message": "The specification is reviewed and the testplan is being prepared. You should receive an notification when the testplan is available. Please check back later.",
                "dismissable": False,
                "level": Alert.warning
            },
            "Melted": {
                "summary": "Testplan Not Ready",
                "message": "The specification is reviewed and the testplan is being prepared. You should receive an notification when the testplan is available. Please check back later.",
                "dismissable": False,
                "level": Alert.warning
            },
            "Frozen": {
                "summary": "Ready for Certification Testing",
                "message": "The testplan has been prepared based upon the solution specifications and the associated certification policy.",
                "dismissable": True,
                "level": Alert.info
            },
            "Confirmed": {
                "summary": "Testplan is Complete",
                "message": "Each of the items in the testplan has been credited.",
                "dismissable": True,
                "level": Alert.success
            },
            "NotRequired": {
                "summary": "Testplan is Complete ",
                "message": "It has been determined that no additional testing is required in this certification.",
                "dismissable": False,
                "level": Alert.info
            },
        }
        self.data["planStatus"] = self.certification.getTestPlanStatus()
        if self.certification.getTestPlanStatus() in statusMessages:
            statusMessage = statusMessages.get(self.certification.getTestPlanStatus())
            summaryAndMessage = "<strong>" + statusMessage.get("summary") + "</strong> " + statusMessage.get("message")
            alert = Alert(statusMessage.get("level"), message=summaryAndMessage, request=None, dismissable=statusMessage.get("dismissable") )
            self.data['planStatusAlert'] = alert
        else:
            self.data['planStatusMessage'] = "Test Plan Status: %s" % self.certification.getTestPlanStatus()
        for component in self.certification.getPlanComponents():
            self.data["planComponents"][component.getID()] = {"planComponent": component, "test": None}

        """ find the best run for each plan component by going over all the results documents """
        if self.certification.isMultiHost():
            resultsDocuments = self.getStore().getMultiHostResultsDocuments(self.program, self.certificationID)
        else:
            resultsDocuments = self.getStore().getResultsDocumentsFor(self.certification)
        for rpath in sorted(resultsDocuments.keys(), reverse=True):
            resultsDocument = resultsDocuments.get(rpath)
            for test in resultsDocument.getTests():
                if not self.data["tests"].get(test.getPath()):
                    self.data["tests"][test.getPath()] = dict()
                bestRun = test.bestRun()
                if bestRun:
                    if self.__betterTestRun(self.data["tests"][test.getPath()], bestRun, resultsDocument):
                        self.data["tests"][test.getPath()]["bestRun"] = bestRun
                        self.data["tests"][test.getPath()]["results"] = resultsDocument

                    if test.getPlanComponentID():
                        if not self.data["planComponents"].get(test.getPlanComponentID()):
                            self.data["planComponents"][test.getPlanComponentID()] = dict()
                            self.data["planComponents"][test.getPlanComponentID()]["planComponent"] = resultsDocument.getPlanComponent(test.getPlanComponentID())
                            self.data["planComponents"][test.getPlanComponentID()]["tests"] = list()

                        self.data["planComponents"][test.getPlanComponentID()]["test"] = self.data["tests"][test.getPath()]

    def __betterTestRun(self, testDictionary, nextRunDocument, nextResultsDocument):
        if not testDictionary.get("bestRun"):
            return True
        # otherwise:
        currentResult = TestResult(testDictionary["bestRun"].getSummary())
        # if test results are equal, prefer the more recent result if it's been submitted
        if currentResult == TestResult(nextRunDocument.getSummary()):
            if nextResultsDocument.getSubmitTime() and (testDictionary.get("result") and not testDictionary["result"].getSubmitTime()):
                return True
            # otherwise
                return False
        return currentResult.better(nextRunDocument.getSummary())

    def processForm(self):
        if self.getParameter("assign"):
            return self.__processAssign()
        if self.getParameter("unassign"):
            return self.__unassign()
        return self.updateCWETestPlan()

    def __processAssign(self):
        updatedResultsDocuments = dict()
        for assignment in self.getListParameter("assignment"):
            self.data["assignments"] = self.getListParameter("assignment")
            # assignment values are:<componentID>|<test path>|[<device>]|[<udi>]|<results doc path>
            parts = assignment.split("|")
            if len(parts) != 5 :
                self.appendError("Error: could not parse results assignment %s" % assignment)
                continue

            # otherwise
            componentID = parts[0]
            testPath = parts[1]
            parameters = dict()
            if parts[2]:
                parameters[Constants.device] = parts[2]
            if parts[3]:
                parameters[Constants.udi] = parts[3]
            documentFilePath = parts[4]

            resultsDocument = updatedResultsDocuments.get(documentFilePath)
            if not resultsDocument:
                resultsDocument = ResultsDocument()
                resultsDocument.load(documentFilePath)
            test = resultsDocument.getTest(testPath, parameters)
            if not test:
                self.appendError("Error: could not find test %s in document %s" % (testPath, documentFilePath))
                continue

            # otherwise
            component = resultsDocument.getPlanComponent(componentID)
            if not component:
                self.appendError("Error: could not find plan component %s in document %s" % (componentID, documentFilePath))
                continue

            #otherwise
            test.setPlanComponentID(componentID)
            updatedResultsDocuments[documentFilePath] = resultsDocument

        for resultsDocument in updatedResultsDocuments.values():
            resultsDocument.save()

        if self.getErrors():
            return False
        return True

    def __unassign(self):
        componentID = self.getParameter("unassign")
        resultsDocumentPath = self.getParameter("rpath")
        if not resultsDocumentPath:
            self.appendError("Error: no document path (rpath)")
            return False

        resultsDocument = ResultsDocument()
        resultsDocument.load(resultsDocumentPath)
        for test in resultsDocument.getTests():
            if test.getPlanComponentID() == componentID:
                test.setPlanComponentID("")
        resultsDocument.save()

        return True

    def __compileLeverage(self):
        self.leverage = None
        if self.request.session.get("leverage"):
            self.leverage = Leverage()
            self.leverage.update(self.request.session.get("leverage"))
            self.data["leverage"] = self.leverage.__dict__
