# Copyright (c) 2014 Red Hat, Inc. All rights reserved. This copyrighted material
# is made available to anyone wishing to use, modify, copy, or
# redistribute it subject to the terms and conditions of the GNU General
# Public License v.2.
#
# This program is distributed in the hope that it will be useful, but WITHOUT ANY
# WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A
# PARTICULAR PURPOSE. See the GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
#
# Author: Greg Nichols
#
from datetime import datetime
from copy import deepcopy

from rhcert.tags import Constants, ServerConfiguration, Programs
from ..alert import Alert
from .certification import Certification


class Related(Certification):
    """ presents the a UI page, showing comments on the certification"""

    def __init__(self):
        Certification.__init__(self)

    def view(self, request):
        Certification.view(self, request)
        self.data["certification"] = self.certification
        self.data["certificationTab"] = "related"
        self.getRelCertsDetails()
        return self.data

    def getRelCertsDetails(self):
        relatedCertifications = self.getCertificationWorkflow().getRelatedCertifications(self.certificationID)
        self.data["cweRelatedCerts"] = deepcopy(relatedCertifications)
        if not relatedCertifications:
            return None
        """ Structure for RelatedCertifications
            'cweRelatedCerts': {'parent_certification_id': '',
                                'related_certifications': [
                                                {'certification_id': 3366983,
                                                 'program': 'hardware',
                                                 'redhat_product_platform_id': 1,
                                                 'relationship_type': 'layered',
                                                 'rh_base_version_id': 71,
                                                 'rh_product_id': 6,
                                                 'rh_product_version_id': 72,
                                                 'vendor_id': 677,
                                                 'vendor_product_id': 101}
                                ],
                                'self_type': 'original'},
        """
        # original/layered/pass-through/supplemental/pass-through with supplemental
        thisCertificationType = relatedCertifications['self_type']

        if relatedCertifications.get("parent_certification"):
            self.data["parent_certification"] = self.addProductNames(relatedCertifications.get("parent_certification"))

        relatedCertificationDetails = {}

        if thisCertificationType != "layered" and thisCertificationType != "supplemental":
            relatedCertificationDetails['layered'] = {
                "title": "Layered Certifications",
                "description": "Layered Certifications are additional Red Hat product certifications based on RHEL hardware certifications.",
                "certifications": []
            }
        if thisCertificationType == "original":
            relatedCertificationDetails['pass_through'] = {
                "description": "Pass-Through Certifications are RHEL hardware certifications based on previously certified partner products.",
                "title": "Pass-Through Certifications",
                "certifications": []
            }
            relatedCertificationDetails['supplemental'] = {
                "title": "Supplemental Certifications",
                "description": "Supplemental Certifications are for partner products that have revised components from a previously-certified product.",
                "certifications": []
            }
        relatedCertificationDetails['leverage'] = {
            "title": "Leveraged Certifications",
            "description": "Leveraged Certifications are certifications with test results that are proposed as meeting test requirements.",
            "certifications": []
        }

        if relatedCertifications.get("related_certifications"):
            for certificationData in relatedCertifications.get("related_certifications"):
                relationshipType = certificationData.get("relationship_type")
                # show pass-through-with-supplemental with other pass-through certs
                if relationshipType == "pass_through_with_supplemental":
                    certificationData["relationship_type"] = "pass_though"
                if not relatedCertificationDetails.get(relationshipType):
                    self.appendAlert(Alert(Alert.danger, "Error: invalid certification relationship %s " % relationshipType))
                    continue
                self.addProductNames(certificationData)
                relatedCertificationDetails[relationshipType]['certifications'].append(certificationData)

        # leveraged certs

        self.data['relatedCertifications'] = relatedCertificationDetails

        if relatedCertifications.get('parent'):
            self.data["relatedParent"] = self.getCertDetails(relatedCertifications['parent'])

        self.data['relatedSelfType'] = relatedCertifications['self_type']

    def getCertDetails(self, certificationID):
        certification = self.getConnection().getCertification(Programs.hardware, certificationID)
        if not certification:
            return None
        return self.addProductNames(certification.getCertificationParameters().getAll())

    def addProductNames(self, certificationData):

        (vendor, vendorProduct) = self.getStore().getVendorProducts().getVendorProductByID(
                                                                          certificationData.get("vendor_id"),
                                                                          certificationData.get("vendor_product_id"))
        # get vendor product info
        if not (vendor and vendorProduct):
                Alert(Alert.danger, "Error: no such vendor %s or vendor product %s in related certification" %
                                        (certificationData.get("vendor_id"),
                                        certificationData.get("vendor_product_id")))
        certificationData["vendor"] = vendor
        certificationData["vendorProduct"] = vendorProduct

        # get Red Hat product info
        try:
            redhatProductID = int(certificationData["rh_product_id"])
            redhatProductVersionID = int(certificationData["rh_product_version_id"])
            redhatBaseVersionID = 0
            if certificationData.get("rh_base_version_id"):
                redhatBaseVersionID = int(certificationData.get("rh_base_version_id"))
            (redhatProduct, redhatProductVersion, redhatBaseVersion) = self.productNameService.getProductPath(
                                                                                redhatProductID,
                                                                                redhatProductVersionID,
                                                                                redhatBaseVersionID)
            if not (redhatProduct and redhatProductVersion):
                self.appendError("Error: Could not determine Red Hat Product ID %s version ID %s for certification %s" % (
                    redhatProductID, redhatProductVersionID, certificationData.get("certification_id")))
                return None
        except Exception as error:
            self.appendError("Error: Could not determine Red Hat Product from IDs for certification:\n%s\n%s" %
                                        (certificationData, error))
            return None

        certificationData["redhatProduct"] = redhatProduct.get("name")
        certificationData["redhatProductVersion"] = redhatProductVersion.get("name")
        for platform in redhatProductVersion.get("platforms"):
            if platform.get("id") == certificationData.get("redhat_product_platform_id"):
                certificationData["redhatProductVersionPlatform"] = platform.get('name')
        certificationData["redhatBaseVersion"] = redhatBaseVersion
        return certificationData
