# Copyright (c) 2014 Red Hat, Inc. All rights reserved. This copyrighted material
# is made available to anyone wishing to use, modify, copy, or
# redistribute it subject to the terms and conditions of the GNU General
# Public License v.2.
#
# This program is distributed in the hope that it will be useful, but WITHOUT ANY
# WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A
# PARTICULAR PURPOSE. See the GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
#
# Author: Greg Nichols
#
import os

from rhcert.tags import Tags
from rhcert.host import Host
from rhcert.model.task import Task
from rhcert.compatability import XmlrpcServerProxy


from wsgi.certification.certification import Certification
from wsgi.include.testselector import TestSelector
from wsgi.alert import Alert


class CertificationResults(Certification):
    """ presents the certification results document listing UI page """

    def __init__(self):
        Certification.__init__(self)
        self.hostsByRoleID = {} # hosts by role ID

    def view(self, request):
        result = Certification.view(self, request)
        if type(result) != dict:
            return result
        self.data["certificationTab"] = "testing"

        # otherwise
        self.data["bodyEvents"] = "onbeforeunload='resetSelectors();'"

        # classic results view
        self.__viewCertification()

        # multihost testing?
        if self.certification.isMultiHost():
            self.template = 'certification/results-multihost.html'
            self.__viewMultihost()
            redirect = self.__runMultiHost()
            return redirect if redirect else self.data

        return self.data

    def __viewCertification(self):
        resultsDocuments = self.getStore().getResultsDocumentsFor(self.certification, skipEmpty=False)
        if resultsDocuments:
            hosts = dict()
            paths = dict()
            for (path, document) in resultsDocuments.items():
                host = Host(document.getRequiredOS().getName(), ipAddress=document.getRequiredOS().getIPAddress())
                hosts[host.getConnection()] = host
                if host.getConnection() not in paths:
                    paths[host.getConnection()] = list()
                paths[host.getConnection()].append(path)
                if document.getRequiredOS().getID(): # a multihost role ID is set
                    self.hostsByRoleID[document.getRequiredOS().getID()] = host

                self.data["hosts"] = list()
            for (connection, host) in hosts.items():
                hostDictionary = dict()
                self.data["hosts"].append(hostDictionary)
                hostDictionary["name"] =  host.getName()
                hostDictionary["connection"] =  connection
                hostDictionary["latestPath"] = paths[connection][0]
                hostDictionary["host"] = host

    def __viewMultihost(self):
        self.hostRoles = self.certification.getRequiredHostRoles()
        self.state = "testSelection"
        hosts = []
        for hostRole in self.hostRoles.values():
            if self.hostsByRoleID:
                host = self.hostsByRoleID.get(hostRole.getID())
                if host:
                    (document, path) = self.getStore().getLatestResultsDocument(host, self.program, self.certificationID)
                    if document:
                        testSelector = TestSelector(hostRole, document, path)

                    hosts.append({
                        "role": hostRole,
                        "host": host,
                        "document": document,
                        "path": path,
                        "testSelector": testSelector,
                        'context': testSelector.getContext()
                    })
                    continue
            # otherwise, no host selected for the role ID
            hosts.append({
                        "role": hostRole,
                        "host": None,
                        "document": None,
                        "path": None,
                        "testSelector": None,
                    })
            self.state = "hostSelection"

        self.data["testingState"] = self.__getTestingState()

        self.data['hostRoles'] = hosts

        resultsDocuments = self.getStore().getResultsDocumentsFor(self.certification, skipEmpty=False)
        results = []
        for resultsDocument in resultsDocuments.values():
            results.append({"document": resultsDocument, "hosts": resultsDocument.getRequiredHostRoles()})
        self.data["results"] = results

    def __getTestingState(self):
        """ determine state: hostSelection, testSelection, testing """
        if self.state == "hostSelection":
            return self.state

        # is the state testSelection, or testing ?
        allHostsReady = True
        hostStatus = {}
        self.data["multiHostStatus"] = hostStatus
        for hostID, host in self.hostsByRoleID.items():
            (status, info) = self.getHostStatus(host)
            hostStatus[hostID] = (status, info)

            # is this an interactive task?
            if status == "task":
                if info.getTask() == Task.message:
                    message = info.getParameter(Task.message)
                    hostStatus[hostID] = (status, message)
                    if not self.__isReady(message):
                        allHostsReady = False
                else:
                    allHostsReady = False

        if allHostsReady:
            return "testSelection"

        # otherwise busy/testing and not ready for test selection
        return "testing"

    def __isReady(self, message):
        """ move to RHCertClient? """
        return message == "ready" or "Finished" in message or ("clean" in message and "succeeded" in message)


    def __runMultiHost(self):
        if not self.getParameter("multi-host-run"):
            return None

        kickOffTimestamp = self.getCurrentUTCTime()
        for hostID, host in self.hostsByRoleID.items():
            (document, path) = self.getStore().getLatestResultsDocument(host, self.program, self.certificationID)
            if document:
                document.setMultiHostRunTime(kickOffTimestamp)
                self.requestRun(document, self.hostRoles[hostID])
            else:
                self.appendAlert(Alert(Alert.danger, "Could not find host document for %s" % host.getName(), self.request))

        return self.redirect("results", queryDict=None, paramList=[self.program,  self.certificationID])

    def requestRun(self, resultsDocument, hostDocument):
        # test options are "<test path> [--<parameter> <value>]*"
        # note, no tests may have been picked, electing to run just mandatory tests
        tests = []
        testOptions = self.getListParameter("new-run-" + hostDocument.getRoleAndID())
        if testOptions:
            for testOption in testOptions:
                testSpec = self.parseTestOption(testOption)
                if testSpec:
                    tests.append(testSpec)

        # prune XML doc to just the checked test specs
        self.selectTests(tests, resultsDocument)

        # set the test server to this one
        resultsDocument.setTestServer(self.getHostName())
        self.testHost = self.hostsByRoleID[hostDocument.getID()]
        self.testHostURL = 'http://%s:%s' % (self.testHost.getConnection(), self.environment.getServerListenerPort())
        self.server = XmlrpcServerProxy(self.testHostURL, allow_none=True)

        try:
            xml = resultsDocument.toXML()
            self.result = self.server.run(xml)
            if not self.result:
                self.appendError("Error: Run Request Failed.")
                return False

        except Exception as exception:
            self.appendError("Error: call to %s failed:\n%s" % (hostDocument.getRoleAndID(), self.testHostURL))
            self.appendError("%s\n" % exception)
            return False

        return True


