# Copyright (c) 2017 Red Hat, Inc. All rights reserved. This copyrighted material
# is made available to anyone wishing to use, modify, copy, or
# redistribute it subject to the terms and conditions of the GNU General
# Public License v.2.
#
# This program is distributed in the hope that it will be useful, but WITHOUT ANY
# WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A
# PARTICULAR PURPOSE. See the GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
#
# Author: Greg Nichols
#

import json
from django.shortcuts import render

from rhcert.tags import Tags, Programs
from rhcert.services.rhospfeatures import RHOSPFeatures
from .specification import Specification


class OpenStack(Specification):
    """ presents openstack spec pages (AKA "Product Details") """

    def __init__(self):
        Specification.__init__(self)
        self.components = {"certification-objectstorage-plugin": { "id": "objectstorage", "label": "Swift" },
                           "certification-blockstorage-plugin": { "id": "blockstorage",  "label": "Cinder" },
                           "certification-dataprocessing-plugin": { "id": "dataprocessing",  "label": "Sahara" },
                           "certification-networking-plugin": { "id": "networking",  "label": "Neutron" },
                           "certification-filestorage-plugin": { "id": "filestorage",  "label": "Manila" },
                           }
        self.rhospFeatures = RHOSPFeatures()

    def serve(self, request, template, details, certID):
        self.request = request
        self.details = details
        self.program = Programs.openstack
        self.template = template
        Specification.serve(self, request, self.template, self.program, certID)
        result = self.view(request)
        if type(result) != dict:
            return result
        self.data["template"] = self.template
        return render(request, self.template, self.data)

    def view(self, request):
        Specification.view(self, request)

        if not self.details:
            self.data["activeTab"] = "summary"
        else:
            if self.details == "details":
                self.data["activeTab"] = "specification-details"
            else:
                self.data["activeTab"] = self.details
            self.data["protocolFeatureReadOnly"] = True

        # pick sub-template files for picky components
        self.component = self.certification.getCertificationParameters().get("component")
        self.data["component"] = self.component
        # set up more-details tab
        if self.component and self.components.get(self.component):
            self.data["moreDetails"] = self.components.get(self.component)
            # if the request is for openstack "more" component details
            if self.details   and self.details != "details":
                self.template = "certification/specification/details/%s.html" % self.components.get(self.component).get("id")

        self.data["redhatProducts"] = json.dumps(self.productNameService.getProgram(self.program))

        self.ospProduct = None
        productCertification = self.certification.getProductCertification()
        if productCertification and productCertification.get(Tags.product):
            self.ospProduct = productCertification.get(Tags.product)
            if self.component in ['certification-networking-plugin', 'certification-blockstorage-plugin', "certification-filestorage-plugin"]:
                self.data["ospFeatures"] = {
                        "id": self.ospProduct.getVersion().getID(),
                        "component": self.rhospFeatures.getComponent(self.component, self.ospProduct.getVersion().getID())
                }

                self.__setProtocolFeatureSelections()

        return self.data

    def __setProtocolFeatureSelections(self):
        if self.data.get("certificationDetails") and self.data["certificationDetails"].get("supported_features"):
            # build the fully-populated tree
            self.data["featureSelection"] = {}
            for protocol, tests in self.data["certificationDetails"]["supported_features"].items():
                self.data["featureSelection"][protocol] = self.data["ospFeatures"]["component"]["tests"]

            # mark the selection
            for selectedProtocol, availableTests in self.data["featureSelection"].items():
                for availableTest in availableTests:
                    for test in self.data["certificationDetails"]["supported_features"][selectedProtocol]:
                        if test == availableTest.get("name"):
                            availableTest["selection"] = "checked"
