# Copyright (c) 2017 Red Hat, Inc. All rights reserved. This copyrighted material
# is made available to anyone wishing to use, modify, copy, or
# redistribute it subject to the terms and conditions of the GNU General
# Public License v.2.
#
# This program is distributed in the hope that it will be useful, but WITHOUT ANY
# WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A
# PARTICULAR PURPOSE. See the GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
#
# Author: Greg Nichols
#
import os

from django.shortcuts import render

from rhcert.model.resultsdocument import ResultsDocument
from rhcert.tags import Constants
from ..utils import checkGzippedFile

import gzip
from rhcert.compatability import StringIO, rhcert_make_str


from ..view import View
from ..alert import Alert


class UploadResults(View):

    def __init__(self):
        View.__init__(self)
        self.resultsDocument = None
        self.resultsPath = None
        self.resultsData = None
        self.state = None
        self.template = "certification/upload/choose.html"
        self.loginRequired = True

    def serve(self, request, program, certificationID):
        View.authenticate(self, request)
        self.program = program
        self.certificationID = certificationID
        result = self.view(request)
        if type(result) != dict:
            return result
        return render(request, self.template, self.data)

    def view(self, request):
        View.view(self, request)
        self.data["program"] = self.program
        self.data["certificationID"] = self.certificationID

        # check if we're cancelled
        if not self.getParameter("cancel"):
            if self.getParameter("state"):
                self.state = self.getParameter("state")
            if self.choosingFile():
                return self.data
            if self.uploading():
                return self.data

            # NOTE: In case of Management mode, we have already submitted
            # the file. So we skip this part to avoid re-submitting
            if not self.isManagementMode and self.submitting():
                return self.data

        if self.isManagementMode:
            self.resultsPath = self.getParameter("resultsPath")
            return self.redirect("progress", queryDict=None,
                                     paramList=[self.program,  self.certificationID])

        # if returnAfter set and it indicates "progress"
        if "progress" in self.getParameter("returnAfter"):
            return self.redirect("progress", queryDict=None,
                                     paramList=[self.program,  self.certificationID])
        # otherwise, go to the results
        return self.redirect("results", queryDict=None,
                                     paramList=[self.program,  self.certificationID])

    def choosingFile(self):
        if self.state or self.getParameter("resultsPath"):
            return False

        # Here we don't need to save the file
        if self.getParameter("results-file") and self.checkFile():
            self.state = "uploaded"
            return False

        self.data["nextButtonLabel"] = "Upload"
        self.template = "certification/upload/choose.html"
        self.data["uploadState"] = ""
        if not self.data.get("returnAfter"):
            self.data["returnAfter"] = self.request.META.get("HTTP_REFERER")
        return True

    def uploading(self):
        if self.state == "uploadConfirmed": 
            if self.isManagementMode:
                self.submitFile()
            else:
                self.fixFileProgramAndID()

            self.data["resultsPath"] = self.resultsPath
            return False

        if self.state != "uploaded":
            return False

        self.data["resultsPath"] = self.resultsPath
        self.data["resultsFileName"] = self.request.session['sResultsFileName']
        self.data["nextButtonLabel"] = "Confirm Upload"
        self.template = "certification/upload/confirm.html"
        self.data["uploadState"] = "uploadConfirmed"
        return True

    def submitting(self):
        if self.state == "submitConfirmed":
            self.submitFile()
            self.state = "submitted"
            # incase errors occur on submit
            self.data["nextButtonLabel"] = "Upload"
            self.template = "certification/upload/choose.html"
            self.data["uploadState"] = ""
            return self.getErrors()

        self.data["resultsPath"] = self.resultsPath
        self.data["resultsFileName"] = os.path.basename(self.resultsPath)
        self.data["nextButtonLabel"] = "Confirm Submission"
        self.template = "certification/upload/submit.html"
        self.data["uploadState"] = "submitConfirmed"
        return True

    # Check file to replace saving the file but still perform the validation we need on the file.
    def checkFile(self):
        uploadedFile = self.request.FILES.get('results-file')
        if not uploadedFile:
            return False
        
        if not (self.getStore().resultsFileRegex.search(uploadedFile.name) or self.getStore().multiHostFileRegex.search(uploadedFile.name) or (os.path.basename(uploadedFile.name) == "results.xml")):
            alert = Alert(Alert.danger, "Error: the name of the file '%s' does not match the format of a results file. Please upload a valid certification test results file ( .xml or .xml.gz )." % (uploadedFile.name), self.request)
            
            self.appendAlert(alert)
            return False
        
        try:
            # it might be compressed
            (isGzip, fileName) = checkGzippedFile(uploadedFile)

            self.resultsPath = fileName
            self.request.session['sResultsFileName'] = self.resultsPath

        except Exception as e:
            self.appendError("Error: Could not save file %s  %s\n" %  (fileName, e))
            return False

        if not self.resultsPath:
            self.appendError("Error: Could not save file %s  %s\n" %  (fileName, e))
            return False
            
        self.resultsDocument = ResultsDocument()

        if isGzip:
            stream = StringIO(uploadedFile.read())
            gzfile = gzip.GzipFile(fileobj=stream)
            fileContents = gzfile.read()
            # reset our session file data for later use
            self.request.session['sResultsData'] = fileContents
            self.resultsDocument.loadString(fileContents)
        else:
            if "octet-stream" in uploadedFile.content_type:
                self.resultsData = uploadedFile.read().decode("base64")
            else:
                self.resultsData = uploadedFile.read()

            # Without the file save we lose the data we need when we need to submit it.
            self.request.session['sResultsData'] = self.resultsData
            self.resultsDocument.loadString(self.resultsData)
        
        if self.resultsDocument.getCertificationID() and (self.resultsDocument.getCertificationID() != self.certificationID):
            self.appendAlert(Alert(Alert.warning, "This results file indicates it is for a different certification: %s %s"
                                   % (self.resultsDocument.getCertificationProgram(), self.resultsDocument.getCertificationID())))

        return True

    def saveFile(self):
        uploadedFile = self.request.FILES.get('results-file')
        if not uploadedFile:
            return False

        # make sure the results file name is the right format
        if not (self.getStore().resultsFileRegex.search(uploadedFile.name) or self.getStore().multiHostFileRegex.search(uploadedFile.name) or (os.path.basename(uploadedFile.name) == "results.xml")):
            alert = Alert(Alert.danger, "Error: the name of the file '%s' does not match the format of a results file. Please upload a valid certification test results file ( .xml or .xml.gz )." % (uploadedFile.name), self.request)
            # alert.appendMessage("regex: %s" % self.getStore().resultsFileRegex.pattern)
            self.appendAlert(alert)
            return False

        try:
            # it might be compressed
            fileName = checkGzippedFile(uploadedFile)

            if "octet-stream" in uploadedFile.content_type:
                fileContents = uploadedFile.read().decode("base64")
            else:
                fileContents = uploadedFile.read()

            if not self.isManagementMode:
                self.resultsPath = self.getStore().saveResultsFileSimple(fileName, fileContents)
            else:
                # Copy the file content in a temporary file and do not store it in rhcert store
                tempResultsFile = open("/tmp/" + fileName, mode="wb")
                tempResultsFile.write(fileContents)
                tempResultsFile.close()
                self.resultsPath = tempResultsFile.name

            if self.getStore().getErrors():
                self.appendErrors(self.getStore().getErrors())
                return False
        except Exception as e:
            self.appendError("Error: Could not save file %s  %s\n" %  (fileName, e))
            return False
        if not self.resultsPath:
            self.appendError("Error: Could not save file %s  %s\n" %  (fileName, e))
            return False

        # check to see if it's already got a certification ID from another certification
        self.resultsDocument = ResultsDocument()
        self.resultsDocument.load(self.resultsPath)
        if self.resultsDocument.getCertificationID() and (self.resultsDocument.getCertificationID() != self.certificationID):
            self.appendAlert(Alert(Alert.warning, "This results file indicates it is for a different certification: %s %s"
                                   % (self.resultsDocument.getCertificationProgram(), self.resultsDocument.getCertificationID())))


        return True

    # We will remove all file operations on this one as well.
    def fixFileProgramAndID(self):
        # we need to node id too
        certification = self.getConnection().getCertification(self.program, self.certificationID)
        

        # When we are calling this from other places than the submit file we don't have a results document

        self.resultsPath = self.request.session['sResultsFileName']
        self.resultsData = self.request.session['sResultsData']
        
        self.resultsDocument = ResultsDocument()
        self.resultsDocument.loadString(rhcert_make_str(self.resultsData))
        
        self.resultsDocument.setCertificationProgram(self.program)
        self.resultsDocument.setCertificationID(self.certificationID)
        self.resultsDocument.setCertificationState(certification.getCertificationState())
        self.resultsDocument.setNodeID(certification.getNodeID())

        # if it's not the right format for the name - rename it.
        if self.resultsDocument.hasMultiHostResults():
            newFileName = self.resultsDocument.getMultiHostFileName()
        else:
            newFileName = self.resultsDocument.getVerboseFileName()

        (_, ext) = os.path.splitext(self.resultsPath)
        if ext == ".gz":
            newFileName += ".gz"
        self.resultsPath = newFileName

    def moveFile(self):
        self.fixFileProgramAndID()
        return self.getStore().fixResultsFileLocation(self.resultsDocument)

    def submitFile(self):     
        self.fixFileProgramAndID()
        
        self.getCertificationWorkflow().submitTestRunNoFileSave(self.resultsPath, self.resultsDocument, self.resultsData)
        self.appendErrors(self.getCertificationWorkflow().getErrors())
        if not self.getErrors():
            # self.resultsDocument.setSubmitTime(self.getCurrentUTCTime())
            # self.resultsDocument.setState(Constants.closed)
            # self.resultsDocument.save()
            self.appendAlert(Alert(Alert.success, "Results file %s submitted for review" % (os.path.basename(self.resultsPath)), self.request))
