import importlib
import json
import sys

from django.http import HttpResponse
from django.views.generic import TemplateView
from rhcert.client.client import RHCertClient
from rhcert.collabrequest import CollabRequest
from rhcert.environment import Environment
from rhcert.host import Host
from rhcert.model.task import Task


class CommandView(TemplateView):
    """ View used for test collaboration function run requests, ie. calling static
        functions existing in test suite test classes on LTS during a test run directed by
        the test being run on the SUT. """

    template_name = "commands.html"

    def get(self, request, *args, **kwargs):
        """ Request a function be run on LTS.

            Request should have a parameter "message_body"
            Currently only supports non-root operations

            ex: ?message_body={"test_name": "IPXETest", "function_name": "get_details", "test_package_trail": "hwcert.tests.ipxe.ipxe", "args":[], "kwargs":{}}
            where args and kwargs are optional.

            Return value from the function called is returned with 'OK' 200 status.
            Errors an input return with 400 status and specific reason.
            Errors occurring when running the function return with 500 and a general reason.
        """
        context = {}
        # Check if message_body is present
        # If not present render normal html
        if "message_body" in request.GET.keys():
            post_body = request.GET.get("message_body")
            print("Message Body is %s" % post_body)

            collab_request = self.parse_collab_request(post_body)
            # If it's an error, send it back
            if isinstance(collab_request, HttpResponse):
                return collab_request

            # Check if the package_trail and function is valid
            function = self.find_function(collab_request)
            if isinstance(function, HttpResponse):
                return function

            # Only supports non-root operations
            return self.without_root(collab_request, function)
        else:
            return self.render_to_response(context)

    def without_root(self, collab_request, function):
        try:
            function_result = function(*collab_request.args, **collab_request.kwargs)
            print("function result = %s" % function_result)
        except Exception as e:
            return HttpResponse(str(e), status=500, reason="Error running function on server")
        return HttpResponse(function_result)


    def post(self, request, *args, **kwargs):
        """ Request a function be run on LTS.

            Message body must be in json format

            ex: {"test_name":"LidTest", "function_name":"say_hello", "test_package_trail":"hwcert.tests.lid.lid", "args":[], "kwargs":{}} where args and kwargs are optional.

            Return value from the function called is returned with 'OK' 200 status.
            Errors in input return with 400 status and specific reason.
            Errors occurring when running the function return with 500 and a general reason.
        """

        environment = Environment()
        collab_request = self.parse_collab_request(request.body)

        # If it's an error, send it back
        if isinstance(collab_request, HttpResponse):
            return collab_request

        # Check if the package_trail and function is valid
        function = self.find_function(collab_request)
        if isinstance(function, HttpResponse):
            return function

        if not collab_request.requires_root:
            return self.without_root(collab_request, function)

        host = Host("127.0.0.1")
        clientProxy = RHCertClient(host, timeout=5)
        task = Task(Task.collab)
        task.setParameter(Task.testName, collab_request.test_name)
        task.setParameter(Task.testPackageTrail, collab_request.test_package_trail)
        task.setParameter(Task.functionName, collab_request.function_name)
        task.setParameter(Task.args, collab_request.args)
        task.setParameter(Task.kwargs, collab_request.kwargs)
        result = clientProxy.runXML(task.toXML())
        print("result=%s" % result)
        return HttpResponse(result)


    def find_test(self, collab_request):
        environment = Environment()
        suites_dir = environment.getSuiteDirectory()
        if suites_dir not in sys.path:
            sys.path.append(suites_dir)
        try:
            module = importlib.import_module(collab_request.test_package_trail)
        except ImportError:
            return HttpResponse(status=400, reason="Invalid package trail")
        if hasattr(module, collab_request.test_name):
            return getattr(module, collab_request.test_name)
        else:
            return HttpResponse(status=400, reason="Invalid test name")

    def find_function(self, collab_request):
        TestClass = self.find_test(collab_request)

        if isinstance(TestClass, HttpResponse):
            return TestClass

        if hasattr(TestClass, collab_request.function_name):
            return getattr(TestClass, collab_request.function_name)
        else:
            return HttpResponse(status=400, reason="Invalid function name")

    def parse_collab_request(self, message_body):
        try:
            body_unicode = message_body.decode('utf-8')
            body = json.loads(body_unicode)
        except ValueError:
            return HttpResponse(status=400, reason="invalid json format")
        test_name = body.get('test_name', '')
        test_package_trail = body.get('test_package_trail', '')
        function_name = body.get('function_name', '')
        requires_root = body.get('requires_root', False)
        args = body.get('args', [])
        kwargs = body.get('kwargs', {})
        print("test=%s, package=%s, func=%s, root=%s" % (test_name, test_package_trail, function_name, requires_root))
        print("args = %s" % args)
        print("kwargs = %s" % kwargs)
        if test_name and test_package_trail and function_name:
            return CollabRequest(test_name, test_package_trail, function_name, requires_root, args, kwargs)
        else:
            return HttpResponse(status=400, reason="missing data; test name, package trail, and function name required")
