# Copyright (c) 2017 Red Hat, Inc. All rights reserved. This copyrighted material
# is made available to anyone wishing to use, modify, copy, or
# redistribute it subject to the terms and conditions of the GNU General
# Public License v.2.
#
# This program is distributed in the hope that it will be useful, but WITHOUT ANY
# WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A
# PARTICULAR PURPOSE. See the GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
#
# Author: Greg Nichols <gnichols@redhat.com>
import os
import errno
#
from rhcert.tags import Constants, ServerConfiguration, Programs
from rhcert.environment import Environment
from rhcert.rhcertStore import RedHatCertificationStore
from rhcert.model.vendorproducts import VendorProductsDocument
from rhcert.workflow.config import Config as WorkflowConfig
from rhcert.workflow.certification import Certification as CertificationWorkflow
from rhcert.workflow.vendorProduct import VendorProduct as VendorProductWorkflow
from rhcert.workflow.engine import Engine as EngineWorkflow
from .alert import Alert


class Connection(object):
    """ Connector  from page generators to certification data/metadata """

    def __init__(self, request):
        self.request = request
        self.environment = Environment()
        self.vendorProducts = None
        self.certifications = dict()
        self.rhcertStore = None
        self.certificationWorkflow = None
        self.vendorProductWorkflow = None
        self.engineWorkflow = None
        self.errors = list()
        self.reloadVendorProducts = False


        self.configuration = self.environment.getServerConfiguration()
        database = ""
        self.username = ""
        if self.request.session.get(Constants.username) and self.environment.isMultiStoreEnabled():
            self.username = self.request.session.get(Constants.username)
            database = os.path.join(self.username, self.environment.getPhase())
        self.rhcertStore = RedHatCertificationStore(database)
        self.data = {"alerts": []}
        debug = Constants.off
        if self.environment.getServerConfiguration() != ServerConfiguration.management:
            debug = self.getServerSettings().getDebug()
        if self.environment.getServerConfiguration() != ServerConfiguration.testing:
            self.workflowConfig = WorkflowConfig(self.environment, self.rhcertStore, debug)
            self.certificationWorkflow = CertificationWorkflow(self.workflowConfig)
            self.certificationWorkflow.setLogin(self.request.session)
            self.vendorProductWorkflow = VendorProductWorkflow(self.workflowConfig)
            self.vendorProductWorkflow.setLogin(self.request.session)
            self.engineWorkflow = EngineWorkflow(self.workflowConfig)
            self.engineWorkflow.setLogin(self.request.session)

    def getUsername(self):
        return self.username # will be "" if not using multistore

    def getErrors(self):
        errors = list()
        for alert in self.data["alerts"]:
            if alert.getLevel() == Alert.danger:
                errors.append(alert)
        return errors

    def appendError(self, error):
        if error:
            self.appendAlert(Alert(Alert.danger, error, self.request))

    def appendErrors(self, errors):
        for error in errors:
            self.appendAlert(Alert(Alert.danger, error, self.request))

    def appendAlert(self, alert):
        if not isinstance(alert, Alert):
            raise Exception("Error: adding non-Alert to appendAlert call: %s" % alert)
        self.data["alerts"].append(alert)

    def clearErrors(self):
        self.errors = list()

    def isLoggedIn(self):
        if self.certificationWorkflow and self.certificationWorkflow.isLoggedIn():
            return True
        return False

    def getCertificationWorkflow(self):
        return self.certificationWorkflow

    def getVendorProductWorkflow(self):
        return self.vendorProductWorkflow

    def getEngineWorkflow(self):
        return self.engineWorkflow

    def getStore(self):
        return self.rhcertStore

    def getServerSettings(self):
        return self.rhcertStore.getServerSettings()

    def getVendorProducts(self):
        # default - file store based
        if self.configuration != ServerConfiguration.management:
            reload = False
            if self.reloadVendorProducts:
                reload = True
                self.reloadVendorProducts = False
                self.__getCWEVendorProducts()
            return self.rhcertStore.getVendorProducts(reload)

        # otherwise - management - session-based
        if not self.vendorProducts:
            self.__getSessionVendorProducts()
        if not self.vendorProducts:
            self.__getCWEVendorProducts()
        return self.vendorProducts

    def resetVendorProducts(self):
        if self.configuration != ServerConfiguration.management:
            if self.request.session.get("vendorproducts"):
                self.request.session["vendorproducts"] = None
            self.vendorProducts = None
            self.reloadVendorProducts = True
            return True

        # otherwise - default
        self.__getCWEVendorProducts()

    def getVendorProductByID(self, vendorID, productID):
        (vendor, product) = self.getVendorProducts().getVendorProductByID(vendorID, productID)
        if not (vendor and product):
            self.resetVendorProducts()
            self.getVendorProducts()
        return  self.getVendorProducts().getVendorProductByID(vendorID, productID)

    def getCertifications(self):
        if self.configuration != ServerConfiguration.management:
            return self.rhcertStore.getCertifications()
        self.certificationWorkflow.setLogin(self.request.session)
        self.vendorProductWorkflow.setLogin(self.request.session)
        self.__getCWECertifications()
        return [ value for value in self.certifications.values() ]

    def getCertification(self, program, certificationID):
        if not self.certificationWorkflow:
            return None

        # For management configuration, get the cert from CWE/hydra
        if self.configuration == ServerConfiguration.management:
            return self.__getCertificationFromCWE(program, certificationID)

        # For testing and default:
        # For sandbox, get it from store
        (_, certification) = self.rhcertStore.getCertification(program, certificationID)
        if certification and certification.isSandbox():
            return certification

        # otherwise get it now from CWE/hydra, and save it locally
        certification = self.__getCertificationFromCWE(program, certificationID)
        if certification:
            certification = self.rhcertStore.updateCertification(certification)

        return certification

    def getVendorProductCertifications(self, product, searchID=None):
        """ returns new Certification objects for a product
            if certificationID is set, it only returns that cert if found """
        certificationsData = self.certificationWorkflow.getCertifications(product.getID(), product.getProgram())
        self.appendErrors(self.certificationWorkflow.getErrors())
        if not certificationsData:
            print(("No certifications for %s product %s", product.getProgram(), product.getID()))
            self.appendError("Error: No certifications for (%s) product (%s)" % (product.getProgram(), product.getID()))
            return None

        certifications = list()
        for certificationData in certificationsData:
            certification = self.getCertificationWorkflow().makeCertification(certificationData)
            if not certification:
                self.appendErrors(self.certificationWorkflow.getErrors())
                continue

            certification = self.getStore().updateCertification(certification)
            if certification and certification.getCertificationID() == searchID:
                    return [certification]

            certifications.append(certification)

        if searchID:
            return []
        return certifications

    def __getCertificationFromCWE(self, program, certificationID):
        if not (self.certifications and self.certifications.get(certificationID)):
            details = self.certificationWorkflow.get(program, certificationID)
            self.appendErrors(self.certificationWorkflow.getErrors())
            if not details:
                return None

            # workaround for difference in API response of Cert.getCertifications and Cert.getHardware_4_10
            if not details.get('vendor_product_make') and details.get('make'):
                details['vendor_product_make'] = details['make']
            if not details.get('vendor_product_name') and details.get('model'):
                details['vendor_product_name'] = details['model']
            if not details.get('status') and details.get('certification_result'):
                details['status'] = details['certification_result']
            if not details.get('platform') and details.get('architecture'):
                details['platform'] = details['architecture']

            certification = self.certificationWorkflow.makeCertification(details, program=program)
            self.appendErrors(self.certificationWorkflow.getErrors())
            if not certification:
                return None
            certification.setCertificationState(Constants.cwe)
            self.__updateVendorAndProduct(details, certification)
            self.certifications[certification.getCertificationID()] = certification
            self.certifications[certificationID] = certification
        return self.certifications.get(certificationID)

    def __getSessionVendorProducts(self):
        if self.request.session.get("vendorproducts"):
            self.vendorProducts = VendorProductsDocument()
            try:
                self.vendorProducts.loadString(self.request.session["vendorproducts"].encode('utf-8'))
            except Exception as error:
                self.appendError("Error: could not read session vendor products")
                self.appendError(str(error))
                self.vendorProducts = None

    def __getCWEVendorProducts(self):
        catalogProducts = self.vendorProductWorkflow.list()
        if self.vendorProductWorkflow.getErrors():
            self.appendErrors(self.vendorProductWorkflow.getErrors())
            self.appendError("Error: Could not download vendor products:")
            for error in self.vendorProductWorkflow.getErrors():
                self.appendError(error)
            if not catalogProducts:
                return False

        self.vendorProducts = self.rhcertStore.getVendorProducts()

        if not catalogProducts:
            self.appendError("Warning: user has no vendor products")
            return self.vendorProducts

        self.getVendorProductWorkflow().updateVendorProducts(catalogProducts, self.vendorProducts)
        self.appendErrors(self.getVendorProductWorkflow().getErrors())

        if self.configuration != ServerConfiguration.management:
            self.vendorProducts.save()
        else:
            # save in the users session
            self.request.session["vendorproducts"] = self.vendorProducts.document.toxml(encoding='utf-8')

    def __makeDirectoryPath(self, directory):
        """Emulate mkdir -p behavior: create the given directory, including all
        needed parent directories. Do not return an error if the directories
        already exist."""
        try:
            os.makedirs(directory)
        except OSError as e:
            if e.errno != errno.EEXIST: raise

    def __getCWECertifications(self):
        cweCertifications = self.certificationWorkflow.getCertifications(dateRange=45)
        if not cweCertifications:
            return []
        self.certifications = dict()
        for certificationData in cweCertifications:
            if certificationData.get("cert_type") == "container":
                continue # ignore container certs
            certification = self.certificationWorkflow.makeCertification(certificationData)

            if not certification:
                continue # failed to make certification

            certification.setCertificationState(Constants.cwe)
            if not self.__updateVendorAndProduct(certificationData, certification):
                continue
            self.certifications[certification.getCertificationID()] = certification

        # Collect all errors that might have occurred previous calls
        self.appendErrors(self.certificationWorkflow.getErrors())

    def __updateVendorAndProduct(self, certificationData, catalogCertification):
        productID = certificationData.get("vendor_product_id")
        vendorID = certificationData.get("vendor_id")

        if not vendorID:
            # search by vendor name and product ID to find a match
            for vendor in self.getVendorProducts().getVendorsByName(certificationData.get("vendor")):
                (vendor, product) = self.getVendorProductByID(vendor.getID(), productID)
                if vendor and product:
                    vendorID = int(vendor.getID())
                    break

        productName = certificationData.get("vendor_product_name")

        (vendor, product) = self.getVendorProductByID(vendorID, productID)
        if vendor and product:
            vendor = catalogCertification.setVendor(vendor)
            vendor.setProduct(product)
            return True

        # errors
        if not vendor:
            self.appendError("Error: no such vendor ID %s" % vendorID)
        if not product:
            self.appendError("Error no such product ID %s name %s" % (productID, productName))
        return False
