# Copyright (c) 2017 Red Hat, Inc. All rights reserved. This copyrighted material
# is made available to anyone wishing to use, modify, copy, or
# redistribute it subject to the terms and conditions of the GNU General
# Public License v.2.
#
# This program is distributed in the hope that it will be useful, but WITHOUT ANY
# WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A
# PARTICULAR PURPOSE. See the GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
#
# Author: Greg Nichols
#
import json

from django.http import HttpResponseRedirect
from rhcert.services.rhospfeatures import RHOSPFeatures
from rhcert.tags import Constants, Programs, ServerConfiguration

from ..alert import Alert
from .create import Create
from .product import Product

class Certification(Product):
    """ creates a new certification on either a new or existing vendor product """

    def __init__(self):
        Product.__init__(self)
        self.rhospFeatures = RHOSPFeatures()

    def serve(self, request, vendorID, productID):
        self.vendorID = vendorID
        self.productID = productID
        return Product.serve(self, request)

    def view(self, request):
        result = Create.view(self, request)
        if type(result) != dict:
            return result

        # if "workflow" is on the query string, make sure it's in our "parameters" dictionary
        if self.getParameter("workflow"):
            self.data["parameters"]["workflow"] = self.getParameter("workflow")

        if self.choosingRedHatCertification():
            return self.data

        if self.choosingProduct():
            return self.data

        check_layered = self.isLayeredCertification(self.getParameter("rh_product_id"), self.program)
        if check_layered:
            params = {
                "rh_base_version_id": self.getParameter("rh_base_version_id"),
                "rh_product_id": self.getParameter("rh_product_id"),
                "rh_product_version_id": self.getParameter("rh_product_version_id"),
                "layered_product_format": self.getParameter("layered_product_format"),
                "vendorID": self.getParameter("vendor_id"),
                "productID": self.getParameter("vendor_product_id")
            }
            if isinstance(check_layered, tuple) and check_layered[1] == Constants.RHOSP:
                # For RHOSP layered cert the rh_product_id is 12
                # But in the new certification dialog we show only one RHOSP product which is
                # for OpenStack program and that rh_product_id as 3,
                # so we are fixing the rh_product_id and rh_product_version_id for RHOSP hardware
                # layered certification
                redhatProductID = "12"
                redhatProductVersionID = str()
                rhosp_hardware_product = self.productNameService.getProduct("12")
                rhosp_openstack_product = self.productNameService.getProduct("3")
                # Convert the product version of openstack product to hardware product
                openstack_product_version = self.productNameService.getProductVersion(
                                                             rhosp_openstack_product,
                                                             self.getParameter("rh_product_version_id"))
                if openstack_product_version:
                   hardware_product_version = [version for version in rhosp_hardware_product["versions"] if version["name"] == openstack_product_version["name"]][0]
                if hardware_product_version:
                   redhatProductVersionID = str(hardware_product_version.get("id"))
                params["rh_product_id"] = redhatProductID
                params["rh_product_version_id"] = redhatProductVersionID

            return self.redirect("create-layered-cert", queryDict=params)

        if self.creatingCertification():
            return self.data

        # Certificate creation and download is now complete
        if self.certificationID and self.program:
            msg = "Created Certification for Program {} with ID: {}".format( \
                self.getProgramFullName(), str(self.certificationID))

            Alert(Alert.success, msg, request)
            return self.redirect("certification", queryDict=None,
                                 paramList=[self.program,  self.certificationID])
        # otherwise
        Alert(Alert.danger, "Error in creating new Certification", request)
        return self.redirect("certifications")

    def choosingRedHatCertification(self):
        if self.getParameter("rh_product_id"):
            return False

        self.data["redhatProducts"] = json.dumps(self.productNameService.getAllProducts())
        self.data["nextButtonValue"] = "done"
        self.data["createTitle"] = "New Certification"
        self.data["createDescription"] = "Choose the Red Hat certification."
        self.data["nextButtonLabel"] = "Next"
        self.template = "create/certification/all.html"
        return True

    def choosingProduct(self):
        if self.getParameter("vendor_id"):
            self.vendorID = self.getParameter("vendor_id")
        if self.getParameter("vendor_product_id"):
            self.productID = self.getParameter("vendor_product_id")

        if self.getParameter("vendorProduct"):
            #vendorProduct is chosen
            return False

        if self.creatingProduct():
            return True

        if self.choosingExistingProduct():
            return True

        # otherwise
        self.program = self.getParameter("program")
        try:
            self.data["vendorID"] = self.vendorID
            self.data["productID"] = self.productID
            vendor = self.getConnection().getVendorProducts().getVendorByID(self.vendorID)
            if not vendor:
                self.appendError("Error: Could not find vendor ID %s" % self.vendorID)
                return True # not done: broken
            product = vendor.getProduct(self.productID)
            if not product:
                self.appendError("Error: Could not find vendor product ID %s" % self.productID)
                return True # not done: broken
            self.data["vendorProduct"] = product
            self.data["redhatProducts"] = json.dumps(self.productNameService.getProgram(self.program))
            self.program = product.getProgram()
            if self.program == Programs.openstack and self.data['parameters']['rh_product_id'] == '1':
                self.program = Programs.software
            elif self.program == Programs.cloud and self.data['parameters']['rh_product_id'] == '31':
                self.program = Programs.cloud_sap
            self.template = "create/certification/%s.html" % self.program
            return False

        except Exception as error:
            self.appendError("Error: could not determine selected product:")
            self.appendError(error)
            return True # not done: broken!


    def choosingExistingProduct(self):
        if not (self.vendorID and self.productID):
            self.compileVendorProducts(self.getParameter("rh_product_id"), self.getParameter("workflow"))
            self.data["createTitle"] = "New Certification"
            self.data["createDescription"] = "Choose the product to be certified."
            self.data["nextButtonValue"] = "next"
            self.data["nextButtonLabel"] = "Next"
            self.template = "create/choose-product.html"

            # Disable the button to create New Product for Layered Certification
            if self.isLayeredCertification(self.getParameter("rh_product_id"), self.program):
                self.data["disableNewProduct"] = True
            return True
        return False


    def creatingCertification(self):
        if self.program != Programs.openstack:
            return self.createCertification()

        elif self.program == Programs.openstack:
            if self.getParameter("next-button") == "done":
                # done with specification steps (features, etc)
                return self.createCertification()

            # otherwise, openstack Step 2: has another step for specifying the plugin features etc.
            if self.getParameter("next-button") == "create-certification-openstack":

                director_integration_attachment = self.getParameter("director_integration_file")
                if director_integration_attachment:
                    self.data['director_integration_file'] = {
                        "name": director_integration_attachment.name,
                        "data": director_integration_attachment.read(),
                        "content_type": director_integration_attachment.content_type,
                    }

                # if there's a step two, set it up
                component = self.getParameter("component")
                componentKey = component.split("-")[1]
                if (componentKey in ["blockstorage", "dataprocessing", "filestorage", "networking", "objectstorage", "vnf"]
                                    and not self.getParameter("component-done")):
                    self.template = "create/certification/%s.html" % (componentKey)
                    if component in ['certification-networking-plugin', 'certification-blockstorage-plugin', 'certification-filestorage-plugin']:
                        self.data["ospFeatures"] = {
                                "id": self.data['parameters']['rh_product_version_id'],
                                "component": self.rhospFeatures.getComponent(component, self.data['parameters']['rh_product_version_id'])
                        }
                    self.data["createTitle"] = "Create a Red Hat OpenStack Platform Certification"
                    self.data["nextButtonValue"] = "done"
                    self.data["nextButtonLabel"] = "Create"
                    self.data["parameters"]["component-done"] = componentKey
                    return True
                # otherwise, no second step, or second step is done
                return self.createCertification()

            # otherwise, openstack step 1: pick the plugin type, etc.
            self.data["nextButtonValue"] = "create-certification-openstack"
            self.data["createTitle"] = "New Certification"
            self.data["nextButtonLabel"] = "Next"
            self.template = "create/certification/openstack.html"
            return True

        return False


    def choosingCertification(self):

        if self.getParameter("next-button") == "done":
            return False

        # openstack has more steps
        if self.getParameter("next-button") == "create-certification-openstack":
            # if there's a step two, set it up
            component = self.getParameter("component")
            componentKey = component.split("-")[1]
            if (componentKey in ["blockstorage", "dataprocessing", "filestorage", "networking", "objectstorage"]
                                and not self.getParameter("component-done")):
                self.template = "create/certification/%s.html" % (componentKey)
                if component in ['certification-networking-plugin', 'certification-blockstorage-plugin', 'certification-filestorage-plugin']:
                    self.data["ospFeatures"] = {
                            "id": self.data['parameters']['rh_product_version_id'],
                            "component": self.rhospFeatures.getComponent(component, self.data['parameters']['rh_product_version_id'])
                    }
                self.data["createTitle"] = "Create a Red Hat OpenStack Platform Certification"
                self.data["nextButtonValue"] = "done"
                self.data["nextButtonLabel"] = "Create"
                self.data["parameters"]["component-done"] = componentKey
                return True
            # otherwise, no second step, or second step is done
            return False

        self.data["nextButtonValue"] = "done"
        if self.program == Programs.openstack:
            self.data["nextButtonValue"] = "create-certification-openstack"
        self.data["createTitle"] = "Create a %s" % self.getProgramFullName()
        self.data["nextButtonLabel"] = "Next"
        self.template = "create/certification/%s.html" % self.program
        return True


    def createCertification(self):
        self.getVendorAndProduct()
        parameters = self.getPostDictionary()
        if self.product:
            parameters["vendor_product_portal_id"] = self.product.getNodeID()

        if parameters.get("supported_regions"):
            if isinstance(parameters["supported_regions"], list):
                parameters["supported_regions"] = ','.join(parameters["supported_regions"])

        response = self.getCertificationWorkflow().createCertification(self.product, self.program, parameters)

        self.appendErrors(self.getCertificationWorkflow().getErrors())
        if not self.getErrors() and response:
            self.cweResponse = response
            # we're going to redirect, so no need to worry about downloading the new cert
            self.certificationID = self.cweResponse.get("cert_id")
            self.certificationNID = self.cweResponse.get("cert_nid")

            # upload cert attachments
            # Example: vnf certification workflow asks or to uploading director integration guide attachment
            # upload vnf configuration report during cert creation
            # upload director integration guide attachment during cert creation

            director_integration_file = str()
            if self.getParameter("director_integration_file"):
                # for management and other plugin
                director_integration_file = self.data['director_integration_file']
            elif 'director_file_content' in parameters:
                # file content for the following plugins:- "blockstorage", "dataprocessing", "filestorage", "networking", "objectstorage"
                director_integration_file = eval(parameters['director_file_content']) if parameters['director_file_content'] else ""

            if director_integration_file:
                self.fileUpload(response.get("cert_id"), parameters.get("director_isSpec"),
                                parameters.get("director_description"),
                                director_integration_file["name"], director_integration_file["data"],
                                director_integration_file["content_type"])

            chosenFile = self.getParameter("chosenFile")
            if chosenFile:
                self.fileUpload(response.get("cert_id"), parameters.get("isSpec"), parameters.get("description"),
                                chosenFile.name, chosenFile.read(), chosenFile.content_type)
            return False  # done

        self.data["redhatProducts"] = json.dumps(self.productNameService.getAllProducts())
        self.data["nextButtonValue"] = "done"
        self.data["createTitle"] = "New Certification"
        self.data["createDescription"] = "Choose the Red Hat certification."
        self.data["nextButtonLabel"] = "Next"
        self.template = "create/certification/all.html"
        return True # not done - errors

    def fileUpload(self, cert_id, isSpec, description, fileName, fileContent, content_type):
        cweFileResponse = self.getEngineWorkflow().uploadFileToCertification(cert_id, isSpec, description, fileName,
                                                                             fileContent, content_type)

        if not self.getEngineWorkflow().getErrors() and cweFileResponse:
            Alert(Alert.success, "Attachment uploaded successfully", self.request)
        else:
            Alert(Alert.danger, "Error in attachment upload", self.request)
            Alert(Alert.danger, self.getEngineWorkflow().getErrors(), self.request)

    def getVendorAndProduct(self):
        self.data["vendorID"] = self.vendorID
        self.data["productID"] = self.productID
        self.vendor = self.getConnection().getVendorProducts().getVendorByID(self.vendorID)
        if self.vendor:
            self.product = self.vendor.getProduct(self.productID)
        if self.product:
            self.data["vendorProduct"] = self.product
