# Copyright (c) 2017 Red Hat, Inc. All rights reserved. This copyrighted material
# is made available to anyone wishing to use, modify, copy, or
# redistribute it subject to the terms and conditions of the GNU General
# Public License v.2.
#
# This program is distributed in the hope that it will be useful, but WITHOUT ANY
# WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A
# PARTICULAR PURPOSE. See the GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
#
# Author: Greg Nichols
#
import json

from django.shortcuts import render
from django.http import HttpResponseRedirect

from rhcert.tags import Constants, Programs

from ..view import View
from rhcert.model.vendordocument import VendorDocument


class Create(View):
    """ abstract base class for object creation wizards """

    def __init__(self):
        View.__init__(self)
        self.loginRequired = True
        self.template = None
        self.program = None
        self.programs = [
                        {"name": Programs.hardware, "fullname": "Red Hat Enterprise Linux Certified Hardware"},
                        {"name": Programs.openstack, "fullname": "Red Hat OpenStack Platform"},
                        {"name": Programs.cloud, "fullname": "Red Hat Certified Cloud Service Provider"},
                        {"name": Programs.cloud_sap, "fullname": "Red Hat Enterprise Linux HA - Update Services for SAP Solutions Certified Cloud Service Provider"},
                        {"name": Programs.software, "fullname": "Red Hat Enterprise Linux Certified Software"}
                    ]
        self.local = False  # True if creating a local (not on CWE) object

    def serve(self, request):
        """ redefine the serve method to allow subclasses to pick the template """
        View.authenticate(self, request)
        result = self.view(request)
        if type(result) != dict:
            return result
        return render(request, self.template, self.data)

    def view(self, request):
        View.view(self, request)

        if self.getParameter("returnAfter"):
            self.data["returnAfter"] = self.getParameter("returnAfter")
        self.program = self.getParameter("program")
        if self.program:
            self.data["program"] = self.program
        self.data["parameters"] = dict()
        self.data["list_parameters"] = dict()
        for key, value in self.getPostDictionary().items():
            if key not in ["return-after", "next-button", 'cancelled', "csrfmiddlewaretoken"]:
                self.data["parameters"][key] = value
                if isinstance(value, list):
                    self.data["list_parameters"][key] = value

        # check if we're cancelled
        if self.getParameter("cancelled"):
            if self.getParameter("return-after"):
                return HttpResponseRedirect(self.getParameter("return-after"))
            else:
                return HttpResponseRedirect("/")

        return self.data

    def getProgramFullName(self):
        if not self.program:
            return None
        if self.program == Programs.software:
            return Programs.software
        elif self.program == Programs.cloud_sap:
            return Programs.cloud_sap
        programData = self.productNameService.getProgram(self.program)
        if not programData:
            return None
        return programData.get("name")

    def choosingProgram(self):
        self.program = self.getParameter("program")
        if self.program:
            self.data["program"] = self.program
            if self.getParameter("vendorID"):
                self.vendorID = self.getParameter("vendorID")
            # cloud currently has no version data - use "hardware" for the RHEL version
            if self.program == 'cloud':
                self.data["redhatProducts"] = json.dumps(self.productNameService.getProductTree('hardware', 1)) # just RHEL
            else:
                self.data["redhatProducts"] = json.dumps(self.productNameService.getProgram(self.program))
            return False

        self.data["createTitle"] = "Choose the Certification Program"
        self.data["nextButtonValue"] = "next"
        self.data["nextButtonLabel"] = "Next"
        self.template = "create/choose-program.html"
        if self.getListParameter("allowedPrograms"):
            self.data["programs"] = []
            # use the order from allowedPrograms for the selector
            for programName in self.getListParameter("allowedPrograms"):
                for program in self.programs:
                    if program["name"] == programName:
                        self.data["programs"].append(program)
        else:
            self.data["programs"] = self.programs
        # account for multiple vendors with the same name
        vendors = list()
        for vendor in self.getConnection().getVendorProducts().getVendors():
            if vendor.getWorkflow() == self.getParameter("workflow"):
                vendors.append(vendor)
        self.data["vendors"] = vendors
        vendorsByName = dict(zip(map(VendorDocument.getName, vendors), vendors))
        self.data["showVendorIDs"] = vendorsByName < vendors
        return True

    def choosingExistingProduct(self):
        if not (self.vendorID and self.productID):
            self.compileVendorProducts(self.getParameter("rh_product_id"), self.getParameter("workflow"))
            self.data["createTitle"] = "New Certification"
            self.data["createDescription"] = "Choose the product to be certified."
            self.data["nextButtonValue"] = "next"
            self.data["nextButtonLabel"] = "Next"
            self.template = "create/choose-product.html"

            # Disable the button to create New Product for Layered Certification
            if self.getParameter("rh_product_id") in ["5","6","8"]:
                self.data["disableNewProduct"] = True
            return True
        return False

    def isLayeredCertification(self, redhatProductID, program):
        if redhatProductID in ["5","6","8","18"]:
            # check if we have to create a layered cert
            # In rhcert local data model (info/redhat-products.json)
            # we use id to product mapping
            # rh_product_id | Product Name
            # ----------------------------
            #      3        | RHOSP Compute or Ready State
            #      5        | RHGS
            #      6        | RHELRT
            #      8        | RHHI
            #      18       | RHOSPforRT
            return True
        if (redhatProductID == "3" and program == Programs.hardware) or\
           (redhatProductID == "12" and program == Programs.hardware):
            return True, Constants.RHOSP
        return False

    def getVendorAndProduct(self):
        self.vendor = None
        self.product = None
        if self.vendorID:
            self.vendor = self.getConnection().getVendorProducts().getVendorByID(self.vendorID)
        if self.vendor and self.productID:
            self.product = self.vendor.getProduct(self.productID)
 

