# Copyright (c) 2017 Red Hat, Inc. All rights reserved. This copyrighted material
# is made available to anyone wishing to use, modify, copy, or
# redistribute it subject to the terms and conditions of the GNU General
# Public License v.2.
#
# This program is distributed in the hope that it will be useful, but WITHOUT ANY
# WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A
# PARTICULAR PURPOSE. See the GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
#
# Author: Rajasi Kulkarni
#
import json

from rhcert.tags import Programs, Constants
from .create import Create
from ..alert import Alert


class CreateLayeredCert(Create):
    """ presents the a UI page, showing comments on the certification"""

    def __init__(self):
        Create.__init__(self)

    def view(self, request):
        result = Create.view(self, request)
        if type(result) != dict:
            return result

        self.certificationID = self.getParameter("certificationID")
        if self.getParameter("base_cert_id"):
            # Here we are creating layered cert from the homepage and
            # have chosen the base cert to create a layert cert on
            self.certificationID = self.getParameter("base_cert_id")
        self.vendorID = self.getParameter("vendorID")
        self.productID = self.getParameter("productID")
        self.redhatProductID = self.getParameter("rh_product_id")
        self.redhatProductVersionID = self.getParameter("rh_product_version_id")
        self.layeredProductFormat = self.getParameter("rh_product_version_format")
        self.redhatBaseCertVersionID = self.getParameter("rh_base_version_id")
        self.program = Programs.hardware

        choosing_layered = self.choosingLayeredCertification()
        if choosing_layered:
            if not isinstance(choosing_layered, bool):
                return choosing_layered
            return self.data

        if self.choosingOriginalCertification():
            return self.data

        # otherwise - done!  create it
        response = self.getCertificationWorkflow().createLayeredCertification(
                                                                         self.certificationID,
                                                                         self.redhatProductID,
                                                                         self.redhatProductVersionID,
                                                                         self.layeredProductFormat)
        if not response or self.getCertificationWorkflow().getErrors():
            Alert(Alert.danger, "Error in creating Layered Certification", request)
            self.appendErrors(self.getCertificationWorkflow().getErrors())
            if self.getParameter("base_cert_id"):
                return self.redirect("certifications")
            return self.redirect("related", queryDict=None, paramList=[Programs.hardware, self.certificationID])

        # otherwise
        layeredCertificationID = response.get("cert_id")
        Alert(Alert.success, "Created Layered Certification with ID : " + str(layeredCertificationID), request)
        return self.redirect("certification", queryDict=None, paramList=[Programs.hardware,  layeredCertificationID])

    def choosingLayeredCertification(self):
        if self.redhatProductID and self.redhatProductVersionID:
            return False # not choosing

        # otherwise, choosing
        self.template = "create/choose-layered-cert.html"
        redhatProducts = self.__patchIDsInProducts()
        if not redhatProducts:
            if not self.getParameter("base_cert_id"):
                Alert(Alert.danger, "Error: Additional layered certs cannot be created on this certification", self.request)
                return self.redirect("related", queryDict=None, paramList=[Programs.hardware, self.certificationID])
        self.data["redhatProducts"] = json.dumps(redhatProducts)
        self.data["nextButtonValue"] = "done"
        self.data["createTitle"] = "New Layered Certification"
        self.data["createDescription"] = "Choose the Red Hat Certification."
        self.data["nextButtonLabel"] = "Create"
        self.appendErrors(self.getCertificationWorkflow().getErrors())
        return True

    def __patchIDsInProducts(self):
        """ set id to rh_product_id and rh_product_version_id so it can be used in the redhat-product-picker.html """
        products = self.getCertificationWorkflow().getLayeredProducts(self.certificationID)
        if not products:
            return None
        for product in products:
            product["id"] = product["rh_product_id"]
            for productVersion in product.get("versions"):
                if productVersion.get("rh_product_version_id"):
                    productVersion["id"] = productVersion["rh_product_version_id"]
        return {"products": products}

    def choosingOriginalCertification(self):
        if self.certificationID and self.vendorID and self.productID:
            return False # not choosing

        # otherwise, choosing
        check_RHOSP = self.isLayeredCertification(self.getParameter("rh_product_id"), self.program)
        if isinstance(check_RHOSP, tuple) and check_RHOSP[1] == Constants.RHOSP:
            self.data["create_RHOSP_layered"] = True
        self.__populateOriginalCerts()
        return True

    def __populateOriginalCerts(self):
        # Step1: Get all the certifications for the selected Product
        baseCertifications = self.getCertificationWorkflow().getCertifications(self.productID, self.program)
        self.appendErrors(self.getCertificationWorkflow().getErrors())
        if not baseCertifications:
            self.appendAlert(Alert(Alert.danger,
                                   "Error: could not get the certifications for selected product\
                                   to create a Layered certification. Select a different product",
                                   self.request))
            self.productID = None
            return self.choosingExistingProduct()

        # check which of the base certifications can support creating the required layered cert
        supported_base_certs = list()
        in_progress_supported_base_certs = False
        all_supported_base_certs_in_progress = True
        for cert in baseCertifications:
            cert_supports_layered = False
            if not cert.get('rh_product_version_id'):
                # Currently for RHHI certs there is no rh_product_version_id
                continue

            if self.redhatBaseCertVersionID and str(cert['rh_product_version_id']) != self.redhatBaseCertVersionID:
                # Check that the base version of required layered cert
                # is equal to the base cert's product version
                continue

            available_product_versions = self.getCertificationWorkflow().getLayeredProducts(cert['cert_id'])

            if not available_product_versions:
                # The base cert does not support creating layered certs
                continue

            for available_product in available_product_versions:
                if str(available_product['rh_product_id']) == self.redhatProductID:
                    for version in available_product['versions']:
                        if str(version['rh_product_version_id']) == self.redhatProductVersionID:
                            # This cert supports creating layered Certificate with
                            # required product type and version
                            cert_supports_layered = True
                            if self.data.get("create_RHOSP_layered") and\
                               self.data["create_RHOSP_layered"]:
                                # For RHOSP currently either format is 'Compute' or 'Ready State'
                                # never both
                                # So the length of the version list is always 1
                                cert['RHOSP_supported_format'] = version['formats'][0]['name']
                            supported_base_certs.append(cert)
                            if cert['status'] == "In Progress":
                                in_progress_supported_base_certs = True
                            else:
                                all_supported_base_certs_in_progress = False
                            break # Dont need to check for the other version of this product
                if cert_supports_layered:
                    break # Dont need to check for the other products

        warn_msg = "In Progress certifications need to be Certified first, to enable creating layered certifications on them"

        if len(supported_base_certs) == 0:
            Alert(Alert.danger,
                  "No RHEL Certification of this product supports the required layered Certification.\
                  Select a different product",
                  self.request)
            self.productID = None
            return self.choosingExistingProduct()

        elif in_progress_supported_base_certs:
            # Atleast one base cert is in progress
            if all_supported_base_certs_in_progress:
                # Disable the Create button when all certs are in progress
                self.data["disable_next_button"] = True
                Alert(Alert.warning, warn_msg, self.request)
            else:
                Alert(Alert.warning,
                      "Only Certified certifications can be chosen. " + warn_msg,
                      self.request)

        self.data["supported_base_certs"] = supported_base_certs
        self.data["createTitle"] = "New Certification"
        self.data["createDescription"] = "Choose the base RHEL Certification for creating the layered Certification."
        self.data["nextButtonValue"] = "create-certification-layered"
        self.data["nextButtonLabel"] = "Create"
        self.template = "create/choose-cert-for-layering.html"
        return True


