# Copyright (c) 2017 Red Hat, Inc. All rights reserved. This copyrighted material
# is made available to anyone wishing to use, modify, copy, or
# redistribute it subject to the terms and conditions of the GNU General
# Public License v.2.
#
# This program is distributed in the hope that it will be useful, but WITHOUT ANY
# WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A
# PARTICULAR PURPOSE. See the GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
#
# Author: Greg Nichols
#

from rhcert.tags import Constants, ServerConfiguration, SystemCategories

from .create import Create
from ..alert import Alert


class Product(Create):
    """ creates a new certification on either a new or existing vendor product """

    def __init__(self):
        Create.__init__(self)
        self.cweResponse = None
        self.vendor = None

    def view(self, request):
        result = Create.view(self, request)
        if type(result) != dict:
            return result

        self.request = request
        # if self.data["errors"]:
        #    return self.data
        self.vendorID = self.getParameter("vendorID")
        self.productID = self.getParameter("productID")

        if self.creatingProduct():
            return self.data

        # otherwise, done!
        if self.confirming():
            return self.data

        if self.getParameter("productID") and self.getParameter("program"):
            return self.redirect("product", queryDict=None,
                        paramList=[self.getParameter("vendorID"),  self.getParameter("productID")])
        # otherwise
        return self.redirect("products")

    def creatingProduct(self):
        if not self.getParameter("new-product"):  # still creating
            return False

        if self.choosingProgram():
            self.data["parameters"]["new-product"] = "new-product"
            return True # not done - choosing program

        if self.isLayeredCertification(self.getParameter("rh_product_id"), self.program):
            # New Product selected for creating a RHOSP hardware layered cert
            self.template = "create/product-layered-%s.html" % self.program
        else:
            self.data["systemCategories"] = SystemCategories.getAll()
            self.template = "create/product-%s.html" % self.program

        self.data["createTitle"] = "Define the %s Vendor Product" % self.getProgramFullName()
        self.data["nextButtonValue"] = "create-product"
        self.data["nextButtonLabel"] = "Next"
        self.vendorID = self.getParameter("vendorID")
        self.vendor = self.getConnection().getVendorProducts().getVendorByID(self.vendorID)
        self.data["vendorName"] = self.vendor.getName()

        if self.getParameter("next-button") != "create-product": # create product form not submitted
            self.data["parameters"]["new-product"] = "new-product"
            self.data["parameters"].pop("vendor_id") # git rid of stale value from product chooser
            return True # not done yet

        # otherwise actually create the product in CWE
        if not self.createProduct():
            # errors!
            return True # not done - errors!
        return False

    def createProduct(self):
        self.vendor = self.getConnection().getVendorProducts().getVendorByID(self.getParameter("vendorID"))
        params = self.getPostDictionary()
        if params.get("supported_regions"):
            if isinstance(params["supported_regions"], list):
                params["supported_regions"] = ','.join(params["supported_regions"])

        self.cweResponse = self.getVendorProductWorkflow().createProduct(self.vendor, self.program, params)
        self.appendErrors(self.getVendorProductWorkflow().getErrors())
        self.data["cweDialogResponse"] = self.cweResponse
        if not self.getErrors() and self.cweResponse:
            self.data["cweDialogConfirm"] = True
            self.vendorID = self.vendor.getID()
            self.data["parameters"]["vendorID"] =  self.vendorID
            self.productID = self.cweResponse.get("vendor_product_id")
            self.data["parameters"]["productID"] = self.productID
            Alert(Alert.success,
                  "Created new Product for Program '" + self.getProgramFullName() + "' with ID : " + str(self.productID),
                  self.request)

            # Upload product logo
            self.logo = self.getParameter("logo")
            if self.logo:
                self.cweResponseLogo = self.getVendorProductWorkflow().uploadProductLogo(self.program,
                                                                     self.cweResponse.get("vendor_product_nid"),
                                                                     self.logo.content_type,
                                                                     self.logo.read())
                if self.getErrors() or not self.cweResponseLogo:
                    Alert(Alert.danger, "Error in uploading product logo", self.request)
                    Alert(Alert.danger, self.getVendorProductWorkflow().getErrors(), self.request)

            return self.__downloadProduct(self.vendor, self.cweResponse.get("vendor_product_id"))

        Alert(Alert.danger, "Error in creating new Product", self.request)
        return False # errors

    def __downloadProduct(self, vendorObject, productID):
        catalogProducts = self.getVendorProductWorkflow().list()
        self.appendErrors(self.getVendorProductWorkflow().getErrors())
        if self.getErrors() and not catalogProducts:
            return False
        catalogProduct = None
        for product in catalogProducts:
            if str(product.get("product_id")) == str(productID):
                catalogProduct = product
                break
        if not catalogProduct:
            self.appendError("Error: could not download new product ID %s" % productID)
            return False

        # RHCWE-2363 - vendor name ambiguity means the vendor ID may be different than requested
        newProductVendor = self.getConnection().getVendorProducts().getVendorByID(catalogProduct.get("vendor_id"))
        if not newProductVendor:
            self.appendError("Error: could not locate new product vendor ID %s" % catalogProduct.get("vendor_id"))
            return False
        if vendorObject.getID() != newProductVendor.getID():
            # print "replaced vendor %s ID %s with ID %s" % (vendorObject.getName(), vendorObject.getID(), newProductVendor.getID())
            vendorObject = newProductVendor
            self.vendorID = newProductVendor.getID()
        product = vendorObject.createProduct(catalogProduct.get("product_name"), productID)
        product.setMake(catalogProduct.get("make"))
        if catalogProduct.get("bug-id"):
            product.setCertificationID(catalogProduct.get("bug-id"))
        product.setNodeID(catalogProduct.get("product_nid"))
        product.setProgram(self.program)
        product.setWorkflow(Constants.cwe)
        for setAttribute, attribute in [(product.setName, "product_name"), (product.setMake, "make")]:
                if catalogProduct.get(attribute):
                    setAttribute(catalogProduct.get(attribute))

        if self.environment.getServerConfiguration() == ServerConfiguration.default:
            self.getStore().getVendorProducts().save()
        elif self.environment.getServerConfiguration() == ServerConfiguration.management:
            self.getConnection().resetVendorProducts()
        return True

    def confirming(self):
        if self.getParameter("next-button") == "confirmed":
            return False
        # otherwise, ask for confirmation
        self.data["cweDialogResponse"] = self.cweResponse
        self.data["programFullName"]  = self.getProgramFullName()
        self.data["parameters"]["program"] = self.program
        if self.cweResponse:
            self.data["parameters"]["productID"] = self.cweResponse.get("vendor_product_id")
            self.data["parameters"]["productNodeID"] = self.cweResponse.get("vendor_product_nid")
        self.data["createTitle"] = "Created Vendor Product"
        self.data["nextButtonValue"] = "confirmed"
        self.data["nextButtonLabel"] = "Confirm"
        self.data["showCertificationConfirm"] = True
        self.data["message"] = None # hide redundant message alert
        return True
