# Copyright (c) 2017 Red Hat, Inc. All rights reserved. This copyrighted material
# is made available to anyone wishing to use, modify, copy, or
# redistribute it subject to the terms and conditions of the GNU General
# Public License v.2.
#
# This program is distributed in the hope that it will be useful, but WITHOUT ANY
# WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A
# PARTICULAR PURPOSE. See the GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
#
# Author: Greg Nichols
#

from rhcert.tags import Constants, Tags
from rhcert.model.certificationdocument import CertificationDocument

from ..alert import Alert
from .create import Create
from .certification import Certification


class Sandbox(Certification):
    """ creates a new sandbox  """

    def __init__(self):
        Certification.__init__(self)
        self.loginRequired = False

    def view(self, request):
        result = Create.view(self, request)
        self.request = request
        if type(result) != dict:
            return result

        if self.choosingRedHatCertification():
            return self.data

        if self.choosingProgram():
            return self.data

        self.createLocalProduct()

        if self.creatingCertification():
            return self.data

        return self.redirect("sandbox", queryDict={}, paramList=[self.program, self.sandboxID])

    def choosingRedHatCertification(self):
        if Certification.choosingRedHatCertification(self):
            # revise the title and description when choosing the RH product cert
            self.data["createTitle"] = "New Sandbox"
            self.data["createDescription"] = "Choose the Red Hat certification for sandbox testing."
            return True
        return False

    def choosingProgram(self):
        if self.getParameter("program"):
            self.program = self.getParameter("program")
            return False

        # get a list of allowed vendor product programs based on the RH certification:
        allowedPrograms = self.productNameService.getAllowedPrograms(self.getParameter("rh_product_id"))
        if allowedPrograms:
            if len(allowedPrograms.get("programs")) == 1:
                self.program = allowedPrograms.get("programs")[0]
                self.data["program"] = self.program
                self.data["createTitle"] = "Name the Sandbox"
                self.template = "create/choose-sandbox-name.html"
            else:
                # otherwise, choose
                self.data["allowedProgramsDescription"] = allowedPrograms.get("description")
                allowedPrograms = allowedPrograms.get("programs")
                self.data["allowedPrograms"] = allowedPrograms
                self.data["createTitle"] = "Choose the Certification Program"
                self.template = "create/choose-program.html"
        self.data["nextButtonValue"] = "next"
        self.data["nextButtonLabel"] = "Next"
        return True

    def createLocalProduct(self):
        """ override create.Product.createProduct to create a local product as a sandbox name"""
        vendorProducts = self.getStore().getVendorProducts()
        self.vendor = vendorProducts.getRecentLocalVendor()
        sandboxName = self.getParameter("new-sandbox-name")
        if not sandboxName:
            sandboxName = "Sandbox"
        if not self.vendor:
            self.vendor = vendorProducts.createVendor("Sandbox Vendor", id=-1)
            self.vendor.setWorkflow(Constants.local)
        self.vendorProduct = self.vendor.getProductByName(sandboxName)
        if not self.vendorProduct or self.vendorProduct.getProgram() != self.program:
            self.vendorProduct = self.vendor.createProduct(sandboxName)
            self.vendorProduct.setProgram(self.program)
            vendorProducts.save()
        self.vendorID = self.vendor.getID()
        self.productID = self.vendorProduct.getID()

    def creatingCertification(self):
        """ override just to add Sandbox to the title """
        if Certification.creatingCertification(self) and self.getParameter("next-button") != "done":
            self.data["createTitle"] += " Sandbox"
            self.data["nextButtonValue"] = "done"
            self.data["nextButtonLabel"] = "Create"
            self.template = "create/certification/%s.html" % self.program
            return True
        return False

    def createCertification(self):
        """ override to create a local Sandbox """
        self.sandboxID = None

        certification = CertificationDocument()
        certification.new()
        certification.setCertificationProgram(self.program)
        try:
            (redhatProduct, redhatProductVersion, redhatBaseVersion) = self.productNameService.getProductPath(
                    self.getParameter("rh_product_id"),
                    self.getParameter("rh_product_version_id"),
                    self.getParameter("rh_base_version_id"))

        except Exception as error:
            alert = Alert(Alert.danger, "Error: could not determine product information:", self.request)
            alert.appendMessage("%s" % error)
            self.appendAlert(alert)
            return True # not done - errors

        path = certification.getProductCertification()
        redhatVendorDocument = path.get(Tags.vendor)
        redhatProductDocument = redhatVendorDocument.createProduct(redhatProduct.get("name"), redhatProduct.get("id"))
        if redhatProduct.get("shortName"):
            redhatProductDocument.setShortName(redhatProduct.get("shortName"))
        productVersionDocument = redhatProductDocument.createVersion()
        productVersionDocument.setVersion(redhatProductVersion.get("name"))
        productVersionDocument.setID(redhatProductVersion.get("id"))
        if not redhatBaseVersion:
            certification.getRequiredOS().setProduct(redhatProductDocument.getName())
            certification.getRequiredOS().setRelease(productVersionDocument.getVersion())
        else:
            certification.getRequiredOS().setProduct(Constants.Red_Hat_Enterprise_Linux)
            certification.getRequiredOS().setRelease(redhatBaseVersion.get("name"))


        certification.setCertificationID(self.getStore().makeScratchCertificationID())
        certification.setCertificationState(Constants.local) # probably redundant
        certification.setSandbox()

        self.getVendorAndProduct()
        vendor = certification.setVendor(self.vendor)
        vendor.setProduct(self.product)

        self.getStore().saveCertification(certification)
        self.sandboxID = certification.getCertificationID()
        self.appendAlert(Alert(Alert.info, "Created %s Sandbox ID %s" % (self.program, self.sandboxID), self.request))

        return False # done


