# Copyright (c) 2014 Red Hat, Inc. All rights reserved. This copyrighted material
# is made available to anyone wishing to use, modify, copy, or
# redistribute it subject to the terms and conditions of the GNU General
# Public License v.2.
#
# This program is distributed in the hope that it will be useful, but WITHOUT ANY
# WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A
# PARTICULAR PURPOSE. See the GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
#
# Author: Greg Nichols <gnichols@redhat.com>
#
# Implementation of the wsgi for deleting test results/certification documents

from django.shortcuts import render
from django.http import HttpResponseRedirect

from rhcert.tags import Tags
from rhcert.rhcertStore import FileType
from .view import View


class Delete(View):
    def __init__(self):
        View.__init__(self)
        self.fileType = None
        self.document = None
        self.certificationID = None
        self.certificationProgram = None

    def serve(self, request, template):
        View.authenticate(self, request)

        result = self.view(request)
        if type(result) != dict:
            return result
        # otherwise
        return render(request, template, result)

    def view(self, request):
        View.view(self, request)
        self.path = self.getParameter("rpath")
        self.certificationID = self.getParameter("id")
        self.certificationProgram = self.getParameter("type")
        self.productID = self.getParameter("productID")
        self.vendorID = self.getParameter("vendorID")

        if self.path:
            error = ""
            (self.fileType, self.document) = self.getStore().getCertificationInfo(self.path, error)
            if error:
                self.appendError(error)

        elif self.certificationID and self.certificationProgram:
            self.fileType = FileType.certification
            (self.path, self.document) = self.getStore().getCertification(self.certificationProgram, self.certificationID)
            if not self.document:
                self.appendError("Error: could not locate certification")
                return self.data

        elif self.productID and self.vendorID:
            self.fileType = FileType.product
            self.vendor = self.getStore().getVendorProducts().getVendorByID(self.vendorID)
            if self.vendor:
                self.product = self.vendor.getProduct(self.productID)

        if "confirmed" in self.fieldStorage:
            if not self.delete():
                return self.data # show error(s)

        if "cancelled" in self.fieldStorage or "confirmed" in self.fieldStorage:
            if self.fileType in [FileType.certification, FileType.product, FileType.results]:
                if self.getParameter("return-after"):
                    return HttpResponseRedirect(self.getParameter("return-after"))
                else:
                    return HttpResponseRedirect("/")
            # otherwise
            return self.redirect(FileType.certification, queryDict=None, 
                    paramList=[self.document.getCertificationProgram(),  self.document.getCertificationID()])

        # otherwise
        self.compileConfirmForm()
        return self.data
    
    def compileConfirmForm(self):
        if self.fileType:
            self.data["fileType"] = self.fileType
            if self.path:
                self.data["path"] = self.path
            fileData = dict()
            self.data[self.fileType] = fileData

            if self.fileType == FileType.results:
                self.addHardwareAndOSValues(self.document, fileData)
                fileData["runTime"] = self.document.getRunTime()
            if self.fileType == FileType.multihost:
                self.addHardwareAndOSValues(self.document, fileData)
                fileData["runTime"] = self.document.getMultiHostRunTime()
            elif self.fileType == FileType.certification:
                fileData["id"] = self.document.getCertificationID()
                fileData["type"] = self.document.getCertificationProgram()
                fileData["isSandbox"] = self.document.isSandbox()
            elif self.fileType == FileType.product:
                self.data["vendor"] = self.vendor
                self.data["product"] = self.product

        if self.getParameter("return-after"):
            self.data["returnAfter"] = self.getParameter("return-after")

    def delete(self):
        if self.fileType == FileType.results:
            return self.getStore().deleteResults(self.path)
        if self.fileType == FileType.multihost:
            return self.getStore().deleteMultiHostResults(self.path)
        if self.certificationID and self.certificationProgram:
            return self.getStore().deleteCertification(self.certificationProgram, self.certificationID)
        if self.fileType == FileType.product:
            self.product.delete()
            self.getStore().getVendorProducts().save()
            return True
        # otherwise
        return False

