# Copyright (c) 2014 Red Hat, Inc. All rights reserved. This copyrighted material
# is made available to anyone wishing to use, modify, copy, or
# redistribute it subject to the terms and conditions of the GNU General
# Public License v.2.
#
# This program is distributed in the hope that it will be useful, but WITHOUT ANY
# WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A
# PARTICULAR PURPOSE. See the GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
#
# Author: Greg Nichols <gnichols@redhat.com>
#

import os, mimetypes

from django.http import HttpResponse, StreamingHttpResponse
from django.shortcuts import render

from rhcert.tags import Attributes
from rhcert.model.resultsdocument import ResultsDocument
from wsgiref.util import FileWrapper

from .view import View


class Download(View):
    def __init__(self):
        View.__init__(self)
        self.xslFilterPath = os.path.join(self.environment.getServerDirectory(), "xsl/attachment.xsl")
        self.document = None

    def validateParameters(self):

        if not self.getParameter("rpath"):
            self.appendError("Error: Download Failed ! No download path specified")
            return None

        if self.getParameter("rpath"):
            file_path = os.path.realpath(self.getParameter("rpath"))
            if os.path.commonprefix([file_path, self.getStore().getStoreDirectory()]) != self.getStore().getStoreDirectory():
                self.appendError("Error: Download Failed ! Specified download path not under store directory")
                return None

        if self.getParameter("attachment"):
            if not self.getParameter("test") or not self.getParameter("run"):
                self.appendError("Error: attachment download missing test or run information")
                return None

            xsltParameters = dict()
            xsltParameters["path"] = "\'%s\'" % self.getParameter("test")
            xsltParameters["run"] = "\'%s\'" % self.getParameter("run")
            xsltParameters["attachment"] = "\'%s\'" % self.getParameter("attachment")
            if self.getParameter('roleID'):
                xsltParameters["roleID"] = "\'%s\'" % self.getParameter("roleID")
            self.document = ResultsDocument()
            self.document.loadFiltered(self.getParameter("rpath"), self.xslFilterPath, xsltParameters)

            test = self.document.getTest(self.getParameter("test"))
            if not test:
                self.appendError("Error: test %s not found\n" % self.getParameter("test"))
                return None
            run = test.getRun(int(self.getParameter("run")))
            if not run:
                self.appendError("Error: run %s not found\n" % self.getParameter("run"))
                return None

            for attachment in run.getAttachments():
                if attachment.getAttribute(Attributes.name) == self.getParameter("attachment"):
                        return attachment

            self.appendError("Error: %s not found\n" % self.getParameter("attachment"))
            return None

        # else: # download the whole thing
        return None

    def serve(self, request, template):
        View.authenticate(self, request)
        result = self.view(request)
        if type(result) != dict:
            return result
        # otherwise
        return render(request, template, result)

    def view(self, request):
        View.view(self, request)
        attachment = self.validateParameters()
        if self.getErrors():
            return self.data # show error page

        # account for zero-length files
        length = 0
        data = ""
        chunkSize = 8192

        # otherwise
        if attachment:
            response = HttpResponse()
            (mimeType, encoding) = (attachment.getAttribute(Attributes.mime_type), attachment.getAttribute(Attributes.encoding))
            if mimeType != "text/html":
                response["Content-Description"] = "File Transfer"
                response["Content-Disposition"] = str("attachment; filename=\"%s\"" % attachment.getAttribute(Attributes.name))
            if attachment.childNodes:
                base64data = attachment.childNodes[0].data
                if base64data:
                    data = base64data.decode("base64")
                    length = len(data)
                    response.write(data)
            response['Content-Type']= mimeType
            if encoding:
                response["Content-Encoding"] = encoding
            response['Content-Length'] = length
            return response

        # otherwise - download the whole thing
        filePath = os.path.realpath(self.getParameter("rpath"))
        filename = os.path.basename(filePath)
        (mimeType, encoding) = mimetypes.guess_type(filename)
        response = StreamingHttpResponse(FileWrapper(open(filePath), chunkSize), content_type=mimeType)
        response["Content-Disposition"] =  "attachment; filename=\"%s\"" % filename
        response['Content-Length'] = os.path.getsize(filePath)

        return response



