# Copyright (c) 2014 Red Hat, Inc. All rights reserved. This copyrighted material 
# is made available to anyone wishing to use, modify, copy, or 
# redistribute it subject to the terms and conditions of the GNU General 
# Public License v.2.
# 
# This program is distributed in the hope that it will be useful, but WITHOUT ANY
# WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A
# PARTICULAR PURPOSE. See the GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
#
# Author: Greg Nichols <gnichols@redhat.com>
#

import os, cgi, magic, time
from rhcert.compatability import XmlrpcServerProxy

from rhcert.tags import Constants, Tags
from rhcert.model.task import Task
from rhcert.host import Host

from .view import View
from .run import Run


class Interactive(Run):
    def __init__(self):
        Run.__init__(self)
        self.hostStatus = None
        self.hostInfo = None
        self.resultsPath = None
        self.testHost = None

    def view(self, request):
        View.view(self, request)

        self.validateParameters()

        if self.getParameter("restart"):
            self.restartHost(Host(self.getParameter("host"), self.getParameter("connection")))
            if not self.getErrors():
                if self.certification.isSandbox():
                    return self.redirect("sandbox", queryDict=None, paramList=[self.certificationProgram,  self.certificationID])
                else:
                    return self.redirect("certification", queryDict=None, paramList=[self.certificationProgram,  self.certificationID])

        tests = self.getRequest()
        if tests:
            self.requestRun(tests)
            if not self.getErrors():
                return self.redirect("interactive", {"host": self.testHost.getName(), "connection": self.testHost.getConnection()})

        (self.hostStatus, self.hostInfo) = self.getHostStatus(self.testHost)
        if self.hostStatus == "task" and self.hostInfo.getTask() != Task.message:
            answerSent = self.sendAnswer()
            if not self.getErrors() and answerSent:
                return self.redirect(self.parentURL, queryDict=None, paramList=[self.certificationProgram,  self.certificationID])
                # return self.redirect("interactive", {"host": self.testHost.getName(), "connection": self.testHost.getConnection()})

        # return to cert/testing if done
        message = self.getStatusMessage()
        if message and ("Finished test run" in message):
            return self.redirect(self.parentURL, queryDict=None, paramList=[self.certificationProgram,  self.certificationID])

        # otherwise
        self.compileView()
        return self.data

    def validateParameters(self):
        # try results path 
        if self.getParameter("rpath") and os.path.exists(self.getParameter("rpath")):
            self.resultsPath = self.getParameter("rpath")
            self.resultsDocument = self.getStore().getResultsDocument(self.getParameter("rpath"))
            self.testHost = Host(self.resultsDocument.getOS(Tags.hostname), ipAddress=self.resultsDocument.getOS(Tags.ip_address))
            self.certificationID = self.resultsDocument.getCertificationID()
            self.certificationProgram = self.resultsDocument.getCertificationProgram()

        # otherwise, try host/task
        elif self.getParameter("host"):
            # use the hostname to get certID and Type
            self.testHost = Host(self.getParameter("host"), self.getParameter("connection"))
            (self.hostStatus, self.hostInfo) = self.getHostStatus(self.testHost)
            # if the host is on a task, get the document from the task
            if self.hostStatus == "task":
                self.certificationID = self.hostInfo.getParameter(Task.certificationID)
                self.certificationProgram =  self.hostInfo.getParameter(Task.certificationProgram)
                if self.certificationID and self.certificationProgram:
                    (self.resultsDocument, self.resultsPath) = self.getStore().getLatestResultsDocument(self.testHost,
                                                        self.certificationProgram,
                                                        self.certificationID)
                else:
                    self.appendErrors("Error: Could not find certification type and id from host")
        else:
            self.certificationID = self.getParameter("certificationID")
            self.certificationProgram = self.getParameter("certificationProgram")

        (_, self.certification) = self.getStore().getCertification(self.certificationProgram,
                                                                   self.certificationID)
        if self.testHost:
            self.testHostURL = 'http://%s:%s' % (self.testHost.getConnection(), self.environment.getServerListenerPort())
            self.server = XmlrpcServerProxy(self.testHostURL, allow_none=True)

        # compile view dictionary so far
        if self.testHost:
            self.data["host"] = self.testHost
        if self.resultsPath:
            self.data["path"] = self.resultsPath
        if self.certificationID:
            self.data["certification"] = {
                    "id": self.certificationID,
                    "type": self.certificationProgram
            }

        if self.hostStatus == "task":
            self.data["task"] = dict()
            self.data["task"]["type"] = self.hostInfo.getParameter("type")
            if self.hostInfo.getTask() == Task.message:
                self.data["task"]["message"] = self.hostInfo.getParameter(Task.message)

        self.setParentURL()

    def getRequest(self):
        # submit button name is "interactive:<test name> [--<parameter> <value>]*"
        tests = list()
        for parameter in self.fieldStorage:
            if type(parameter) is not list:
                parts = parameter.split(":", 1)
                if len(parts) > 1 and parts[0] == "interactive":
                    testSpec = self.parseTestOption(parts[1])
                    if testSpec:
                        tests.append(testSpec)
        return tests

    def getStatusMessage(self):
        # status messages can either be plain text, or wrapped in an XML Task
        if self.hostStatus == "status":
            return self.hostInfo
        if self.hostStatus == "task" and self.hostInfo.getTask() == Task.message:
            return self.hostInfo.getParameter(Task.message)
        return None

    def sendAnswer(self):
        task = self.hostInfo
        # for debugging
        # self.body.write("<pre>%s</pre>" % task.toXML())

        # did we get a response from the user?
        answer = True
        if self.getParameter("prompt-answer"):
            if task.getParameter(Task.type) == Task.type.integer:
                try:
                    int(self.getParameter("prompt-answer"))
                except:
                    self.appendError("Please enter an integer")
                    return False
            if task.getParameter(Task.type) == Task.type.multi_select:
                checkboxes = list()
                try:
                    for checkbox in self.getListParameter("prompt-answer"):
                        checkboxes.append(checkbox)
                except TypeError:
                    checkboxes.append(self.getParameter("prompt-answer"))
                task.setParameter(Task.response, self.htmlUnescape(" ".join(checkboxes)))
            else:
                task.setParameter(Task.response, self.htmlUnescape(self.getParameter("prompt-answer")))
        elif self.getParameter("file-answer"):
            destinationPath = self.saveUploadedFile("file-answer")
            if self.getErrors():
                answer = False
            else:
                task.setParameter(Task.response, destinationPath)
        else:
            answer = False

        if answer:
            try:
                self.result = self.server.run(task.toXML())
                if not self.result:
                    self.appendError("Error: Sending prompt answer Failed.")

            except Exception as exception:
                self.appendError("Error: call to host %s failed:\n" % self.testHostURL)
                self.appendError("%s\n" % exception)
                self.appendError("%s\n" % self.resultsDocument.document.toxml())

            # refresh host status
            (self.hostStatus, self.hostInfo) = self.getHostStatus(self.testHost)
        return answer

    def saveUploadedFile(self, fileField):
        path = ""
        try:
            # determine incoding
            data = self.fieldStorage[fileField].value
            m = magic.open(magic.MAGIC_MIME_ENCODING)
            m.load()
            encoding = m.buffer(data)
            if "base64" in encoding:
                data = data.decode("base64")
            path = self.rhcertStore.saveTransferFile(self.fieldStorage[fileField].filename, data, self.getClientIP())
        except Exception as e:
            self.appendError("Error: could not save uploaded file:\n")
            self.appendError("%s" % e)
            self.appendError("encoding: %s" % encoding)
        return path

    def compileView(self):
        message = self.getStatusMessage()
        if message:
            self.data["message"] = message
            return True

        # otherwise
        if self.hostStatus == "task":
            task = self.hostInfo
            self.data["task"] = task.getParametersDictionary()
            if not "default" in self.data["task"]:
                self.data["task"]["default"] = ""
            if self.data["task"]["default"].count('\n') > 1:
                self.data["task"]["multiline"] = self.data["task"]["default"].count('\n')

    def restartHost(self, host):
        self.getHostStatus(host) # setup XMLRPC server (self.server)
        try:
            self.result = self.server.restart()
            time.sleep(5)
            if not self.result:
                self.appendError("Error: Host stop request failed for %s" % host.getConnection())

        except Exception as exception:
            self.appendError("Error: Host stop Request Failed for %s\n" % host.getConnection())
            self.appendError("%s\n" % exception)



