# Copyright (c) 2014 Red Hat, Inc. All rights reserved. This copyrighted material
# is made available to anyone wishing to use, modify, copy, or
# redistribute it subject to the terms and conditions of the GNU General
# Public License v.2.
#
# This program is distributed in the hope that it will be useful, but WITHOUT ANY
# WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A
# PARTICULAR PURPOSE. See the GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
#
# Author: Greg Nichols
#
import os
import time
import json
from django.shortcuts import render
from django.conf import settings

from rhcert.compatability import XmlrpcServerProxy
from rhcert.tags import Tags, Constants, Sync, Programs, Operations, ServerConfiguration
from rhcert.model.task import Task
from rhcert.host import Host
from rhcert.model.resultsdocument import ResultsDocument
from rhcert.client.client import RHCertClient
from wsgi.view import View
from wsgi.test import Test


class Systems(View):

    def __init__(self):
        View.__init__(self)
        self.operations = Operations.getAll()

    def serve(self, request, template):
        View.authenticate(self, request)
        result = self.view(request)
        if type(result) != dict:
            return result
        return render(request, template, self.data)

    def view(self, request):
        View.view(self, request)

        self.vendorProducts = self.getStore().getVendorProducts()
        self.serverSettings = self.getStore().getServerSettings()

        if settings.CONFIGURATION == ServerConfiguration.management:
            for operation in self.operations:
                if self.getParameter(operation):
                    self.appendError("Error: Operation '%s'not allowed" % operation)
        else:
            if self.getParameter("validator"):
                if not self.getParameter("new-system"):
                    self.data["showMissingInfo"] = True
                else:
                    self.addSystem()

            elif self.getParameter("restart"):
                self.restartHost()
            elif self.getParameter("remove"):

                filePath = self.getParameter("remove")
                self.data["filePath"] = filePath
                systemDocument = ResultsDocument()
                systemDocument.load(filePath)
                self.data["hostname"] = systemDocument.getHostOS().getName()
                self.data["ipAddress"] = systemDocument.getHostOS().getIPAddress()
                self.getStore().removeSystem(filePath)

            elif self.getParameter("toggle-debug"):
                value = Constants.off
                if self.getParameter("debug"):
                    value = self.getParameter("debug")
                self.serverSettings.setDebug(value)
                self.data["rhcert"]["debug"] = value
                self.serverSettings.save()

            elif self.getParameter("reset-sync"):
                self.serverSettings.setSyncStatus(Sync.reset)
                self.serverSettings.save()
            elif self.getParameter("cwe-sync-date-range"):
                if self.getParameter("cwe-sync-date-range") != self.serverSettings.getSyncRange():
                    self.serverSettings.setSyncRange(self.getParameter("cwe-sync-date-range"))
                    self.serverSettings.save()
            elif self.getParameter("dialog"):
                if self.getParameter("dialog") != self.serverSettings.getDialogOrder():
                    self.serverSettings.setDialogOrder(self.getParameter("dialog"))
                    self.serverSettings.save()
                self.data["rhcert"]["dialog_order"] = self.serverSettings.getDialogOrder()

            self.data["rhcert"]["sync"] = self.serverSettings.getSyncStatus()
            try:
                lastSyncTime = int(self.serverSettings.getSyncTime())
            except ValueError:
                lastSyncTime = 0
            self.data["rhcert"]["syncTime"] = (int(time.time()) - lastSyncTime)/60
            self.data["rhcert"]["syncRange"] = self.serverSettings.getSyncRange()

            systems = self.getStore().getSystemsDocuments()
            self.data["systems"] = list()
            for (filePath, systemDocument) in systems.items():
                systemData = dict()
                if not systemDocument:
                    continue
                self.addHardwareAndOSValues(systemDocument, systemData)
                host = Host(systemData.get(Tags.hostname), systemData.get("ip_address"))
                systemData = Test().systemType(systemData)
                systemData["host"] = host
                systemData["filePath"] = filePath
                self.data["systems"].append(systemData)
        self.compileCertifications()
        self.compileConfiguration()

        return self.data

    def addSystem(self):
        newSystem = self.getParameter("new-system")
        host = Host(newSystem)

        if not host.isValid():
            self.appendError("Error: could not validate %s as a hostname or IP address" % newSystem)
            return False

        clientProxy = RHCertClient(host, timeout=5)
        (status, info) = clientProxy.getStatus()

        if status == "status" and "waiting for" in info:
            self.appendError("Error: could not reach %s (%s)" % (newSystem, info))
            return False

        # otherwise
        try:
            task = Task(Task.register)
            task.setParameter(Task.server, self.getHostName())
            task.setParameter(Task.serverPath, self.__getServerPath())
            self.result = clientProxy.runXML(task.toXML())
            # wait a couple seconds in hopes that it's ready
            time.sleep(4)
            if not self.result:
                self.appendError("Error: Register Request Failed for host \"%s\"" % newSystem)
            else:
                self.result = clientProxy.getStatus()
                if self.result:
                    task = Task(self.result)
                    if task.isValid():
                        self.data["message"] = task.getParameter(Task.message)
                    else:
                        self.data["message"]

        except Exception as exception:
            self.appendError("Error: Register Request Failed")
            self.appendError("%s\n" % exception)
            return False
        return True

    def __getServerPath(self):
        if self.getConnection().getUsername():
            return os.path.join(self.getConnection().getUsername(),
                                    self.environment.getPhase(),
                                    Constants.systems)
        # otherwise
        return Constants.systems

    def compileCertifications(self):
        productNamePrograms = self.productNameService.getAll()
        self.data["programs"] = self.productNameService.get_filtered_data(productNamePrograms)

        # Need the package list in the old index
        self.data["packages"] = dict()
        for program in Programs.getAll():
            self.data["packages"][program] = "redhat-certification-" + program

    def compileConfiguration(self):
        self.data["configuration"] = {"version": "redhat-certification-" + self.data["rhcert"]["version"] + '.' + self.data["rhcert"]["release"],
                                      "catalogURL": self.environment.getCatalogURL(),
                                      "servicesURL": self.environment.getServicesURL(),
                                      "clientConnection": Host(self.getHostName()).getConnection(),
                                      "debug": self.serverSettings.getDebug(),
                                      "phase": self.environment.getPhase(),
                                      "ssoServer": self.environment.getSSOServer(),
                                      "catalogServer": self.environment.getCatalogServer()
                                      }

    def restartHost(self):
        systemDocument = ResultsDocument()
        systemDocument.load(self.getParameter("restart"))
        host = Host(systemDocument.getHostOS().getName(), systemDocument.getHostOS().getIPAddress())
        clientProxy = RHCertClient(host, timeout=5)
        try:
            self.result = clientProxy.restart()
            time.sleep(5)
            if not self.result:
                self.appendError("Error: Host stop request failed for %s" % host.getConnection())

        except Exception as exception:
            self.appendError("Error: Host stop Request Failed for %s\n" % host.getConnection())
            self.appendError("%s\n" % exception)
