# Copyright (c) 2015 Red Hat, Inc. All rights reserved. This copyrighted material
# is made available to anyone wishing to use, modify, copy, or
# redistribute it subject to the terms and conditions of the GNU General
# Public License v.2.
#
# This program is distributed in the hope that it will be useful, but WITHOUT ANY
# WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A
# PARTICULAR PURPOSE. See the GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
#
# Author: Greg Nichols
#
import socket, os, cgi, time, subprocess
from rhcert.compatability import XmlrpcServerProxy, StringIO

from cgi import parse_qs, FieldStorage
from xml.sax.saxutils import escape, unescape

import rhcert.version
from rhcert.model.resultsdocument import ResultsDocument
from rhcert.model.task import Task
from rhcert.tags import Constants, Tags
from rhcert.host import Host


class Page:
    """ Page is the abstract base class of rhcert server pages """

    def __init__(self, request, rhcertStore):
        self.request = request
        self.rhcertStore = rhcertStore
        self.environment = rhcertStore.getEnvironment()
        self.query = parse_qs(request['QUERY_STRING'])
        self.path = request['PATH_INFO']
        # get any POST form fields
        try:
            self.requestSize = int(request.get('CONTENT_LENGTH', 0))
        except ValueError:
            self.requestSize = 0
        # requestBody = request['wsgi.input'].read(self.requestSize)
        # self.fieldStorage = parse_qs(requestBody)
        self.fieldStorage = FieldStorage(fp=request['wsgi.input'], environ=request, keep_blank_values=True)

        self.resultsFileRegex = "^rhcert-results-(?P<hostname>.+)-(?P<date>[0-9]+)\.xml(.gz)?$"
        self.socketDefaultTimeout = socket.getdefaulttimeout()
        self.body = StringIO()
        self.responseStatus = '200 OK'
        self.responseContent = 'text/html'
        self.responseLocation = None

        # escape() and unescape() takes care of &, < and >.
        self.htmlEscapeTable = {
            '"': "&quot;",
            "'": "&apos;"
        }
        self.htmlUnescapeTable = {
            "&quot;": '"',
            "&apos;": "'"
        }

    def htmlEscape(self, text):
        return escape(text, self.htmlEscapeTable)

    def htmlUnescape(self, text):
        return unescape(text, self.htmlUnescapeTable)

    def getStore(self):
        return self.rhcertStore

    def getQueryParameter(self, name):
        try:
            if type(self.query[name]) is list:
                return self.query[name][0]
            else:
                return self.query[name]
        except:
            pass

        return None

    def setQueryParameter(self, name, value):
        self.query[name] = value

    def getParameter(self, name):
        if name in self.query:
            return self.getQueryParameter(name)
        if name in self.fieldStorage:
            return self.fieldStorage[name].value
        return None

    def getCurrentUTCTime(self):
        return time.gmtime(time.time())

    def serve(self):
        """ default page service is html wrapping + navigation """
        self.makeHTMLHeader()
        self.makeHTMLBody()
        self.makeHTMLFooter()
        responseHeaders = [('Content-Type', self.responseContent),
                       ('Content-Length', str(len(self.body.getvalue())))]
        if self.responseLocation:
            responseHeaders.append(('Location', self.responseLocation))

        return (self.responseStatus, responseHeaders, self.body.getvalue())

    def makeHTMLBody(self):
        return "<!-- call to abstract makeHTMLBody -->"

    def getHostName(self):
        return socket.gethostname()

    def makeHTMLHeader(self, scripts=None, bodyEvents=None):
        header = """
        <!DOCTYPE html>
        <html lang="en">

        <head>
          <title>Red Hat Certification</title>
          <meta name="viewport" content="width=device-width, initial-scale=1.0">
          <link rel="shortcut icon" href="/rhcert/images/favicon.ico">
          <link rel="stylesheet" href="/rhcert/css/rcue.min.css">
          <!--[if lt IE 9]>
          <script src="/rhcert/js/html5shiv.min.js"></script>
          <script src="/rhcert/js/respond.min.js"></script>
          <![endif]-->
          <!-- IE8 requires jQuery and Bootstrap JS to load in head to prevent rendering bugs -->
          <!-- IE8 requires jQuery v1.x -->
          <!-- <script src="//code.jquery.com/jquery-1.10.2.min.js"></script> -->
          <script src="/rhcert/js/jquery-1.10.2.min.js"></script>
          <script src="/rhcert/js/bootstrap.min.js"></script>
          <script src="/rhcert/js/bootstrap-select.min.js"></script>
          <script src="/rhcert/js/jquery.dataTables.js"></script>
          <script src="/rhcert/js/patternfly.min.js"></script>
          <script src="/rhcert/js/rhcert.js"></script>"""
        if scripts:
            for script in scripts:
                header += "%s\n" % script
        header += """
        </head>
        """
        if bodyEvents:
            header += "<body %s>" % (" ".join(bodyEvents))
        else:
            header += "<body>"

        header += """

        <div class="container-fluid">

        <div class="row">
            <nav class="navbar navbar-default navbar-pf" role="navigation">
                <div class="navbar-header">
                  <button type="button" class="navbar-toggle" data-toggle="collapse" data-target=".navbar-collapse-1">
                    <span class="sr-only">Toggle navigation</span>
                    <span class="icon-bar"></span>
                    <span class="icon-bar"></span>
                    <span class="icon-bar"></span>
                  </button>
                  <a class="navbar-brand" href="/">
                    <img src="/rhcert/images/logo.svg" alt="Red Hat Certification" width="151"/>
                  </a>
                </div>
                <div class="collapse navbar-collapse navbar-collapse-1">
                  <ul class="nav navbar-nav navbar-utility">
                    <li>
        """
        header += "<a href'#'>%s (%s %s)</a>" % (self.getHostName(), rhcert.version.version, rhcert.version.release)
        header += """
                    </li>
                  </ul>
                  <ul class="nav navbar-nav navbar-primary">
                    <li>
                      <a href="/certifications">Certifications</a>
                    </li>
                    <li>
                      <a href="/configuration">Configuration</a>
                    </li>
                  </ul>
                </div>
            </nav>
        </div>

        """
        self.body.write(header)

    def makeHTMLFooter(self):
        self.body.write("""</div></body></html>""")

    def renderError(self, error):
        if error:
            self.renderErrors([error])

    def renderErrors(self, errors):
        if errors:
            self.body.write("""
                    <div class="page-header">
                    <div class="alert alert-danger">
                        <span class="pficon-layered">
                            <span class="pficon pficon-error-octagon"></span>
                            <span class="pficon pficon-error-exclamation"></span>
                        </span>""")
            for error in errors:
                self.body.write("%s<br/>" % error)
            self.body.write("""
                    </div>
                    </div>""")

    def getXSLTParameters(self):
        #defaults
        defaultQueryParameters = { "sheet": "default", "content":"xml", "rpath":self.environment.getResultsPath()}
        xsltParameters = { "devices":"\'hide\'", "device-classes": "\'hide\'", "rpath":"\'%s\'" % self.environment.getResultsPath(), "server-email":"\'%s\'" % self.environment.getServerEMail()}
        for key in defaultQueryParameters.keys():
            if key not in self.request:
                self.request[key] = defaultQueryParameters[key]
        return xsltParameters

    def validateParameters(self, requirements):
        """ requirements are of the form {value:type}"""
        valid = True
        for (name, ptype) in requirements.items():
            if not self.getParameter(name):
                if valid:
                    self.body.write("<pre>")
                self.body.write("Error: %s not set" % name)
                valid = False
            elif ptype == "int":
                try:
                    int(self.getParameter(name))
                except:
                    if valid:
                        print("<pre>")
                    self.body.write("Error: %s must be an integer" % name)
                    valid = False
        if not valid:
            self.body.write("</pre>")
        return valid

    def checkHostStatus(self, host):
        """ host is a Host object"""
        (status, info) = self.getHostStatus(host)
        if status == "idle":
            return "reachable"
        if status == "status":
            return info
        if status == "task":
            task = info
            return "test: %s  asks: %s" %(task.getParameter(Task.test), task.getParameter(Task.question))

    def getHostStatus(self, host, timeout=1):
        """  host is a Host object, timeout = 1 # sec """
        socket.setdefaulttimeout(timeout)
        self.server = XmlrpcServerProxy(('http://%s:%s' % (host.getConnection(), self.environment.getServerListenerPort())), allow_none=True)
        try:
            result = self.server.getResult()
            socket.setdefaulttimeout(self.socketDefaultTimeout)
            if result:
                # is it a Task for a prompt?
                task = Task()
                task.loadString(result)
                if task.isValid():
                    return ("task", task)
                else:
                    return ("status", result)
        except socket.timeout:
            return ("status", "waiting for response")
        except Exception as e:
            print("Host Status Exception: ")
            print(e)
            return ("status", "waiting for connection")
        finally:
            socket.setdefaulttimeout(self.socketDefaultTimeout)
        return ("idle", "idle")

    def renderHostStatus(self, host):
        """  host is a Host object  """
        self.body.write("""
        <div id="%s-status">checking...</div>
        <script type="text/javascript">
           loadHostStatus('%s');
        </script>
        """ % (host.getConnection(), host.getConnection()))

    def renderHostLog(self, host):
        """  host is a Host object  """
        self.body.write("""
        <table class="table"><thead><th>Log</th></thead><tr><td>
        <div id="%s-log">checking...</div>
        <script type="text/javascript">
           loadHostLog('%s');
        </script></td></tr></table>
        """ % (host.getConnection(), host.getConnection()))


    def restartHost(self):
        errors = ""
        host = Host(self.getParameter("restart"))
        self.checkHostStatus(host)
        try:
            self.result = self.server.restart()
            time.sleep(5)
            if not self.result:
                errors += "Error: Host stop request failed for %s" % host.getConnection()

        except Exception as exception:
            errors +="Error: Host stop Request Failed for %s\n" % host.getConnection()
            errors += "%s\n" % exception

        return errors

    def taskMatchesCertification(self, task, document):
        # is this taskfor the current certification, or something else?
        return (task.getParameter(Task.certificationID) == str(document.getCertificationID())
                        and task.getParameter(Task.certificationProgram) == document.getCertificationProgram())

    def generateHostResults(self, host, paths, documents):
        # find the latest results document
        latestPath = None
        if paths:
            latestPath = sorted(paths, reverse=True)[0]

        self.body.write("""<div class="table-responsive" id="%s-results">""" % host.getConnection())

        # if latest, status, new and continue
        if latestPath:
            self.__generateHostStatus(host, latestPath, documents)

        # results
        self.__generateHostResults(host, latestPath, paths, documents)

        # end host-results div
        self.body.write("</div>")

        return latestPath

    def __generateHostStatus(self, host, latestPath, documents):
        self.body.write("""<div class="table-responsive">
                        <form method="post" action="test" id="%s-continue">
                        <table class="table"><thead><th>Status</th><th>Testing</th></thead>""" % latestPath)
        self.body.write("<tr>")
        self.body.write("")
        (status, info) = self.getHostStatus(host)

        # is this an interactive task?
        if status == "task" and info.getTask() != Task.message:
            task = info
            # is this for the current certification, or something else?
            if self.taskMatchesCertification(task, documents[latestPath]):
                self.body.write("""<td><a href="interactive?host=%s">%s</a></td>""" % (host.getConnection(), task.getParameter(Task.question)))
                self.body.write("""<td>
                                    <button type="button" class="btn btn-sm btn-primary" name="continue" 
                                        onclick="location.href='interactive?host=%s'">Continue Testing</button></td>""" % host.getConnection())
            # busy on a different cert:
            elif task.getParameter(Task.certificationID) and task.getParameter(Task.certificationProgram):
                self.body.write("""<td><a href="certification?id=%s&type=%s">Running %s Certification %s</a></td>""" 
                                % (task.getParameter(Task.certificationID), task.getParameter(Task.certificationProgram), 
                                   task.getParameter(Task.certificationProgram).capitalize(), task.getParameter(Task.certificationID)
                                   ))
                self.body.write("""<td>&nbsp;</td>""")

        # it's a message
        else:
            # is this a message wrapped in a task?
            if status == "task" and info.getTask() == Task.message:
                    message = info.getParameter(Task.message)
                    certID = info.getParameter(Task.certificationID)
                    certType = info.getParameter(Task.certificationProgram)
                    currentCertification =  self.taskMatchesCertification(info, documents[latestPath])
                    if not currentCertification:
                        if certID:
                            message += " for <a href=\"/certification?id=%s&type=%s\">certification %s</a>" % (certID, certType, certID)
            else:
                message = info 
                currentCertification = False # unknown, really

            self.body.write("<td>%s </td>" % (message))
            self.body.write("<td>")
            if self.__isReady(message):
                if currentCertification and latestPath and documents[latestPath].getState() != Constants.closed:
                    self.body.write("""<button type="submit" class="btn btn-sm btn-primary" name="continue" 
                                formaction="results?rpath=%s&sheet=summary&content=html">Continue Testing</button>&nbsp;""" % latestPath)
                else: 
                    self.body.write("""<button type="submit" class="btn btn-sm btn-default" name="new"
                                formaction="test?rpath=%s">Create New Test Run</button>""" % self.certificationPath)
            else:
                self.body.write("(busy)")
            self.body.write("</td>")

        self.body.write("""<input name="host" type="hidden" value="%s"/>""" % host.getConnection())
        self.body.write("""</tr>""")
        self.body.write("</form></table></div>")

    def __isReady(self, message):
        return message == "ready" or "Finished" in message or ("clean" in message and "succeeded" in message)

    def __generateHostResults(self, host, latestPath, paths, documents):
        self.body.write("""<table class="table"><thead><th>Results</th><th>Last Run</th><th>Submitted</th><th>Size</th><th/></thead>""")
        for path in sorted(paths, reverse=True):
            document = documents[path]
            resultsURL = "<a href=\"results?rpath=%s&sheet=summary&content=html\">%s</a>"
            self.body.write("<tr>")

            planTime = document.getPlanTime()
            runTime = document.getRunTime()
            if runTime:
                self.body.write("<td>%s</td>" % (resultsURL % (path, planTime)))
                self.body.write("<td>%s</td>" % (resultsURL % (path, runTime)))
            if not runTime: # don't link to empty test results - use button to get there
                self.body.write("<td>%s</td>" % planTime)
                self.body.write("<td>(not run)</td>")
            # submitted - mock-up
            if document.getSubmitTime():
                self.body.write("<td>%s</td>" % document.getSubmitTime())
            else:
                self.body.write("<td>(not submitted)</td>")
            if document.getLoadedFileSize():
                self.body.write("<td>%.2f MB</td>" % (float(document.getLoadedFileSize())/Constants.MB))
            else:
                self.body.write("<td>&nbsp;</td>")

            self.body.write("""<td><form method="post" action="certification" id="%s">
                                    <select onsubmit='this.options.length = 0;' onchange='submitBySelect(this);'>""" % path)
            self.body.write("""<option name="action" selected>action...</option>""")

            if document.getLoadedFileSize():
                self.body.write("""<option value="submit">submit</option>""")
                if path == latestPath and document.getState() != Constants.closed:
                    self.body.write("""<option value="clean">close</option>""")
                self.body.write("""<option value="save">save</option>""")
                self.body.write("""<option value="download">download</option>""")
            self.body.write("""<option value="delete">delete</option>""")
            self.body.write("""</select><input type="hidden" name="rpath" value="%s"/>""" % path)
            self.body.write("""<input name="host" type="hidden" value="%s"/>""" % host.getConnection())
            self.body.write("""</form></td>""")
            self.body.write("</tr>")

        self.body.write("</table>")

    def beginTestingStandalone(self, certificationDocument):
        """ intitial setup to test a cert stand-alone, on this server """
        errors = list()
        if not self.getHostName():
            errors.append("Error: could not determine local host name")
            return errors

        # otherwise
        resultsDocument = ResultsDocument()
        resultsDocument.copy(certificationDocument)
        self.testHostname = resultsDocument.getRequiredOS().setName(self.getHostName())
        resultsDocument.setCertificationID(certificationDocument.getCertificationID())
        resultsDocument.setCertificationProgram(certificationDocument.getCertificationProgram())
        # delete all the tests - we just want the client to plan
        for testDocument in resultsDocument.getTests():
            testDocument.delete()
        self.getStore().saveResultsDocument(resultsDocument)

        errors.extend(self.requestRun(resultsDocument))

        return errors # may be empty



    def requestRun(self, resultsDocument):
        errors = list()
        self.testHost = Host(resultsDocument.getOS(Tags.hostname), resultsDocument.getOS(Tags.ip_address))
        self.testHostURL = 'http://%s:%s' % (self.testHost.getConnection(), self.environment.getServerListenerPort())
        self.server = XmlrpcServerProxy(self.testHostURL, allow_none=True)

        # set the test server to this one
        resultsDocument.setTestServer(self.getHostName())

        try:
            self.result = self.server.run(resultsDocument.document.toxml())
            if not self.result:
                errors.append("Error: Run Request Failed.")
            return errors
        except Exception as exception:
            errors.append("Error: call to host %s failed:\n" % (self.testHost.getConnection()))
            errors.append("%s\n" % exception)
            errors.append("%s\n" % resultsDocument.document.toxml())
        return errors

