# Copyright (c) 2014 Red Hat, Inc. All rights reserved. This copyrighted material 
# is made available to anyone wishing to use, modify, copy, or 
# redistribute it subject to the terms and conditions of the GNU General 
# Public License v.2.
# 
# This program is distributed in the hope that it will be useful, but WITHOUT ANY
# WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A
# PARTICULAR PURPOSE. See the GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
#
# Author: Greg Nichols <gnichols@redhat.com>
#
import os
from django.shortcuts import render

from rhcert.compatability import XmlrpcServerProxy
from rhcert.model.resultsdocument import ResultsDocument
from rhcert.tags import Constants, Tags
from rhcert.host import Host
from .view import View


class Run(View):
    def __init__(self):
        View.__init__(self)
        self.testsToRun = dict()
        self.result = None
        self.debug = None
        self.certificationID = None
        self.certificationProgram = None
        self.certification = None

    def serve(self, request, template):
        View.authenticate(self, request)
        result = self.view(request)
        if type(result) != dict:
            return result
        # otherwise
        return render(request, template, result)

    def view(self, request):
        View.view(self, request)
        if not self.validateParameters():
            return self.data

        if not self.getParameter("cancelled"):

            if self.getParameter("closeResultsFile"):
                return self.redirect("clean", {"rpath": self.getParameter("rpath"),
                                               "hostname": self.testHost.getName(),
                                               "hostconnection": self.testHost.getConnection(),
                                               "confirmed": "confirmed"}, paramList=None)

            if not self.requestRun():
                return self.data

            if not self.checkStatus():
                return self.data

        # otherwise
        return self.redirect(self.parentURL, queryDict=None, paramList=[self.certificationProgram,  self.certificationID])

    def validateParameters(self):
        requiredParameters = ["rpath"]
        valid = True
        for param in requiredParameters:
            if not self.getParameter(param):
                self.appendError("Error: %s not specified\n" % param)
                valid = False
        if not valid:
            return False

        # get the hostname
        self.data["rpath"] = self.getParameter("rpath")
        self.resultsDocument = ResultsDocument()
        self.resultsDocument.loadFiltered(self.getParameter("rpath"), os.path.join(self.environment.getServerDirectory(), "xsl/default.xsl"), self.getXSLTParameters())
        self.testHost = Host(self.resultsDocument.getOS(Tags.hostname), self.resultsDocument.getOS(Tags.ip_address))
        self.certificationID = self.resultsDocument.getCertificationID()
        self.data["certificationID"] = self.certificationID
        self.certificationProgram = self.resultsDocument.getCertificationProgram()
        (_, self.certification) = self.getStore().getCertification(self.certificationProgram, self.certificationID)
        self.testHostURL = 'http://%s:%s' % (self.testHost.getConnection(), self.environment.getServerListenerPort())
        self.server = XmlrpcServerProxy(self.testHostURL, allow_none=True)
        self.data["new_run"] = self.getListParameter("new-run")

        self.setParentURL()

        return True

    def setParentURL(self):
        self.parentURL = "results" # really is certification-activity-testing
        self.thisURL = "testing"
        if self.certificationID and self.certificationProgram:
            if self.certification and self.certification.isSandbox():
                self.parentURL = "sandbox-results"
                self.thisURL = "sandbox-activity-testing"
                self.data["isSandbox"] = True
        self.data["parentURL"] = self.parentURL
        self.data["thisURL"] = self.thisURL

    def requestRun(self, tests=None):
        # test options are "<test path> [--<parameter> <value>]*"
        # note, no tests may have been picked, electing to run just mandatory tests
        if tests is None:
            tests = []
        if self.getListParameter("new-run"):
            for testOption in self.getListParameter("new-run"):
                testSpec = self.parseTestOption(testOption)
                if testSpec:
                    tests.append(testSpec)

        # prune XML doc to just the checked test specs
        self.selectTests(tests, self.resultsDocument)

        # set the test server to this one
        self.resultsDocument.setTestServer(self.getHostName())

        try:
            xml = self.resultsDocument.toXML()
            self.result = self.server.run(xml)
            if not self.result:
                self.appendError("Error: Run Request Failed.")
                return False
            # close the current results document
            previousDocument = ResultsDocument()
            previousDocument.load(self.getParameter("rpath"))
            previousDocument.setState(Constants.closed)
            previousDocument.save()

        except Exception as exception:
            self.appendError("Error: call to host %s failed:\n" % self.testHostURL)
            self.appendError("%s\n" % exception)
            return False

        return True

    def checkStatus(self):
        try:
            self.result = self.server.getResult()
        except Exception as exception:
            self.appendError("Error: call to host %s failed:\n" % self.testHostURL)
            self.appendError("%s\n" % exception)
            return False
        return True


    def getXSLTParameters(self):
        #defaults
        defaultQueryParameters = { "sheet": "default", "content":"xml", "rpath":self.environment.getResultsPath()}
        xsltParameters = { "devices":"\'hide\'", "device-classes": "\'hide\'", "rpath":"\'%s\'" % self.environment.getResultsPath(), "server-email":"\'%s\'" % self.environment.getServerEMail()}
        """ for key in defaultQueryParameters.keys():
            if key not in self.request:
                self.request[key] = defaultQueryParameters[key]  """
        return xsltParameters



