# Copyright (c) 2014 Red Hat, Inc. All rights reserved. This copyrighted material
# is made available to anyone wishing to use, modify, copy, or
# redistribute it subject to the terms and conditions of the GNU General
# Public License v.2.
#
# This program is distributed in the hope that it will be useful, but WITHOUT ANY
# WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A
# PARTICULAR PURPOSE. See the GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
#
# Author: Greg Nichols
#
# Implementation of the wsgi for saving test results/certification documents
import os
from rhcert.compatability import StringIO
from django.http import HttpResponse, HttpResponseRedirect
from django.shortcuts import render

from rhcert.httpUpload import HttpUpload
from rhcert.model.resultsdocument import ResultsDocument
from rhcert.tags import Constants
from .view import View
from .alert import Alert


class Save(View):
    def __init__(self):
        View.__init__(self)
        self.loginRequired = False

    def serve(self, request, template):
        View.authenticate(self, request)

        result = self.view(request)
        if type(result) != dict:
            return result
        # otherwise
        return render(request, template, result)

    def view(self, request):
        View.view(self, request)
        self.request = request
        # was a file submitted in a form pushed by client or another server?
        if self.request.FILES.get('file'):
            return self.__saveEnclosedFile()

        # otherwise - push file from this server to another server
        return self.__pushFileToAnotherServer()

    def __saveEnclosedFile(self):
        (pathIsURL, path) = self.saveFiles()
        if self.getParameter("show-destination") and self.getParameter("return-after"):
            if path:
                if pathIsURL:
                    return HttpResponseRedirect(path)
                else: # it's a local file path - no cert ID
                    return self.redirect("resultsXML", {"rpath": path})

            # show errors after redirect
            Alert(Alert.danger, "Please upload a valid certification test results file ( .xml or .xml.gz ).", self.request)
            return HttpResponseRedirect(self.getParameter("return-after"))

        # otherwise plan-text response to a different server instance:
        if path:
            # note - the syntax must be as follows to support prior releases of hwcert-client used in fv_network
            message = "Saving File From IP: %s\n" % self.getClientIP()
            if pathIsURL:
                full_path = 'http://%s%s' % (self.getHostName(), path)
                message += "To:\n" + full_path.replace(' ', '%20')
            else:
                path_value = "/results-xml?rpath="
                full_path = 'http://%s%s%s' % (self.getHostName(), path_value, path)
                message += "To:\n" + full_path.replace(' ', '%20')
            return HttpResponse(message, content_type='text/plain')
        # errors
        textErrors = ""
        for error in self.getErrors():
            textErrors += str(error) + "\n"
        return HttpResponse(textErrors, content_type='text/plain')

    def __pushFileToAnotherServer(self):
        self.data["rpath"] = self.getParameter("rpath")
        self.data["return-after"] = self.getParameter("return-after")

        if not self.data["rpath"]:
            Alert(Alert.danger, "Please choose a Results File to upload", self.request)
            return HttpResponseRedirect(self.data["return-after"])

        if not self.getParameter("server"):
            document = self.getStore().getResultsDocument(self.getParameter("rpath"))
            self.data["file"] = os.path.basename(self.getParameter("rpath"))
            if document:
                self.data["resultsDocument"] = document
            return self.data

        self.pushFileToServer()
        return self.data # show confirmation page and/or errors

    def pushFileToServer(self):
        self.serverPath = os.path.join(self.environment.getServerDirectory(), Constants.store, Constants.results)
        try:
            request = HttpUpload(self.getParameter("server"), "/save")
            request.addField("server-path", self.serverPath)
            log = StringIO()
            request.httpUpload(self.getParameter("rpath"), log)

        except Exception as e:
            Alert(Alert.danger, "Error: could not save results to server %s" % self.getParameter("server"), self.request)
            Alert(Alert.danger, "Error: %s" % e, self.request)
            return False

        self.data["savedFile"] = os.path.basename(self.getParameter("rpath"))
        self.data["server"] = self.getParameter("server")
        self.data["serverPath"] = self.serverPath
        self.data["response"] = log.getvalue()
        self.data["destination"] = log.getvalue().splitlines()[4]
        self.data["program"] = self.getParameter("program")
        self.data["certificationID"] = self.getParameter("certificationID")
        self.data["return-after"] = self.getParameter("return-after")
        self.data["isSandbox"] = self.getStore().getResultsDocument(self.getParameter("rpath")).isSandbox()
        return True

    def saveThisServerToStore(self):
        resultsDocument = ResultsDocument()
        resultsDocument.load(self.fieldStorage["rpath"].value)
        if resultsDocument.getNumberOfTestRuns() > 0:
            path = [self.environment.getServerDirectory(), "store"]
            path.extend(resultsDocument.getStorePath())
            serverPath = os.path.join(*path)
            self.makeDirectoryPath(serverPath)
            savedFilename = resultsDocument.getVerboseFileName()
            savedFilePath = os.path.join(serverPath, savedFilename)
            resultsDocument.compressToFile(savedFilePath)
            return savedFilePath + ".gz"
        #otherwise
        return None

    def saveFiles(self):
        ipAddress = self.getClientIP()
        serverPath = self.getParameter("server-path")
        print("Got server-path = %s" % serverPath)
        if not serverPath:
            serverPath = os.path.join(self.environment.getServerDirectory(), Constants.store, self.environment.getServerTransfer())
            print("defaulted server path: %s" % serverPath)

        # django request.FILES is a dictionary of django.core.files.UploadedFile
        uploadedFile = self.request.FILES.get('file')
        if uploadedFile:
            path = None
            pathIsURL = False
            try:
                fileName = uploadedFile.name
                if "octet-stream" in uploadedFile.content_type:
                    fileContents = uploadedFile.read().decode("base64")
                else:
                    fileContents = uploadedFile.read()

                if Constants.results in serverPath:
                    (pathIsURL, path) = self.getStore().saveResultsFile(fileName, fileContents)

                elif Constants.systems in serverPath:
                    path = self.getStore().saveSystemFile(fileName, fileContents, serverPath)

                elif Constants.transfer in serverPath:
                    path = self.getStore().saveTransferFile(fileName, fileContents, ipAddress)

                else:
                    return "Error: could not determine file type."

                if not self.getStore().getErrors():
                    print("Saved %s from %s to %s" % (fileName, ipAddress, path))
                    return (pathIsURL, path)
                # otherwise
                self.appendErrors(self.getStore().getErrors())
            except Exception as e:
                self.appendError("Error: Could not save file %s  %s\n" %  (fileName, e))
        else:
            self.appendError("Error: no \"file\" in upload request")

        # copy errors to log
        for alert in self.getAlerts():
            print(alert.getMessage())

        return (None, None)

