# Copyright (c) 2014 Red Hat, Inc. All rights reserved. This copyrighted material
# is made available to anyone wishing to use, modify, copy, or
# redistribute it subject to the terms and conditions of the GNU General
# Public License v.2.
#
# This program is distributed in the hope that it will be useful, but WITHOUT ANY
# WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A
# PARTICULAR PURPOSE. See the GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
#
# Author: Greg Nichols
#
# Implementation of the wsgi for saving test results/certification documents
import os
from django.shortcuts import render
from django.http import HttpResponseRedirect

from .view import View
from .alert import Alert


class Submit(View):
    def __init__(self):
        View.__init__(self)
        self.document = None

    def serve(self, request, template, program, id):
        View.authenticate(self, request)

        self.certificationProgram = program
        self.certificationID = id
        result = self.view(request)
        if type(result) != dict:
            return result
        return render(request, template, result)

    def view(self, request):
        View.view(self, request)
        self.request = request
        if self.getParameter("cancel"):
            return HttpResponseRedirect(self.getParameter("return-after"))

        if self.getParameter("state"):
            if self.submit():
                return self.redirect("results", queryDict=None,
                        paramList=[self.certificationProgram,  self.document.getCertificationID()])

        elif self.getParameter("rpath"):
            self.loadResults()
        else:
            self.appendError("Error: no file to submit (missing rpath)")

        return self.data


    def loadResults(self):
        # make sure catalog is reachable
        if not self.getWorkflow().isReachable():
            self.appendError("Error: could not reach the certification server")
            return False
        self.data["catalogURL"] = self.environment.getCatalogServer()
        self.imagePath = None
        self.document = None
        self.filePath = self.getParameter("rpath")
        self.data["path"] = self.getParameter("rpath")
        self.data["filename"] = os.path.basename(self.getParameter("rpath"))
        self.certificationProgram = None
        errors = None
        (self.fileType, self.document) = self.getStore().getCertificationInfo(self.filePath, errors)
        if errors:
            self.appendError(errors)
            return False
        if self.document:
            self.certificationProgram = self.document.getCertificationProgram()
            self.data["document"] = self.document
            self.data["fileType"] = self.fileType
            self.data.update(self.getWorkflowParameters(self.document))
            # determine test suite version through which the result file is generated.
            result_source_package = "-".join(["redhat-certification",
                self.document.getVersion(),
                self.document.getRelease()])

            response = self.getCertificationWorkflow().checkSupported(result_source_package)
            result_warning = str()
            if response:
                # Warn if result_source_package is unsupported or not the latest.
                if response["supported"] == 0:
                    result_warning = ("Warning: The result file is generated from version: {0} " \
                    " which is unsupported. Please upgrade to latest: {1}"
                    ).format(result_source_package, response["current NVR"])

                elif response["supported"] == 1 and result_source_package != response["current NVR"]:
                    result_warning = ("Warning: The result file is generated from version: {0} " \
                    "which is not the latest and will expire on {1}. " \
                    "Please upgrade to latest: {2}"
                    ).format(result_source_package, response["expires"], response["current NVR"])

            if result_warning:
                alert = Alert(Alert.warning, result_warning, self.request)
                self.appendAlert(alert)


    def submit(self):
        valid = True
        if self.getParameter("state"):
            if "new" == self.getParameter("state") and not self.validateParameters({"username":"string", "password":"string"}):
                valid = False
            elif "existing" == self.getParameter("state") and not self.validateParameters({"id":"int"}):
                valid = False

        if not valid:
            return False

        if not self.getWorkflow().isReachable():
            self.appendError("Error: could not reach the certification server")
            return False

        self.loadResults()
        if not self.document:
            return False

        return self.submitResults()

    def submitResults(self):
        self.getCertificationWorkflow().submitTestRun(self.filePath, self.document)
        self.appendErrors(self.getCertificationWorkflow().getErrors())
        if not self.getCertificationWorkflow().getErrors():
            certificationDict = dict()
            self.data["certification"] = certificationDict
            certificationDict["id"] = self.document.getCertificationID()
            certificationDict["type"] = self.document.getCertificationProgram()
            return True
        return False
