# Copyright (c) 2014 Red Hat, Inc. All rights reserved. This copyrighted material
# is made available to anyone wishing to use, modify, copy, or
# redistribute it subject to the terms and conditions of the GNU General
# Public License v.2.
#
# This program is distributed in the hope that it will be useful, but WITHOUT ANY
# WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A
# PARTICULAR PURPOSE. See the GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
#
# Author: Greg Nichols
#
from django.shortcuts import render

from rhcert.model.resultsdocument import  ResultsDocument
from rhcert.tags import Tags, Constants, Programs
from rhcert.host import Host
from rhcert.client.client import RHCertClient

from .view import View
from .alert import Alert


class Test(View):
    """ Associates a host with a certification to begin testing """

    def __init__(self):
        View.__init__(self)
        self.certificationID = None
        self.certificationProgram = None
        self.certification = None

    def serve(self, request, template, program, id, host=None):
        View.authenticate(self, request)
        self.request = request
        self.template = template
        self.certificationProgram = program
        self.certificationID = id
        self.host = None
        if host:
            self.host = Host(host)

        result = self.view(request)
        if type(result) != dict:
            return result
        # otherwise
        return render(request, self.template, result)

    def __getRedirectURL(self):
        if self.certification and self.certification.isSandbox():
            return "sandbox-results"
        return "results"

    def view(self, request):
        result = View.view(self, request)
        if type(result) != dict:
            return result

        (_, self.certification) = self.getStore().getCertification(self.certificationProgram, self.certificationID)
        if not self.certification:
            self.appendError("Error: no such %s certification id %s" % (self.certificationProgram, self.certificationID))
            return self.redirect("results", queryDict=None, paramList=[self.certificationProgram, self.certificationID])
        self.data["isSandbox"] = self.certification.isSandbox()

        if self.getParameter("cancel"):
            return self.redirect(self.__getRedirectURL(), queryDict=None, paramList=[self.certificationProgram,  self.certificationID])

        self.data["certificationProgram"] = self.certificationProgram
        self.data["certificationID"] = self.certificationID
        self.__setRequiredOSAndHostRoles()

        redirect = self.beginTesting()
        if redirect:
            return redirect

        # otherwise
        self.compileHostsData()
        if not self.data.get("matches") and not self.getParameter("override"):
            if self.data.get("systems"):
                message = "Sorry: no matching systems found.  Override the mismatch, or "
            else:
                message = "Sorry: no existing systems found. Please click here to "
            alert = Alert(Alert.warning, message, request)
            alert.setCorrection("configuration", [], " add a relevant system.")
            self.appendAlert(alert)

        return self.data

    def __setRequiredOSAndHostRoles(self):
        # OS required for certification
        self.requiredOS = self.certification.getRequiredOS()
        if self.certification.isMultiHost():
            # Host roles required for testing
            self.required_host_roles = self.certification.getRequiredHostRoles()
            self.data['required_host_roles'] = self.required_host_roles
            self.data["bodyEvents"] = "onload=roleSelectorInit();"


    def compileHostsData(self):
        if self.getParameter("override"):
            self.data["overridden"] = True
        self.data["systems"] = list()

        if not self.requiredOS:
            return

        systems = self.getStore().getSystemsDocuments()
        matchCount = 0
        for (filePath, resultsDocument)  in systems.items():
            systemData = dict()
            self.data["systems"].append(systemData)
            systemData["filePath"] = filePath
            _host = self.addHardwareAndOSValues(resultsDocument, systemData)
            match = self.__match(resultsDocument)
            systemData = self.systemType(systemData)

            if match == "match":
                # if not already associated with this certification
                if not self.getStore().getResultsDocumentsFor(self.certification, skipEmpty=False, host=_host):
                    matchCount = matchCount + 1
                    systemData["state"] = "available"
                else:
                    systemData["state"] = "current"
            else:
                systemData["state"] = match # mis-match description string
        if matchCount > 0:
            self.data["matches"] = True
        if matchCount < len(systems):
            self.data["mismatches"] = True

    def systemType(self, systemData):
        # setting type for systems
        system_type = ['vendor', 'make', 'model']
        # eliminates the duplicate values of type
        type = set()
        for rec in system_type:
            type.add(systemData[rec] if rec in systemData else '')
        systemData["type"] = ' '.join(type)
        return systemData

    def __match(self, system):
        # get the matching policy
        policy = Constants.exact
        if self.certificationProgram in [Programs.hardware, Programs.openstack]:
            policy = Constants.minimum

        # require exact match here
        if self.requiredOS.getProduct() != system.getHostOS().getProduct():
            return ("certification for \"%s\" does not match host OS \"%s\"" 
                    % (self.requiredOS.getProduct(), system.getHostOS().getProduct()))

        (certVersion, certUpdate) = self.__trim(self.requiredOS.getRelease())
        (systemVersion, systemUpdate) = self.__trim(system.getHostOS().getRelease())

        # major must be equal
        if certVersion != systemVersion:
            return "Host is running RHEL %s, which does not match the certification on RHEL %s" % (systemVersion, certVersion)
        if policy == Constants.exact and certUpdate != systemUpdate:
            return "Host is running RHEL %s.%s, which does not match the certification on RHEL %s.%s" % (systemVersion, systemUpdate, certVersion, certUpdate)
        if policy == Constants.minimum and systemUpdate < certUpdate:
            return "Host is running RHEL %s.%s, which older than the certification on RHEL %s.%s" % (systemVersion, systemUpdate, certVersion, certUpdate)
        if policy == Constants.maximum and systemUpdate > certUpdate:
            return "Host is running RHEL %s.%s, which is newer than the certification on RHEL %s.%s" % (systemVersion, systemUpdate, certVersion, certUpdate)

        return "match"

    def __trim(self, releaseString):
        """ expand "7" to "7.0" , remove -min or -max"""
        if "-min" in releaseString or "-max" in releaseString:
            releaseString = releaseString[0:-4]
        parts = releaseString.split(".")
        if len(parts) == 1:
            return (releaseString, "0")
        #otherwise
        return (releaseString.split(".")[0], releaseString.split(".")[1])

    def beginTesting(self):
        if self.host:
            hostPath = self.getStore().getSystemPathForHost(self.host)
        else:
            hostPath = self.getParameter("system-path")
        if hostPath:
            return self.__testHost(hostPath)

        # multi-host selection?
        systems = self.getStore().getSystemsDocuments()
        redirect = None
        for (filePath, resultsDocument)  in systems.items():
            if self.getParameter(filePath):
                redirect = self.__testHost(filePath, hostID=self.getParameter(filePath))
        if redirect:
            # wait to redirect until all hosts are ready
            timeOutOnHost = self.__waitForAllHostsReady()
            if timeOutOnHost:
                self.appendAlert(Alert(Alert.danger,
                        ("Time out waiting for %s" % timeOutOnHost.getHost().getName()), self.request))
                return None
            # otherwise, all hosts are ready
            return redirect

        if self.host:
            self.appendAlert(Alert(Alert.danger, "Could not find host document for %s" % self.host.getName(), self.request))

        return None

    def __testHost(self, hostPath, hostID=None):

        resultsDocument = ResultsDocument()
        resultsDocument.load(hostPath)

        # copy the certification info, but save the host info
        self.testHost = Host(resultsDocument.getHostOS().getName(), resultsDocument.getHostOS().getIPAddress())
        if not self.getParameter("hosts"):
            self.hosts = list()
        self.hosts.append(RHCertClient(self.testHost, timeout=3, environment=self.environment))
        resultsDocument.copyCertification(self.certification, deleteProperties=True)
        hostRequirements = resultsDocument.getRequiredOS()
        hostRequirements.setName(self.testHost.getName())
        hostRequirements.setIPAddress(self.testHost.getIPAddress())
        if hostID:
            hostRequirements.setID(hostID)
            hostRequirements.setRole(self.required_host_roles.get(hostID).getRole())
            self.__deleteHostResults(self.testHost) # delete intermediate host docs (multi-host results will remain)
        resultsDocument.copyPlanComponents(self.certification)

        # delete all the tests - we just want the client to plan
        for testDocument in resultsDocument.getTests():
            testDocument.delete()
        self.getStore().saveResultsDocument(resultsDocument)
        if self.requestRun(resultsDocument):
            # redirect to cert results
            return self.redirect(self.__getRedirectURL(), queryDict=None, paramList=[self.certificationProgram,  self.certificationID])
        # otherwise errors
        return None

    def __deleteHostResults(self, host):
        result_document = self.getStore().getResultsDocumentsFor(self.certification, skipEmpty=False, host=self.host)
        for path in result_document.keys():
            if not path or not self.getStore().deleteResults(path):
                continue

    def __waitForAllHostsReady(self):
        """ wait until all hosts are ready """
        for host in self.hosts:
            ready = host.waitUntilReady(60)
            if not ready:
                return host
        return None # all hosts are ready