# Copyright (c) 2017 Red Hat, Inc. All rights reserved. This copyrighted material
# is made available to anyone wishing to use, modify, copy, or
# redistribute it subject to the terms and conditions of the GNU General
# Public License v.2.
#
# This program is distributed in the hope that it will be useful, but WITHOUT ANY
# WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A
# PARTICULAR PURPOSE. See the GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
#
# Author: Rajasi Kulkarni <rajkulka@redhat.com>
#
# Implementation of the wsgi for updating Certification

from django.shortcuts import render
from django.http import HttpResponseRedirect

from rhcert.tags import Programs

from .update import Update
from ..alert import Alert


class UpdateCertification(Update):

    def __init__(self):
        Update.__init__(self)
        self.activeTab = None
        self.certificationID = None
        self.program = None

    def view(self, request):
        Update.view(self, request)
        self.certificationID = self.getParameter("certificationID")
        self.program = self.getParameter("program")
        self.activeTab  = self.getParameter("activeTab")
        self.chosenFile = self.getParameter("chosenFile")
        self.support_case_id = self.getParameter("support_case_id")
        self.kbase_id = self.getParameter("kbase_id")
        self.kbase_type = self.getParameter("kbase_type")

        if self.certificationID:
            # For all programs, check if Support Case is to be added
            if self.support_case_id:
                self.add_support_case()

            # For all programs, check if KBase is to be added
            if self.kbase_id:
                self.add_kbase()

            # Check if there is any file to upload for OpenStack certification
            if self.program == Programs.openstack and self.chosenFile:
                self.upload_file_to_certification()

            # For hardware program check if sample details are to be added
            if self.program == Programs.hardware and self.getParameter("similar_previous_sample"):
                self.add_sample_details()

            # Update Certification Details for all programs
            if (self.program == Programs.hardware and (self.activeTab == "publish" or \
                self.activeTab == "sample")) or \
            (self.program == Programs.cloud and self.activeTab == "details") or \
            (self.program == Programs.openstack and self.activeTab != "details"):
                self.update_certification_details()

        return self.redirect_to_details_or_specification()


    def add_support_case(self):
        # Call API to associate support case to Certification
        supportCaseResponse = self.getCertificationWorkflow().addSupportCase(self.certificationID,
                                                                        self.support_case_id)
        if not self.getErrors() and supportCaseResponse:
            Alert(Alert.success, "Support Case associated with the Certification", self.request)
        else:
            Alert(Alert.danger, "Error in assocating Support Case with the Certification", self.request)
            Alert(Alert.danger, self.getCertificationWorkflow().getErrors(), self.request)


    def add_kbase(self):
        # Call API to associate Kbase to Certification
        kbaseResponse = self.getCertificationWorkflow().addKBase(self.certificationID,
                                                            self.kbase_id,
                                                            self.kbase_type)
        if not self.getErrors() and kbaseResponse:
            Alert(Alert.success, "Reference of Knowlege Base added to the Certification", self.request)
        else:
            Alert(Alert.danger, "Error in adding reference of Knowledge Base to the Certification", self.request)
            Alert(Alert.danger, self.getCertificationWorkflow().getErrors(), self.request)


    def update_certification_details(self):
        # Call API to update Certification Details
        updateResponse = self.getCertificationWorkflow().update(self.program, self.certificationID, self.getPostDictionary())
        if not self.getErrors() and updateResponse:
            Alert(Alert.success, "Certification updated successfully", self.request)
        else:
            Alert(Alert.danger, "Error in updating certification", self.request)
            Alert(Alert.danger, self.getCertificationWorkflow().getErrors(), self.request)


    def upload_file_to_certification(self):
        # API call to upload file
        fileResponse = self.getEngineWorkflow().uploadFileToCertification(self.certificationID,
                                                                          self.getParameter("isSpec"),
                                                                          self.getParameter("description"),
                                                                          self.chosenFile.name,
                                                                          self.chosenFile.read(),
                                                                          self.chosenFile.content_type)
        if not self.getErrors() and fileResponse:
            Alert(Alert.success, "File uploaded successfully", self.request)
        else:
            Alert(Alert.danger, "Error in file upload", self.request)
            Alert(Alert.danger, self.getEngineWorkflow().getErrors(), self.request)


    def add_sample_details(self):
        self.certification = self.getStore().getCertification(self.program, self.certificationID)[1]
        sampleResponse = self.getCertificationWorkflow().addComment(self.certification,
                                                               self.getParameter("similar_previous_sample"))
        if not self.getErrors() and sampleResponse:
            Alert(Alert.success, "Details updated successfully", self.request)
        else:
            Alert(Alert.danger, "Details updated successfully", self.request)
            Alert(Alert.danger, self.getCertificationWorkflow().getErrors(), self.request)


    def redirect_to_details_or_specification(self):
        if self.program == Programs.openstack:
            if self.activeTab in ["networking", "filestorage", "blockstorage", "dataprocessing"]:
                return self.redirect("openstack-specification-details",
                                     queryDict=None,
                                     paramList=[self.certificationID, self.activeTab])
            if self.activeTab == "specification-details":
                return self.redirect("certification-specification-details",
                                     queryDict=None,
                                     paramList=[self.program, self.certificationID])
            if self.activeTab == "details":
                return self.redirect("details",
                                     queryDict=None,
                                     paramList=[self.program, self.certificationID])

        elif self.program == Programs.hardware:
            if self.activeTab == "sample":
                return self.redirect("hardware-specification-sample",
                                     queryDict=None,
                                     paramList=[self.certificationID])
            elif self.activeTab == "details":
                return self.redirect("details",
                                     queryDict=None,
                                     paramList=[self.program,  self.certificationID])
            elif self.activeTab == "publish":
                return self.redirect("hardware-publish",
                                     queryDict=None,
                                     paramList=[self.certificationID])
        else:
            return self.redirect("details", queryDict=None, paramList=[self.program,  self.certificationID])
