""" View: a base class for view classes in rhcert

    develops view data to support the page.html Template

"""
import socket, os, time
import json
from datetime import datetime
from dateutil import tz
from xml.sax.saxutils import escape, unescape

from django.shortcuts import redirect

try: # django 2
    from django.urls import reverse
except: # django 1
    from django.core.urlresolvers import reverse

from django.http import HttpResponseRedirect

from rhcert.compatability import XmlrpcServerProxy, rhcert_make_unicode
from rhcert.tags import Tags, Constants, Programs, ServerConfiguration
from rhcert.environment import Environment
from rhcert.host import Host
from rhcert.model.task import Task
from rhcert.services.productnameservice import ProductNameService
from rhcert.services.microservices import MicroServices
from rhcert.client.client import RHCertClient
import rhcert.version

from .alert import Alert
from .connection import Connection


class LoginRequiredException(Exception):
    """
    raises API exceptions with custom messages and custom status codes
    """
    def __init__(self):
        pass


class View(object):
    def __init__(self):
        self.environment = Environment()
        self.isManagementMode = (self.environment.getServerConfiguration() ==
                                 ServerConfiguration.management)
        # default: False
        self.loginRequired = True if self.isManagementMode else False
        self.productNameService = ProductNameService()
        self.microServices = MicroServices()
        # escape() and unescape() takes care of &, < and >.
        self.htmlEscapeTable = {
            '"': "&quot;",
            "'": "&apos;"
        }
        self.htmlUnescapeTable = {
            "&quot;": '"',
            "&apos;": "'"
        }
        self.request = None
        self.connection = None
        self.data = {"alerts": []}

    def getConnection(self, request=None):
        if request and (request != self.request or not self.connection):
            self.connection = Connection(request)
            self.request = request
        return self.connection

    def getWorkflow(self):
        if self.connection:
            return self.connection.getCertificationWorkflow()
        return None

    def getCertificationWorkflow(self):
        if self.connection:
            return self.connection.getCertificationWorkflow()
        return None

    def getVendorProductWorkflow(self):
        if self.connection:
            return self.connection.getVendorProductWorkflow()
        return None

    def getEngineWorkflow(self):
        if self.connection:
            return self.connection.getEngineWorkflow()
        return None

    def getStore(self):
        if self.connection:
            return self.connection.getStore()

    def authenticate(self, request):
        self.getConnection(request)
        if self.environment.getServerConfiguration() != ServerConfiguration.testing and (
                    self.loginRequired and not self.getWorkflow().isLoggedIn()):
            raise LoginRequiredException
        return None

    def view(self, request):
        self.getConnection(request)
        self.fieldStorage = request.POST
        """ returns a dict referred to by page.html """
        label = Constants.rhcert
        self.data[label] = dict()
        self.data[label][Tags.version] = rhcert.version.version
        self.data[label][Tags.release] = rhcert.version.release
        self.data[label][Tags.hostname] = socket.gethostname()
        self.data[label]["debug"] = self.getStore().getServerSettings().getDebug()
        self.data[label]["dialog_order"] = self.getStore().getServerSettings().getDialogOrder()
        self.data[label]["sync"] = self.getStore().getServerSettings().getSyncStatus()
        self.data[label]["configuration"] = self.environment.getServerConfiguration()
        self.data["MaxDialogAttachmentSize"] = self.environment.getMaximumDialogAttachmentSize()
        return self.data

    def redirect(self, view, queryDict=None, paramList=list()):
        query = ""
        if queryDict:
            query = "?"
            for (key, value) in queryDict.items():
                if value:
                    if query != "?":
                        query += "&"
                    query += "%s=%s" % (key, value)
        return HttpResponseRedirect(reverse(view, args=paramList) + query)

    def redirectOnError(self):
        url = "certifications" if self.loginRequired else "results"
        return self.redirect(url, queryDict=None)

    def tryLoginFirst(self):
        if self.request.session.get("opt_out_login"):
            return None
        if not self.getWorkflow().isLoggedIn() and self.getWorkflow().isReachable():
            return self.redirect("login", queryDict=None)
        return None

    def appendError(self, error):
        if error:
            self.appendAlert(Alert(Alert.danger, error, self.request))

    def appendErrors(self, errors):
        for error in errors:
            self.appendAlert(Alert(Alert.danger, error, self.request))

    def getErrors(self):
        errors = list()
        for alert in self.data["alerts"]:
            if alert.getLevel() == Alert.danger:
                errors.append(alert)
        return errors

    def clearAlerts(self):
        Alert.clearMessages(self.request)

    def appendAlert(self, alert):
        if not isinstance(alert, Alert):
            raise Exception("Error: adding non-Alert to appendAlert call: %s" % alert)
        self.data["alerts"].append(alert)

    def getAlerts(self):
        return self.data["alerts"]


    def getParameter(self, name):
        for requestDictionary in [self.request.GET, self.request.POST, self.request.FILES]:
            value = requestDictionary.get(name)
            if value:
                return value
        return None

    def getListParameter(self, name):
        return self.request.POST.getlist(name)

    def getPostDictionary(self):
        convertedDictionary = dict()
        for key in self.request.POST.keys():
            value = self.request.POST.getlist(key)
            if len(value) ==1:
                convertedDictionary[key] = value[0]
            else:
                convertedDictionary[key] = value
        return convertedDictionary

    def addHardwareAndOSValues(self, resultsDocument, systemData):
        for (tag, value) in resultsDocument.getOSValues().items(): # for pre rhcert-6.3 compatability
            systemData[tag.replace('-', '_')] = value
        for (tag, value) in resultsDocument.getHostOSValues().items():
            systemData[tag.replace('-', '_')] = value
        for (tag, value) in resultsDocument.getHardwareValues().items():
            systemData[tag.replace('-', '_')] = value
        systemData["host"] = Host(systemData.get(Tags.hostname), systemData.get(Tags.ip_address.replace('-', '_')))
        return systemData["host"]

    def getHardwareProduct(self, certificationDocument):
        """ returns a dict: {"vendor": <vendor>, "make": <make>, "product": <product name>, "id: <id>} """
        product = certificationDocument.getHardwareValues()
        key = "-".join(product.values())
        product["id"] = key
        return product

    def getWorkflowParameters(self, certification):
        workflowParameters = {"workflowID": certification.getCertificationID(),
                              "workflow": certification.getCertificationState().capitalize()}

        if certification.getCertificationState() != Constants.local:
            workflowParameters["workflowID"] = certification.getBugID()
        if certification.getCertificationState() == Constants.bz:
            workflowParameters["workflow"] = "Bugzilla"
            workflowParameters["workflowID"] = certification.getBugID()
        elif certification.getCertificationState() == Constants.cwe:
            workflowParameters["workflow"] = "Catalog"
            workflowParameters["workflowID"] = certification.getBugID()
            workflowParameters["ci"] = certification.isSandbox()
            if certification.getNodeID():
                workflowParameters["workflowID"] = certification.getNodeID()
        return workflowParameters

    def getClientIP(self):
        if self.request.META.get('HTTP_X_FORWARDED_FOR'):
            return self.request.META.get('HTTP_X_FORWARDED_FOR').split(',')[0]
        return self.request.META.get('REMOTE_ADDR')

    def getHostName(self):
        return socket.gethostname()

    def getCurrentUTCTime(self):
        return time.gmtime(time.time())

    def requestRun(self, resultsDocument):
        self.testHost = Host(resultsDocument.getRequiredOS().getName(), resultsDocument.getRequiredOS().getIPAddress())
        self.testHostURL = 'http://%s:%s' % (self.testHost.getConnection(), self.environment.getServerListenerPort())
        self.server = XmlrpcServerProxy(self.testHostURL, allow_none=True)

        # set the test server to this one
        resultsDocument.setTestServer(self.getHostName())
        if self.environment.isMultiStoreEnabled():
            resultsDocument.setDatabasePath(os.path.join(self.getConnection().getUsername(), self.environment.getPhase()))

        try:
            xml = resultsDocument.toXML()
            self.result = self.server.run(xml)
            if not self.result:
                self.appendError("Error: Run Request Failed.")
                return False
        except Exception as exception:
            self.appendError("Error: (view) call to host %s failed:\n" % (self.testHost.getConnection()))
            self.appendError("%s\n" % exception)
            return False

        return True

    def getHostStatus(self, host, timeout=1):
        client = RHCertClient(host, timeout, self.environment)
        return client.getStatus()

    def htmlEscape(self, text):
        return escape(text, self.htmlEscapeTable)

    def htmlUnescape(self, text):
        return unescape(text, self.htmlUnescapeTable)

    def validateParameters(self, requirements):
        """ requirements are of the form {value:type}"""
        valid = True
        for (name, ptype) in requirements.items():
            if not self.getParameter(name):
                self.appendError("Error: %s not set" % name)
                valid = False
                continue

            if ptype == "int":
                try:
                    int(self.getParameter(name))
                except:
                    self.appendError("Error: %s must be an integer" % name)
                    valid = False
        return valid

    def formatUTCtoLocal(self, utcTStamp):
        fromZone = tz.tzutc()
        toZone = tz.tzlocal()
        utc = datetime.strptime(utcTStamp, Constants.UTCDATETIMEFORMAT)
        utc = utc.replace(tzinfo=fromZone)
        localtime = utc.astimezone(toZone)
        return localtime.strftime(Constants.DATETIMEFORMAT)

    def syncronizeProduct(self, vendorObject, productID, passthroughVendorObject=None):
        """ DO NOT CALL - use Connection.resetVendorProducts() instead """
        vendorID = vendorObject.getID()
        if passthroughVendorObject:
            passthroughVendorID = passthroughVendorObject.getID()
        catalogProduct = self.getVendorProductWorkflow().get(self.program, vendorID, productID)
        self.appendErrors(self.getVendorProductWorkflow().getErrors())
        if self.getErrors():
            return None
        if not catalogProduct:
            self.appendError("Error: could not download new product ID %s" % productID)
            return None

        # For a pass through product, store it in the pass through vendor object
        # (on whose make the pass through spec is created) and not in the
        # vendor object of the base cert
        if passthroughVendorObject:
            product = passthroughVendorObject.getProduct(productID)
        else:
            product = vendorObject.getProduct(productID)
        if not product:
            if passthroughVendorObject and catalogProduct.get("model"):
                # Pass through products do not return a product_name but a model
                # verify that the vendor_id of catalogProduct is same as passthroughVendorObject
                if int(passthroughVendorID) != catalogProduct['vendor_id']:
                    self.appendError("Error: could not download new pass through product with ID %s" % productID)
                    return None
                product = passthroughVendorObject.createProduct(catalogProduct.get("model"), productID)
            elif catalogProduct.get("model"):
                product = vendorObject.createProduct(catalogProduct.get("product_name"), productID)
        product.setMake(catalogProduct.get("make"))
        if catalogProduct.get("bug-id"):
            product.setCertificationID(catalogProduct.get("bug-id"))
        product.setNodeID(catalogProduct.get("product_nid"))
        product.setProgram(catalogProduct.get("program"))
        product.setWorkflow(Constants.cwe)
        product.setBaseProductID(catalogProduct.get("base_product_id"))
        product.setPassThroughProductIDs(catalogProduct.get("passthrough_product_ids"))
        self.getStore().getVendorProducts().save()
        return product

    def compileVendorProducts(self, redhatProductID, workflow=None):
        # get a list of allowed vendor product programs based on the RH certification:
        allowedPrograms = self.productNameService.getAllowedPrograms(redhatProductID)
        if allowedPrograms: # unpack
            self.data["allowedProgramsDescription"] = allowedPrograms.get("description")
            allowedPrograms = allowedPrograms.get("programs")
            self.data["allowedPrograms"] = allowedPrograms
        else:
            allowedPrograms = None

        noMakeName = "(no make name)"
        vendorProductsData = []
        for vendor in self.getConnection().getVendorProducts().getVendors():
            if workflow and (vendor.getWorkflow() != workflow):
                continue
            vendorProductsData.append({"name": vendor.getName(), "id": vendor.getID(), "makes": []})
            for product in vendor.getProducts():
                if allowedPrograms and (product.getProgram() not in allowedPrograms):
                    continue

                makeName = product.getMake() if product.getMake() else noMakeName
                foundMake = None
                for make in vendorProductsData[-1]["makes"]:
                    if make["name"] == makeName:
                        foundMake = make
                        break
                if not foundMake:
                    foundMake = { "name": makeName, "products": [], "makeName": makeName}
                    if makeName == noMakeName:
                        vendorProductsData[-1]["makes"].insert(0, foundMake)
                    else:
                        vendorProductsData[-1]["makes"].append(foundMake)

                _program = product.getProgram()
                if _program == Programs.openstack and redhatProductID == '1':
                    _program = Programs.software
                elif _program == Programs.cloud and redhatProductID == '31':
                    _program = Programs.cloud_sap
                foundMake["products"].append({"name": product.getName(), "program": _program, "id": product.getID(), "nid": product.getNodeID()})

        self.data["vendorProductsData"] = json.dumps(vendorProductsData)

    def debugSelectedTests(self, selectedTests):
        selectedParams = [params for path, params in selectedTests]
        for params in selectedParams:
            for (name, value) in params.items():
                if name == Constants.debug:
                    return value
        return None

    def selectTests(self, selectedTests, resultsDocument):
        print("selecting tests...")
        debug = self.debugSelectedTests(selectedTests)

        # selectedTests is a list of tuple - (path, params) for tests selected to run
        # Here, params is a dictionary of parameters of the form {'paramater_name':'parameter_value'}
        # We have ignore checking the value of debug parameter of selcted test with tests in resultsDocument
        # So we are deleting the "debug" parameter if any, from each selected test's params

        for (path, param) in selectedTests:
            if Constants.debug in param:
                del param[Constants.debug]

        for testDocument in resultsDocument.getTests():
            path = testDocument.getPath()
            params = testDocument.getTestParameters().getAll()

            if (path, params) not in selectedTests:
                testDocument.delete()
            else:
                testDocument.deleteRuns()
                testDocument.setDebugging(debug)
                testDocument.getTestParameters().set(Constants.debug, debug)

    def parseTestOption(self, option):
        # test options are "<test path> [--<parameter> <value>]*"
        print("parsing test option %s" % option)
        if not option:
            return (None, None)
        options = option.split("--")
        if not options:
            return (None, None)

        testPath = options[0].strip()
        parameters = dict()
        optionNumber = 1
        while optionNumber < len(options):
            tokens = options[optionNumber].strip().split(" ")
            if len(tokens) == 2:
                parameters[tokens[0].strip()] = tokens[1].strip()
            optionNumber += 1
        if self.getParameter(Constants.debug):
            parameters[Constants.debug] = self.getParameter(Constants.debug)
            self.debug = self.getParameter(Constants.debug)
        return (testPath, parameters)
